SET QUOTED_IDENTIFIER ON
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[GenerateEventsAnnouncementCreated]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure GenerateEventsAnnouncementCreated'
		DROP PROCEDURE [dbo].[GenerateEventsAnnouncementCreated]
	END
GO

PRINT 'Creating procedure GenerateEventsAnnouncementCreated'
GO

/******************************************************************************
**		File: GenerateEventsAnnouncementCreated.sql
**		Name: GenerateEventsAnnouncementCreated
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Dmitry.Kolchev
**		Date: 10/12/2006
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [GenerateEventsAnnouncementCreated]
	@FromID TIdentifier,
	@EventType TName,
	@DocumentClass TClass,
	@DocumentID TIdentifier,
	@Data XML
AS
	SET NOCOUNT ON;

	INSERT INTO [MailItem]
		([State], [FileAs], [EventType], [FromID], [ToID], [DocumentClass], [DocumentID], [Read], [Data], [Comments])
	SELECT
		1,
		[Announcement].[FileAs],
		@EventType,
		@FromID,
		[User].[ID],
		@DocumentClass,
		@DocumentID,
		0,
		@Data,
		[Announcement].[Comments]
	FROM
		[Announcement] CROSS JOIN [User]
	WHERE
		[User].[Locked] = 0
	AND
		[Announcement].[ID] = @DocumentID 
	AND
		[User].[ID] != @FromID
	AND
		(
			[Announcement].[ProjectID] IS NULL 
		OR
			([Announcement].[ProjectID] IS NOT NULL AND [User].[ID] IN (SELECT [Employee].[UserID] 
				FROM [vwAssignment] INNER JOIN [Employee] 
					ON ([vwAssignment].[EmployeeID]=[Employee].[ID] AND 
						[Employee].[UserID] IS NOT NULL AND 
						[vwAssignment].[ProjectID] = [Announcement].[ProjectID] AND
						[vwAssignment].[State] = 1))
			)
		)
	RETURN 0
GO

IF EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[pjm5trAnnouncementInsert]'))
BEGIN
	PRINT 'Dropping trigger pjm5trAnnouncementInsert'
	DROP TRIGGER [dbo].[pjm5trAnnouncementInsert]
END
GO

PRINT 'Creating trigger pjm5trAnnouncementInsert'
GO

CREATE TRIGGER [dbo].[pjm5trAnnouncementInsert] ON [dbo].[Announcement] FOR INSERT
AS
    IF @@rowcount = 0
       RETURN

    SET NOCOUNT ON;
    
	DECLARE @ID INT;
	SELECT @ID = [ID] FROM [inserted];
	BEGIN TRY
		EXEC pjm5PostDocumentChangeNotificationEvent 'AnnouncementCreated', 'Adf.Announcement', @ID, NULL
	END TRY
	BEGIN CATCH
	END CATCH
	
	RETURN
	
GO