﻿--//////////////Client scripts////////////////////
--**************CategoryBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[CategoryBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[CategoryBrowse]'
      DROP PROCEDURE [pjmams].[CategoryBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[CategoryBrowse]'
GO
       

create procedure [pjmams].[CategoryBrowse] @CurrentUserName nvarchar(128)
as
begin
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END

	
	select
		[Category].[ID],
		[Category].[Name]
	from
		[pjmams].[Category];

	return 0;
end
go

grant execute on [pjmams].[CategoryBrowse] to [activitymonitorsite];
go


--**************CategoryUpdate**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[CategoryUpdate]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[CategoryUpdate]'
      DROP PROCEDURE [pjmams].[CategoryUpdate]
END
GO

PRINT 'Creating procedure [pjmams].[CategoryUpdate]'
GO
       

create procedure [pjmams].[CategoryUpdate] @ModuleName nvarchar(max), @CategoryID int, @CurrentUserName nvarchar(128)
as
	set nocount on;

	DECLARE
		@HasManageActivityMonitor int

	SET @HasManageActivityMonitor = [pjmams].fnHasAccessRightByNameForUserLogin('ManageActivityMonitor', @CurrentUserName)

	IF(@HasManageActivityMonitor = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END
	
	update [pjmams].[Module]
	set 
		[CategoryID] = @CategoryID
	where
		[Module].[Description] = @ModuleName;

	return 0;
go

grant execute on [pjmams].[CategoryUpdate] to [activitymonitorsite];
go

--**************EmployeeList**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[EmployeeList]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[EmployeeList]'
      DROP PROCEDURE [pjmams].[EmployeeList]
END
GO

PRINT 'Creating procedure [pjmams].[EmployeeList]'
GO
       

create procedure [pjmams].[EmployeeList] @CurrentUserName nvarchar(128), @EmployeeSelector	bit, 
	@IncludeCurrentEmployeeAdditionalRow bit
as
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 1 OR @HasViewAllActivity = 1)
	BEGIN
		IF(@EmployeeSelector = 0)
		BEGIN
			SELECT
				[EmployeeView].[EmployeeID],
				[EmployeeView].[EmployeeName]
			FROM
				[pjmams].[EmployeeView]
				INNER JOIN [dbo].[Employee]
					ON [Employee].[ID] = [EmployeeView].[EmployeeID]
				INNER JOIN [dbo].[User]
					ON [User].[ID] = [Employee].[UserID]
			WHERE
				[Employee].[State] = 1
				AND [User].[Locked] = 0
				AND
				@HasViewAllActivity = 1
				OR
				(	
					@HasViewOwnActivity = 1 
					AND 
					[EmployeeView].[WindowsLogin] = @CurrentUserName
				)
			UNION
			SELECT
				[EmployeeView].[EmployeeID],
				'(текущий пользователь)'
			FROM
				[pjmams].[EmployeeView]
				INNER JOIN [dbo].[Employee]
					ON [Employee].[ID] = [EmployeeView].[EmployeeID]
				INNER JOIN [dbo].[User]
					ON [User].[ID] = [Employee].[UserID]
			WHERE
				[Employee].[State] = 1
				AND 
				[User].[Locked] = 0
				AND
				@IncludeCurrentEmployeeAdditionalRow = 1
				AND				
				[EmployeeView].[WindowsLogin] = @CurrentUserName				
		END
		ELSE
		BEGIN
			SELECT
				[EmployeeView].[EmployeeID],
				[EmployeeView].[EmployeeName]
			FROM
				[pjmams].[EmployeeView]
			WHERE
				@HasViewAllActivity = 1
				OR
				(	
					@HasViewOwnActivity = 1 
					AND 
					[EmployeeView].[WindowsLogin] = @CurrentUserName
				)
			UNION
			SELECT
				[EmployeeView].[EmployeeID],
				'(текущий пользователь)'
			FROM
				[pjmams].[EmployeeView]
			WHERE				
				@IncludeCurrentEmployeeAdditionalRow = 1				
				AND
				[EmployeeView].[WindowsLogin] = @CurrentUserName	

		END
	END
	ELSE
	BEGIN
		SELECT
			-1 AS [EmployeeID],
			'(Нет данных)' AS [EmployeeName]
	END
	

	
	
		

	
	return 0;
go

grant execute on [pjmams].[EmployeeList] to [activitymonitorsite];
go
  

--**************ModuleBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ModuleBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ModuleBrowse]'
      DROP PROCEDURE [pjmams].[ModuleBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[ModuleBrowse]'
GO
       

create procedure [pjmams].[ModuleBrowse] @CategoryID int
as
begin
	set nocount on;

	with [M] ([Module], [CategoryID], [Category]) as
	(
		select distinct
			[Module].[Description] as [Module],
			[Module].[CategoryID],
			[Category].[Name]
		from
			[pjmams].[Module] inner join [pjmams].[Category]
				ON ([Module].[CategoryID] = [Category].[ID])
		where
			[Module].[Active] = 1
		AND
			[Module].[Description] is not null and [Module].[Description] != ''
	)
	select distinct
		row_number() over (order by [Module] asc) as [Row],
		[Module],
		[CategoryID],
		[Category]
	from
		[M]
	where
		((@CategoryID = 0) or ([CategoryID] = @CategoryID))
	order by
		[Module]
	return 0;
end
go

grant execute on [pjmams].[ModuleBrowse] to [activitymonitorsite];
go

--**************ModuleDelete**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ModuleDelete]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ModuleDelete]'
      DROP PROCEDURE [pjmams].[ModuleDelete]
END
GO

PRINT 'Creating procedure [pjmams].[ModuleDelete]'
GO
       

create procedure [pjmams].[ModuleDelete] @Description nvarchar(max), @CurrentUserName nvarchar(128)
as
begin
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END
	
	update [pjmams].[Module]
	set
		[Active] = 0
	where
		[Description] = @Description
		
	return 0;
end
go

grant execute on [pjmams].[ModuleDelete] to [activitymonitorsite];
go


--**************ModuleGet**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ModuleGet]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ModuleGet]'
      DROP PROCEDURE [pjmams].[ModuleGet]
END
GO

PRINT 'Creating procedure [pjmams].[ModuleGet]'
GO
       

create procedure [pjmams].[ModuleGet] 
	@Description nvarchar(max),
	@CategoryID int,
	@CurrentUserName nvarchar(128)
as
begin
	set nocount on;

	DECLARE
		@HasManageActivityMonitor int

	SET @HasManageActivityMonitor = [pjmams].fnHasAccessRightByNameForUserLogin('ManageActivityMonitor', @CurrentUserName)

	IF(@HasManageActivityMonitor = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END
	
	select top 1
		[Module].[Description] as [Module],
		[Category].[Name] as [Category]
	from
		[pjmams].[Module] inner join [pjmams].[Category]
			ON ([Module].[CategoryID] = [Category].[ID])
	where
		[Module].[Description] = @Description
	and
		[Module].[CategoryID] = @CategoryID
	
	select distinct
		[Session].[MachineName],
		[Module].[ID],
		[Module].[CategoryID],
		[Module].[Name],
		[Module].[Version],
		[Module].[Lang]
	from 
		[pjmams].[Module] left outer join [pjmams].[Process]
			on ([Module].[ID] = [Process].[ModuleID])
		left outer join [pjmams].[Session]
			on ([Process].[SessionID] = [Session].[ID])
	where
		[Description] = @Description
	and
		[CategoryID] = @CategoryID
	order by
		[Session].[MachineName],
		[Module].[Version] DESC;
		
	return 0;
end

go

grant execute on [pjmams].[ModuleGet] to [activitymonitorsite];
go

--**************ProgramActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ProgramActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ProgramActivityBrowse]'
      DROP PROCEDURE [pjmams].[ProgramActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[ProgramActivityBrowse]'
GO
       

create procedure [pjmams].[ProgramActivityBrowse] @Date date, @Module nvarchar(max), @CurrentUserName nvarchar(128)
as
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END
	
	declare @Total decimal(18, 2), @Minute decimal(18,4), @Hour decimal(18,4);

	set @Minute = 60;
	set @Hour = 3600;

	with [A] ([Total]) as
	(
		select
			cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18,2))
		FROM 
			[pjmams].[Session] INNER JOIN [pjmams].[Process] 
				ON ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				ON [Process].[ModuleID] = [Module].[ID]
		WHERE
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		and
			isnull([Module].[Description], [Module].[Name]) = @Module
		and
			[Process].[UserName] != '\'
		AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			)
		group by
			[Process].[UserName],
			[Session].[MachineName]
	)
	select
		@Total = max([Total])
	from
		[A];
		 
	select
		@Module as [Module],
		@Date as [Date];

	with [A] ([EmployeeName], [UserName], [MachineName], [CategoryID], [Total], [Idle]) as
	(
		select
			isnull([EmployeeView].[EmployeeName], [Process].[UserName]),
			[Process].[UserName],
			[Session].[MachineName],
			min([Module].[CategoryID]),
			sum(datediff(second, [Created], [Closed]) - [Idle]),
			sum([Idle])
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				on ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				on ([Process].[ModuleID] = [Module].[ID])
			left outer join [pjmams].[EmployeeView]
				on ([Process].[UserName] = [EmployeeView].[WindowsLogin])
		where
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		and
			isnull([Module].[Description], [Module].[Name]) = @Module
		and
			[Process].[UserName] != '\'
		AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			)
		group by
			isnull([EmployeeView].[EmployeeName], [Process].[UserName]),
			[Process].[UserName],
			[Session].[MachineName]
	)
	select
		row_number() over (order by [Total] desc) as [Row],
		[EmployeeName],
		[UserName],
		[MachineName],
		[CategoryID],
		cast([Total] / @Hour as decimal(18,2)) as [Duration],
		cast([Idle] / @Hour as decimal(18,2)) as [Idle],
		cast([Total] * 100.0 / @Minute / @Total as decimal(18,2)) as [RelativeDuration]
	from 
		[A]
	where
		[Total] > 60
	order by 
		 [Total] desc;
	
	return 0;
go

grant execute on [pjmams].[ProgramActivityBrowse] to [activitymonitorsite];
go

--**************ProgramsActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ProgramsActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ProgramsActivityBrowse]'
      DROP PROCEDURE [pjmams].[ProgramsActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[ProgramsActivityBrowse]'
GO
       

create procedure [pjmams].[ProgramsActivityBrowse] @Date date, @CurrentUserName nvarchar(128)
as
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END
	
	declare @Total decimal(18, 2), @Minute decimal(18,4), @Hour decimal(18,4);

	set @Minute = 60;
	set @Hour = 3600;

	select @Date as [Date];
		
	with [A] ([Total]) as
	(
		select
			cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18,2))
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				on ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				on ([Process].[ModuleID] = [Module].[ID])
		where
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		and
			[Process].[UserName] != '\'
		AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			)
		group by
			isnull([Module].[Description], [Module].[Name])
	)
	select
		@Total = max([Total])
	from
		[A];

	with [A] ([Module], [CategoryID], [Total], [Idle]) as
	(
		select
			isnull([Module].[Description], [Module].[Name]),
			[Module].[CategoryID],
			sum(datediff(second, [Created], [Closed]) - [Idle]),
			sum([Idle])
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				on ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				on [Process].[ModuleID] = [Module].[ID]
		where
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		and
			[Process].[UserName] != '\'
		AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			)
		group by
			isnull([Module].[Description], [Module].[Name]),
			[Module].[CategoryID]
	)
	select
		row_number() over (order by [Total] desc) as [Row],
		[Module],
		[CategoryID],
		cast([Total] * 1.0 / @Hour as decimal(18,2)) as [Duration],
		cast([Idle] * 1.0  / @Hour as decimal(18,2)) as [Idle],
		cast([Total] * 100.0 / @Minute / @Total as decimal(18,2)) as [RelativeDuration]
	from 
		[A]
	where
		[Total] > 60
	order by 
		 [Total] desc;
	
	return 0;
go

grant execute on [pjmams].[ProgramsActivityBrowse] to [activitymonitorsite]
go

    

--**************ReportApplicationTimeAllocationByEmployee**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ReportApplicationTimeAllocationByEmployee]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ReportApplicationTimeAllocationByEmployee]'
      DROP PROCEDURE [pjmams].[ReportApplicationTimeAllocationByEmployee]
END
GO

PRINT 'Creating procedure [pjmams].[ReportApplicationTimeAllocationByEmployee]'
GO
       

-- Все данные о времени процедура возвращает в секундах

CREATE PROCEDURE [pjmams].[ReportApplicationTimeAllocationByEmployee]
		@EmployeeID     TIdentifier,
		@CountSelector	int,
		@DateSelector	int,
        @StartDate		TDateTime,
        @EndDate		TDateTime,
		@CurrentUserName nvarchar(128)
AS

SET NOCOUNT ON


DECLARE
	@HasViewApplicationTimeAllocationByEmployeeReport int

SET @HasViewApplicationTimeAllocationByEmployeeReport = [pjmams].fnHasAccessRightByNameForUserLogin('ViewApplicationTimeAllocationByEmployeeReport', @CurrentUserName)	

IF(@HasViewApplicationTimeAllocationByEmployeeReport = 0)
BEGIN
	RAISERROR('#AccessDenied', 18, -1)
	RETURN 1
END


DECLARE 
	@UserID			TIdentifier,
	@UserName		nvarchar(128),
    @EmployeeFileAs	TName,
    @Date			TDate,
    @WorkingDays	decimal(18,2),
	@NormHours		decimal(18,2)

SELECT
    @EmployeeFileAs = [Employee].[FileAs]
FROM
	[Employee]
WHERE 
	[Employee].[ID] = @EmployeeID

SET @UserName = (SELECT
					[WindowsLogin]
				FROM
					[pjmams].[EmployeeView]
				WHERE 
					 [EmployeeID] = @EmployeeID)

SET @NormHours = CAST((SELECT [Value] FROM [dbo].[F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53]
					WHERE [Name] = 'HoursRate' AND [Group] = 'ActivityMonitorClientSettings') as decimal(18,2)) / 1000

-- Выбираются StartDate и EndDate в зависимости от выбранного периода
SET @Date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS DATETIME)
IF @DateSelector = 1 -- current month
	SELECT
		@StartDate = DATEADD(day, 1-DATEPART(day, @Date), @Date),
		@EndDate = DATEADD(day, 1, @Date)
ELSE IF @DateSelector = 2 -- prev month
	SELECT
		@StartDate = DATEADD(day, 1-DATEPART(day, @Date), @Date),
		@StartDate = DATEADD(month, -1, @StartDate),
		@EndDate = DATEADD(month, 1, @StartDate)
ELSE IF @DateSelector = 3 -- quart
	SELECT
		@StartDate = [StartDate],
		@EndDate = DATEADD(day, 1, @Date)
	FROM	
		dbo.adfGetDateInterval(4, @Date, @Date, @Date)
ELSE IF @DateSelector = 4 -- prev quart
	SELECT
		@StartDate = [StartDate],
		@EndDate = [EndDate]
	FROM	
		dbo.adfGetDateInterval(3, @Date, @Date, @Date)
ELSE IF @DateSelector = 5 -- curr year
	SELECT
		@StartDate = CAST(Year(@Date) as varchar(4)),
		@EndDate = DATEADD(day, 1, @Date)
ELSE IF @DateSelector = 6 -- prev year
	SELECT
		@EndDate = CAST(Year(@Date) as varchar(4)),
		@StartDate = CAST(Year(@Date)-1 as varchar(4))
ELSE
	SELECT
		@StartDate = CAST(CONVERT(varchar(8), @StartDate, 112) AS datetime),
		@EndDate = DATEADD(day, 1, CAST(CONVERT(varchar(8), @EndDate, 112) AS datetime))

-- Рабочие дни
SET DATEFIRST 7
SET @WorkingDays = dbo.fnPjm5GetWorkingDays(@StartDate, DATEADD(day, -1, @EndDate))
SET DATEFIRST 1

-- Шапка таблицы
SELECT
       @EmployeeFileAs AS [EmployeeFileAS],
       @CountSelector AS [CountSelector],
       @WorkingDays AS [WorkingDays],
       @WorkingDays*8 AS [WorkingHours],
       @DateSelector AS [DateSelector],
       @StartDate AS [StartDate],
       DATEADD(day, -1, @EndDate) as [EndDate],
       @UserName AS [UserName],
	   @NormHours AS [NormHours]



-- Основная таблица для вывода результата
DECLARE @WorkingApplications TABLE
(
	CategoryID			TIdentifier,
	ApplicationName		TName,
	Days				int,
	Hours				decimal(18,2),
	RemoteWorkHours		decimal(18,2),
	Ordering			int,
	CategoryOrdering	int
) 

INSERT INTO @WorkingApplications
SELECT	DISTINCT
	[Module].[CategoryID], 
	[Module].[Description],
	NULL,
	NULL,
	NULL,
	0,
	CASE
		WHEN [Module].[CategoryID] = 2
		THEN 1
		WHEN [Module].[CategoryID] = 3
		THEN 2
		ELSE 3
	END
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]	
WHERE 
	[UserName] = @UserName
	AND	[Created] >= @StartDate 
	AND [Closed] <= @EndDate   	
	AND [Process].[Created] < [Process].[Closed]



--**************************Дни***********************
DECLARE @t TABLE
(
	ApplicationName		TName,
	Created				DATE
)

INSERT INTO @t
SELECT
	[Module].[Description],
	CAST([Created] AS DATE)
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
WHERE	
	[UserName] = @UserName
	AND	[Created] >= @StartDate 
	AND [Closed] <= @EndDate
	AND [Process].[Created] < [Process].[Closed]
GROUP BY
	CAST([Created] AS DATE),
	[Module].[Description]

DECLARE @t1 TABLE
(
	ApplicationName	TName,
	Days			INT
)

DECLARE
	@tmp_name	TName

DECLARE daysCursor CURSOR LOCAL FAST_FORWARD FOR
SELECT DISTINCT
	[@t].[ApplicationName]
FROM
	@t

OPEN daysCursor
FETCH NEXT FROM daysCursor INTO @tmp_name
WHILE @@FETCH_STATUS = 0
BEGIN
		
	INSERT INTO @t1
	SELECT
		@tmp_name,
		COUNT(*)
	FROM 
		@t
	WHERE 
		[ApplicationName] = @tmp_name

	FETCH NEXT FROM daysCursor INTO @tmp_name
									
END
CLOSE daysCursor
DEALLOCATE daysCursor




--**************************Часы***********************
DECLARE @t2 TABLE
(
	ApplicationName		TName,
	Hours				decimal(18,2)
)

INSERT INTO @t2
SELECT
	[Module].[Description],
	CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2))
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
WHERE	
	[UserName] = @UserName
	AND	[Created] >= @StartDate 
	AND [Closed] < @EndDate
	AND [Process].[Created] < [Process].[Closed]
GROUP BY
	[Module].[Description]




--**************************В том числе удаленно***********************
DECLARE @t3 TABLE
(
	ApplicationName		TName,
	RemoteWorkHours		decimal(18,2)
)

INSERT INTO @t3
SELECT
	[Module].[Description],
	CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2)) AS [Hours]
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
	INNER JOIN [pjmams].[Session]
		ON [Process].[SessionID] = [Session].[ID]
WHERE	
	[Process].[UserName] = @UserName
	AND	[Process].[Created] >= @StartDate 
	AND [Process].[Closed] <= @EndDate
	AND [Session].[RemoteSession] = 1
	AND [Process].[Created] < [Process].[Closed]
GROUP BY
	[Module].[Description]



-- Курсор для объединения данных в основной таблицы
DECLARE 
	@tmp_appName TName,
	@tmp_days int,
	@tmp_hours decimal(18,2),
	@tmp_hoursRemote decimal(18,2)

DECLARE insertCursor CURSOR LOCAL FAST_FORWARD FOR
SELECT
	[@t1].[ApplicationName],
	[@t1].[Days],
	[@t2].[Hours],
	[@t3].[RemoteWorkHours]
FROM
	@t1
	LEFT JOIN @t2
		ON [@t1].[ApplicationName] = [@t2].[ApplicationName]
	LEFT JOIN @t3
		ON [@t1].[ApplicationName] = [@t3].[ApplicationName]

OPEN insertCursor
FETCH NEXT FROM insertCursor INTO @tmp_appName, @tmp_days, @tmp_hours, @tmp_hoursRemote
WHILE @@FETCH_STATUS = 0
BEGIN
	
	UPDATE
		@WorkingApplications
	SET
		[Days] = @tmp_days,
		[Hours] = @tmp_hours,
		[RemoteWorkHours] = @tmp_hoursRemote
	WHERE
		[@WorkingApplications].[ApplicationName] = @tmp_appName


	FETCH NEXT FROM insertCursor INTO @tmp_appName, @tmp_days, @tmp_hours, @tmp_hoursRemote
END
CLOSE insertCursor
DEALLOCATE insertCursor




--*****************Дополнительная таблица для строк "Итого"**************
DECLARE @t4 TABLE
(
	CategoryID			TIdentifier,
	ApplicationName		TName,
	Days				int,
	Hours				decimal(18,2),
	RemoteWorkHours		decimal(18,2),
	Ordering			int,
	CategoryOrdering	int
)

DECLARE @DaysCount int
DECLARE @TempDays TABLE
(
	Created		DATE
)

--Количество дней по неуказанным приложениям
INSERT INTO @TempDays
SELECT 
	CAST([Created] AS DATE) 
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Module].[ID] = [ModuleID]
WHERE
	[UserName] = @UserName
	AND [Created] >= @StartDate
	AND [Closed] < @EndDate
	AND [Process].[Created] < [Process].[Closed]
	AND [CategoryID] = 1
GROUP BY 
	CAST([pjmams].[Process].[Created] AS DATE)

set @DaysCount = @@ROWCOUNT

IF(@DaysCount > 0)
BEGIN
INSERT INTO @t4
SELECT
	1 as [CategoryID],
	'Итого' as [ApplicationName],
	@DaysCount as [Days],
	SUM([Hours]) as [Hours],
	Sum([RemoteWorkHours]) as [RemoteWorkHours],
	2 as [Ordering],
	3 as [CategoryOrdering]
from 
	@WorkingApplications
where
	[CategoryID] = 1
END



-- Количество дней по рабочим приложениям
INSERT INTO @TempDays
SELECT 
	CAST([Created] AS DATE) 
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Module].[ID] = [ModuleID]
WHERE
	[UserName] = @UserName
	AND [Created] >= @StartDate
	AND [Closed] < @EndDate
	AND [Process].[Created] < [Process].[Closed]
	AND [CategoryID] = 2
GROUP BY 
	CAST([Created] AS DATE)

set @DaysCount = @@ROWCOUNT

IF(@DaysCount > 0)
BEGIN
INSERT INTO @t4
SELECT
	2 as [CategoryID],
	'Итого' as [ApplicationName],
	@DaysCount as [Days],
	SUM([Hours]) as [Hours],
	Sum([RemoteWorkHours]) as [RemoteWorkHours],
	2 as [Ordering],
	1 as [CategoryOrdering]
from 
	@WorkingApplications
where
	[CategoryID] = 2
END



--Количество дней по прочим приложениям
INSERT INTO @TempDays
SELECT 
	CAST([Created] AS DATE) 
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Module].[ID] = [ModuleID]
WHERE
	[UserName] = @UserName
	AND [Created] >= @StartDate
	AND [Closed] < @EndDate
	AND [Process].[Created] < [Process].[Closed]
	AND [CategoryID] = 3
GROUP BY 
	CAST([Created] AS DATE)

set @DaysCount = @@ROWCOUNT

IF(@DaysCount > 0)
BEGIN
INSERT INTO @t4
SELECT
	3 as [CategoryID],
	'Итого' as [ApplicationName],
	@DaysCount as [Days],
	SUM([Hours]) as [Hours],
	Sum([RemoteWorkHours]) as [RemoteWorkHours],
	2 as [Ordering],
	2 as [CategoryOrdering]
from 
	@WorkingApplications
where
	[CategoryID] = 3
END



--***********Общая выборка таблицы со строками "Итого"**************
DECLARE @TableForTop TABLE
(
	CategoryID			TIdentifier,
	ApplicationName		TName,
	Days				int,
	Hours				decimal(18,2),
	RemoteWorkHours		decimal(18,2),
	Ordering			int,
	CategoryOrdering	int
)

DECLARE @SumHours decimal(18,2), @SumRemoteHours decimal(18,2)

IF (@CountSelector = 0)
BEGIN
	SELECT * FROM @WorkingApplications
		WHERE 
			[Hours] > 0
	UNION
	SELECT * FROM @t4
		WHERE
			[Hours] > 0
	ORDER BY [CategoryOrdering]
END
ELSE
BEGIN
	INSERT INTO @TableForTop
	SELECT TOP 5 * FROM @WorkingApplications
		WHERE 
			([Hours] > 0) AND ([CategoryID] = 1)
		ORDER BY [Hours] DESC

	SET @SumHours = (SELECT SUM([Hours]) FROM @TableForTop WHERE [CategoryID] = 1) 
	SET @SumRemoteHours = (SELECT SUM([RemoteWorkHours]) FROM @TableForTop WHERE [CategoryID] = 1)
	
	INSERT INTO @TableForTop
	SELECT
		1 AS [CategoryID],
		'Другие' AS [ApplicationName],
		NULL AS [Days],
		SUM([@WorkingApplications].[Hours]) - @SumHours AS [Hours],
		SUM([@WorkingApplications].[RemoteWorkHours]) - @SumRemoteHours AS [RemoteWorkHours],
		1 AS [Ordering],
		3 AS [CategoryOrdering]
	FROM
		@WorkingApplications
	WHERE
		[CategoryID] = 1

	INSERT INTO @TableForTop
	SELECT TOP 5 * FROM @WorkingApplications	
		WHERE 
			([Hours] > 0) AND ([CategoryID] = 2)
		ORDER BY [Hours] DESC

	SET @SumHours = (SELECT SUM([Hours]) FROM @TableForTop WHERE [CategoryID] = 2) 
	SET @SumRemoteHours = (SELECT SUM([RemoteWorkHours]) FROM @TableForTop WHERE [CategoryID] = 2)

	INSERT INTO @TableForTop
	SELECT
		2 AS [CategoryID],
		'Другие' AS [ApplicationName],
		NULL AS [Days],
		SUM([@WorkingApplications].[Hours]) - @SumHours AS [Hours], 
		SUM([@WorkingApplications].[RemoteWorkHours]) - @SumRemoteHours AS [RemoteWorkHours],
		1 AS [Ordering],
		1 AS [CategoryOrdering]
	FROM
		@WorkingApplications
	WHERE
		[@WorkingApplications].[CategoryID] = 2

	INSERT INTO @TableForTop
	SELECT TOP 5 * FROM @WorkingApplications	
		WHERE 
			([Hours] > 0) AND ([CategoryID] = 3)
		ORDER BY [Hours] DESC

	SET @SumHours = (SELECT SUM([Hours]) FROM @TableForTop WHERE [CategoryID] = 3) 
	SET @SumRemoteHours = (SELECT SUM([RemoteWorkHours]) FROM @TableForTop WHERE [CategoryID] = 3)

	INSERT INTO @TableForTop
	SELECT
		3 AS [CategoryID],
		'Другие' AS [ApplicationName],
		NULL AS [Days],
		SUM([@WorkingApplications].[Hours]) - @SumHours AS [Hours],
		SUM([@WorkingApplications].[RemoteWorkHours]) - @SumRemoteHours AS [RemoteWorkHours],
		1 AS [Ordering],
		2 AS [CategoryOrdering]
	FROM
		@WorkingApplications
	WHERE
		[@WorkingApplications].[CategoryID] = 3

	SELECT * FROM @TableForTop
	WHERE
		[Hours] > 0
	UNION
	SELECT * FROM @t4
	WHERE
		[Hours] > 0
	ORDER BY [CategoryOrdering]
END





--Простой
DECLARE @AllHours	decimal(18,2)
SET @AllHours = (SELECT 
					CAST(SUM([Idle]) as decimal(18,2))
				FROM
					[pjmams].[Process]
					INNER JOIN [pjmams].[Session]
						ON [Session].[ID] = [Process].[SessionID]
				WHERE
					[pjmams].[Process].[UserName] = @UserName
					AND	[Created] >= @StartDate 
					AND [Closed] < @EndDate 
					AND [Process].[Created] < [Process].[Closed])

DECLARE @RemoteHours	decimal(18,2)
SET @RemoteHours =  (SELECT 
					CAST(SUM([Idle]) as decimal(18,2))
					FROM
						[pjmams].[Process]
						INNER JOIN [pjmams].[Session]
							ON [Session].[ID] = [Process].[SessionID]
					WHERE
						[pjmams].[Process].[UserName] = @UserName
						AND	[Created] >= @StartDate 
						AND [Closed] < @EndDate
						AND [RemoteSession] = 1
						AND [Process].[Created] < [Process].[Closed])

DECLARE @Idle TABLE
(
	Hours		decimal(18,2),
	RemoteHours	decimal(18,2)
)

INSERT INTO	@Idle
SELECT
	@AllHours,
	@RemoteHours

SELECT * FROM @Idle




--Строка "Всего"
DECLARE @Total TABLE
(
	Days				int,
	Hours				decimal(18, 2),
	RemoteWorkHours		decimal(18, 2)
)
 
 DECLARE @Temp TABLE
 (
	SessionStart	DATE
 )

 INSERT INTO @Temp
SELECT 
	CAST([Created] AS DATE) 
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Module].[CategoryID] = 3
WHERE
	[UserName] = @UserName
	AND [Created] >= @StartDate
	AND [Closed] < @EndDate
	AND [Process].[Created] < [Process].[Closed]
GROUP BY 
	CAST([Created] AS DATE)

SET @DaysCount = @@ROWCOUNT

INSERT INTO @Total
	SELECT
		@DaysCount,
		SUM([@WorkingApplications].[Hours]) + [@Idle].[Hours] as [Hours],
		SUM([RemoteWorkHours]) + [@Idle].[Hours] as [RemoteWorkHours]
	FROM
		@WorkingApplications,
		@Idle
	GROUP BY
		[@Idle].[Hours]

SELECT * FROM @Total




-- Среднедневная активность и отклонение от норматива
DECLARE	@WorkingApplicationsHoursSum decimal(18,2), @Percent decimal(18,2)
SET @WorkingApplicationsHoursSum = (SELECT SUM([Hours]) FROM @WorkingApplications WHERE [CategoryID] = 2)

DECLARE @AverageActivity TABLE
(
	Hours	decimal(18,2),
	Value	decimal(18,2)
)

DECLARE @AverageHours decimal(18,2)
SET @AverageHours = @WorkingApplicationsHoursSum / (SELECT [Days] FROM @Total)

INSERT INTO @AverageActivity
VALUES
	(
		@AverageHours,
		(@NormHours - @AverageHours) / @NormHours
	)

SELECT * FROM @AverageActivity




-- Вычисление процента количества часов по определенной категории приложения от общей суммы часов
DECLARE @UsingApplications TABLE
(
	CategoryName	TName,
	Value			decimal(18,2)
)

DECLARE @HoursSum decimal(18,2)
SET @HoursSum = (SELECT [Hours] FROM @Total) 

INSERT INTO @UsingApplications
SELECT
		[pjmams].[Category].[Name] as [CategoryName],
		SUM([Hours]) / @HoursSum as [Value]
FROM 
		[pjmams].[Category]
		INNER JOIN @WorkingApplications
			ON [Category].[ID] = [@WorkingApplications].[CategoryID]
GROUP BY
	[pjmams].[Category].[Name]
UNION
	SELECT
		'Простой' as [CategoryName],
		CAST(SUM([Idle]) as decimal(18,2)) / @HoursSum AS [Value]
	FROM
		[pjmams].[Process]
	INNER JOIN [pjmams].[Session]
		ON [Session].[ID] = [Process].[SessionID]
WHERE	
	[Process].[UserName] = @UserName
	AND	[Created] >= @StartDate 
	AND [Closed] <= @EndDate
	AND [Process].[Created] < [Process].[Closed]	

SELECT * 
	FROM @UsingApplications
WHERE 
	[Value] > 0




-- Топ 5 активных приложений (процентов часов от общего времени)
DECLARE @ActiveApplications TABLE
(
	ApplicationName	TName,
	Value			decimal(18,2)
)

INSERT INTO	@ActiveApplications
SELECT TOP 5
	[ApplicationName],
	[Hours] / @HoursSum
FROM
	@WorkingApplications
WHERE
	[ApplicationName] <> 'Всего'
ORDER BY
	[Hours] DESC

SELECT * FROM @ActiveApplications

RETURN 0
GO

GRANT EXECUTE ON [pjmams].[ReportApplicationTimeAllocationByEmployee] TO [public];
GO


--**************ReportApplicationTimeAllocationByEmployees**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[ReportApplicationTimeAllocationByEmployees]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[ReportApplicationTimeAllocationByEmployees]'
      DROP PROCEDURE [pjmams].[ReportApplicationTimeAllocationByEmployees]
END
GO

PRINT 'Creating procedure [pjmams].[ReportApplicationTimeAllocationByEmployees]'
GO
       

-- Все данные о времени процедура возвращает в секундах

CREATE PROCEDURE [pjmams].[ReportApplicationTimeAllocationByEmployees]
		@EmployeeID       TIdentifier,
        @DateSelector	  int,
        @StartDate        TDateTime,
        @EndDate          TDateTime,  
		@CurrentUserName nvarchar(128)   
AS

SET NOCOUNT ON   

DECLARE
	@HasViewApplicationTimeAllocationByEmployeesReport int

SET @HasViewApplicationTimeAllocationByEmployeesReport = [pjmams].fnHasAccessRightByNameForUserLogin('ViewApplicationTimeAllocationByEmployeesReport', @CurrentUserName)	

IF(@HasViewApplicationTimeAllocationByEmployeesReport = 0)
BEGIN
	RAISERROR('#AccessDenied', 18, -1)
	RETURN 1
END
	   

DECLARE 
		@CompanyName	TName,
		@EmployeeFileAs	TName,
		@Date			TDate,
		@WorkingDays	decimal(18,2),
		@NormHours		decimal(18,2)	

SELECT TOP 1
	@CompanyName = [Company].[FileAs]
FROM
    [Company]
WHERE 
    [Company].[Own] = 1

SET @NormHours = CAST((SELECT [Value] FROM [dbo].[F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53]
					WHERE [Name] = 'HoursRate' AND [Group] = 'ActivityMonitorClientSettings') as decimal(18,2)) / 1000

SELECT
    @EmployeeFileAs = [Employee].[FileAs]
FROM
	[Employee]
WHERE 
	[Employee].[ID] = @EmployeeID

-- Выбираются StartDate и EndDate в зависимости от выбранного периода
SET @Date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS DATETIME)
IF @DateSelector = 1 -- current month
	SELECT
		@StartDate = DATEADD(day, 1-DATEPART(day, @Date), @Date),
		@EndDate = DATEADD(day, 1, @Date)
ELSE IF @DateSelector = 2 -- prev month
	SELECT
		@StartDate = DATEADD(day, 1-DATEPART(day, @Date), @Date),
		@StartDate = DATEADD(month, -1, @StartDate),
		@EndDate = DATEADD(month, 1, @StartDate)
ELSE IF @DateSelector = 3 -- quart
	SELECT
		@StartDate = [StartDate],
		@EndDate = DATEADD(day, 1, @Date)
	FROM	
		dbo.adfGetDateInterval(4, @Date, @Date, @Date)
ELSE IF @DateSelector = 4 -- prev quart
	SELECT
		@StartDate = [StartDate],
		@EndDate = [EndDate]
	FROM	
		dbo.adfGetDateInterval(3, @Date, @Date, @Date)
ELSE IF @DateSelector = 5 -- curr year
	SELECT
		@StartDate = CAST(Year(@Date) as varchar(4)),
		@EndDate = DATEADD(day, 1, @Date)
ELSE IF @DateSelector = 6 -- prev year
	SELECT
		@EndDate = CAST(Year(@Date) as varchar(4)),
		@StartDate = CAST(Year(@Date)-1 as varchar(4))
ELSE
	SELECT
		@StartDate = CAST(CONVERT(varchar(8), @StartDate, 112) AS datetime),
		@EndDate = DATEADD(day, 1, CAST(CONVERT(varchar(8), @EndDate, 112) AS datetime))



-- Рабочие дни
SET DATEFIRST 7
SET @WorkingDays = dbo.fnPjm5GetWorkingDays(@StartDate, DATEADD(day, -1, @EndDate))
SET DATEFIRST 1




SELECT
		@CompanyName				AS [CompanyName],
		@EmployeeFileAs				AS [EmployeeFileAs],
		@WorkingDays				AS [WorkingDays],
		@WorkingDays*8				AS [WorkingHours],
		@DateSelector				AS [DateSelector],
		@StartDate					AS [StartDate],
		DATEADD(day, -1, @EndDate)	AS [EndDate],
		@NormHours					AS [NormHours]



-- Основная таблица для вывода результата
DECLARE @WorkingApplications TABLE
(
	FileAs						TName,			-- FileAs сотрудника
	Days						int,			-- Всего (дни)
	AllHours					decimal(18,2),	-- Всего (часы)
	WorkingApplicationsHours	decimal(18,2),	-- Приложения, часы (рабочие)
	RemoteWorkHours				decimal(18,2),	-- Приложения, часы (в т.ч. удаленно)
	OtherApplicationsHours		decimal(18,2),	-- Приложения, часы (прочие)
	IdleHours					decimal(18,2),	-- Приложения, часы (простой)
	DayHours					decimal(18,2),	-- За день, часы (всего)
	DayWorkingApplicationsHours	decimal(18,2),	-- За день, часы (рабочие приложения)
	Deviation					decimal(18,2)	-- Отклонение
) 

INSERT INTO @WorkingApplications
(
	[FileAs],
	[Days],
	[AllHours],
	[WorkingApplicationsHours],
	[RemoteWorkHours],				
	[OtherApplicationsHours],	
	[IdleHours],					
	[DayHours],					
	[DayWorkingApplicationsHours],	
	[Deviation]			
)
SELECT
		[pjmams].[EmployeeView].[EmployeeName] AS [FileAs],
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]	
	LEFT JOIN [pjmams].[EmployeeView]
		ON [pjmams].[EmployeeView].[WindowsLogin] = [Process].[UserName]
WHERE 
	[Process].[Created] >= @StartDate 
	AND [Process].[Closed] <= @EndDate  
	AND [Process].[Created] < [Process].[Closed]
	AND 
	(
		@EmployeeID IS NULL
		OR
		[pjmams].[EmployeeView].[EmployeeID] = @EmployeeID
	) 	
GROUP BY
	[pjmams].[EmployeeView].[EmployeeName]
UNION
SELECT
	[Employee].[FileAs] As [FileAs],
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
FROM
	[dbo].[Employee]
	INNER JOIN [dbo].[Company]
		ON [Company].[Own] = [Employee].[CompanyID]
WHERE
	[Employee].[State] = 1




-- Временная таблица для формирования данных о днях и часах (всего), часах за день (всего), простое
DECLARE @t1 TABLE
(
	EmployeeFileAs		TName,
	AllDays				int, 
	AllHours			decimal(18,2),
	DayHours			decimal(18,2),
	IdleHours			decimal(18,2)
)


DECLARE @Temp TABLE
(
	FileAs	TName,
	Created	DATE
)
INSERT INTO @Temp
SELECT 
	[pjmams].[EmployeeView].[EmployeeName] AS [FileAs],
	CAST([Created] AS DATE) AS [Created]
FROM 
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
	LEFT JOIN [pjmams].[EmployeeView]
		ON [pjmams].[EmployeeView].[WindowsLogin] = [Process].[UserName]
WHERE 
	[Created] >= @StartDate
	AND [Closed] < @EndDate
	AND [Process].[Created] < [Process].[Closed]
	AND 
	(
		@EmployeeID IS NULL
		OR
		[pjmams].[EmployeeView].[EmployeeID] = @EmployeeID
	) 	
GROUP BY 
	CAST([pjmams].[Process].[Created] AS DATE),
	[pjmams].[EmployeeView].[EmployeeName]


DECLARE @t TABLE
(
	FileAs	TName,
	Days	int
)
INSERT INTO @t
SELECT
	[@Temp].[FileAs] AS [FileAs],
	COUNT(*) AS [Days]
FROM
	@Temp
GROUP BY
	[@Temp].[FileAs]


INSERT INTO @t1
SELECT
	[pjmams].[EmployeeView].[EmployeeName] AS [EmployeeFileAs],
	[@t].[Days] AS [AllDays],
	CAST(SUM(DATEDIFF(S,[Created],[Closed])) as decimal(18,2)) AS [AllHours],
	CAST(SUM(DATEDIFF(S,[Created],[Closed])) as decimal(18,2)) / [@t].[Days] AS [DayHours],
	CAST(SUM([Process].[Idle]) as decimal(18,2)) AS [IdleHours]
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
	INNER JOIN [pjmams].[Category]
		ON [Module].[CategoryID] = [Category].[ID]
	LEFT JOIN [pjmams].[EmployeeView]
		ON [pjmams].[EmployeeView].[WindowsLogin] = [Process].[UserName]
	INNER JOIN @t
		ON [FileAs] = [pjmams].[EmployeeView].[EmployeeName]
WHERE	
	[Process].[Created] >= @StartDate 
	AND [Process].[Closed] <= @EndDate  
	AND [Process].[Created] < [Process].[Closed]
	AND 
	(
		@EmployeeID IS NULL
		OR
		[pjmams].[EmployeeView].[EmployeeID] = @EmployeeID
	) 	
GROUP BY
	[pjmams].[EmployeeView].[EmployeeName],
	[@t].[Days]




-- Временная таблица для формирования данных о часах, часах за день, отклонениях по рабочим приложениям
DECLARE @t2 TABLE
(
	EmployeeFileAs				TName,
	WorkingApplicationsHours	decimal(18,2),
	DayWorkingApplicationsHours	decimal(18,2),
	Deviation					decimal(18,2)
)

INSERT INTO @t2
SELECT
	[pjmams].[EmployeeView].[EmployeeName] AS [EmployeeFileAs],

	CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2))
	AS [WorkingApplicationsHours],

	CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2))	/ [@t1].[AllDays] 
	AS [DayWorkingApplicationsHours],

	(@NormHours - (CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2))) / [@t1].[AllDays]) / @NormHours 
	AS [Deviation]
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
	INNER JOIN [pjmams].[Category]
		ON [Module].[CategoryID] = [Category].[ID]
	LEFT JOIN [pjmams].[EmployeeView]
		ON [pjmams].[EmployeeView].[WindowsLogin] = [Process].[UserName]
	INNER JOIN @t1
		ON [EmployeeFileAs] = [pjmams].[EmployeeView].[EmployeeName]
WHERE	
	[Created] >= @StartDate 
	AND [Closed] <= @EndDate
	AND [Module].[CategoryID] = 2
	AND [Process].[Created] < [Process].[Closed]
	AND
	(
		@EmployeeID IS NULL
		OR
		[pjmams].[EmployeeView].[EmployeeID] = @EmployeeID
	) 
GROUP BY
	[pjmams].[EmployeeView].[EmployeeName],
	[@t1].[AllDays]




-- Временная таблица для формирования данных о часах (в т.ч. удаленно)
DECLARE @t3 TABLE
(
	EmployeeFileAs		TName,
	RemoteWorkHours		decimal(18,2)
)

INSERT INTO @t3
SELECT
	[pjmams].[EmployeeView].[EmployeeName] AS [EmployeeFileAs],
	CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2)) AS [Hours]
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
	INNER JOIN [pjmams].[Session]
		ON [Process].[SessionID] = [Session].[ID]
	LEFT JOIN [pjmams].[EmployeeView]
		ON [pjmams].[EmployeeView].[WindowsLogin] = [Process].[UserName]
WHERE	
	[Process].[Created] >= @StartDate 
	AND [Process].[Closed] <= @EndDate
	AND [Session].[RemoteSession] = 1
	AND [Process].[Created] < [Process].[Closed]
	AND [Module].[CategoryID] = 2
	AND 
	(
		@EmployeeID IS NULL
		OR
		[pjmams].[EmployeeView].[EmployeeID] = @EmployeeID
	) 	
GROUP BY
	[pjmams].[EmployeeView].[EmployeeName]




-- Временная таблица для формирования данных о часах (прочие приложения)
DECLARE @t4 TABLE
(
	EmployeeFileAs			TName,
	OtherApplicationsHours	decimal(18,2)
)

INSERT INTO @t4
SELECT
	[pjmams].[EmployeeView].[EmployeeName] AS [EmployeeFileAs],
	CAST(SUM(DATEDIFF(S,[Created],[Closed]) - [Idle]) as decimal(18,2)) AS [OtherApplicationsHours]
FROM
	[pjmams].[Process]
	INNER JOIN [pjmams].[Module]
		ON [Process].[ModuleID] = [Module].[ID]
	INNER JOIN [pjmams].[Category]
		ON [Module].[CategoryID] = [Category].[ID]
	LEFT JOIN [pjmams].[EmployeeView]
		ON [pjmams].[EmployeeView].[WindowsLogin] = [Process].[UserName]
WHERE	
	[Created] >= @StartDate 
	AND [Closed] <= @EndDate
	AND (([Module].[CategoryID] = 3) OR ([Module].[CategoryID] = 1))
	AND [Process].[Created] < [Process].[Closed]
	AND
	(
		@EmployeeID IS NULL
		OR
		[pjmams].[EmployeeView].[EmployeeID] = @EmployeeID
	) 
GROUP BY
	[pjmams].[EmployeeView].[EmployeeName]




-- Объединение данных в итоговую таблицу
DECLARE 
	@tmp_emplFileAs TName,
	@tmp_days		int,
	@tmp_allHours	decimal(18,2),
	@tmp_waHours	decimal(18,2),
	@tmp_rwHours	decimal(18,2),
	@tmp_oaHours	decimal(18,2),
	@tmp_idleHours	decimal(18,2),
	@tmp_dayHours	decimal(18,2),
	@tmp_dwaHours	decimal(18,2),
	@tmp_deviation	decimal(18,2)


DECLARE insertCursor CURSOR LOCAL FAST_FORWARD FOR
SELECT
	[@t1].[EmployeeFileAs],
	[@t1].[AllDays],
	[@t1].[AllHours],
	[@t2].[WorkingApplicationsHours],
	[@t3].[RemoteWorkHours],
	[@t4].[OtherApplicationsHours],
	[@t1].[IdleHours],
	[@t1].[DayHours],
	[@t2].[DayWorkingApplicationsHours],
	[@t2].[Deviation]
FROM
	@t1
	LEFT JOIN @t2
		ON [@t1].[EmployeeFileAs] = [@t2].[EmployeeFileAs]
	LEFT JOIN @t3
		ON [@t1].[EmployeeFileAs] = [@t3].[EmployeeFileAs]
	LEFT JOIN @t4
		ON [@t1].[EmployeeFileAs] = [@t4].[EmployeeFileAs]

OPEN insertCursor
FETCH NEXT FROM insertCursor INTO @tmp_emplFileAs, @tmp_days, @tmp_allHours,
									@tmp_waHours, @tmp_rwHours, @tmp_oaHours,
									@tmp_idleHours, @tmp_dayHours, @tmp_dwaHours,
									@tmp_deviation
WHILE @@FETCH_STATUS = 0
BEGIN
	
	UPDATE
		@WorkingApplications
	SET
		[Days] = @tmp_days,
		[AllHours] = @tmp_allHours,
		[WorkingApplicationsHours] = @tmp_waHours,
		[RemoteWorkHours] = @tmp_rwHours,
		[OtherApplicationsHours] = @tmp_oaHours,
		[IdleHours] = @tmp_idleHours,
		[DayHours] = @tmp_dayHours,
		[DayWorkingApplicationsHours] = @tmp_dwaHours,
		[Deviation] = @tmp_deviation
	WHERE
		[@WorkingApplications].[FileAs] = @tmp_emplFileAs


	FETCH NEXT FROM insertCursor INTO @tmp_emplFileAs, @tmp_days, @tmp_allHours,
									@tmp_waHours, @tmp_rwHours, @tmp_oaHours,
									@tmp_idleHours, @tmp_dayHours, @tmp_dwaHours,
									@tmp_deviation
									
END
CLOSE insertCursor
DEALLOCATE insertCursor

SELECT * FROM @WorkingApplications
	
RETURN 0
GO

grant execute on [pjmams].[ReportApplicationTimeAllocationByEmployees] to [activitymonitorsite];
go

--**************SessionBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[SessionBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[SessionBrowse]'
      DROP PROCEDURE [pjmams].[SessionBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[SessionBrowse]'
GO
       

create procedure [pjmams].[SessionBrowse]
as
	set nocount on;


	declare @date date;

	set @date = getdate();

	with [T]([ID], [MachineName], [EmployeeName]) as 
	(
		select 
			max([S].[ID]),
			[S].[MachineName],
			coalesce([E].[EmployeeName], [S].[UserName], [P].[UserName]) as [EmployeeName]
		from 
			[pjmams].[Session] as [S] left outer join [pjmams].[Process] as [P]
				on ([S].[ActiveProcessID] = [P].[ID])
			left outer join [pjmams].[EmployeeView] as [E]
				on ([S].[UserName] = [E].[WindowsLogin])
			left outer join [pjmams].[Module] as [M]
				on ([P].[ModuleID] = [M].[ID])
		where
			cast([SessionEnd] as date) = @date			
		group by
			[S].[MachineName],
			coalesce([E].[EmployeeName], [S].[UserName], [P].[UserName])				
	)
	select 
		[S].[ID],
		[S].[MachineName],
		[S].[SessionStart],
		[S].[SessionEnd],
		(case when [S].[UserStatus] = 1 and datediff(second, [S].[SessionEnd], getdate()) > 300 then 2 else [S].[UserStatus] end) as [UserStatus],
		[S].[RemoteSession],
		[S].[WinDir],
		[S].[ProgramFiles],
		[S].[ActiveProcessID],
		[S].[UserStatusTime],
		[S].[UserName],
		[P].[Created],
		isnull([M].[Description], [M].[Name]) as [Module],
		coalesce([E].[EmployeeName], [S].[UserName], [P].[UserName]) as [EmployeeName]
	from 
		[pjmams].[Session] as [S] inner join [T]	
			on ([S].[ID] = [T].[ID])
		left outer join [pjmams].[Process] as [P]
			on ([S].[ActiveProcessID] = [P].[ID])
		left outer join [pjmams].[EmployeeView] as [E]
			on ([S].[UserName] = [E].[WindowsLogin])
		left outer join [pjmams].[Module] as [M]
			on ([P].[ModuleID] = [M].[ID])
	where
		cast([SessionEnd] as date) = @date
	order by
		coalesce([E].[EmployeeName], [S].[UserName], [P].[UserName]),
		[S].[MachineName];

	return 0;
go

grant execute on [pjmams].[SessionBrowse] to [activitymonitorsite];
go

--**************UserActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[UserActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[UserActivityBrowse]'
      DROP PROCEDURE [pjmams].[UserActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[UserActivityBrowse]'
GO
       

create procedure [pjmams].[UserActivityBrowse] @Date date, @MachineName nvarchar(128),@UserName nvarchar(128),
@CurrentUserName nvarchar(128)

as
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END

	
	DECLARE @EmployeeName nvarchar(256);
	
	select
		@EmployeeName = [EmployeeName]
	from
		[pjmams].[EmployeeView]
	where
		[WindowsLogin] = @UserName
	
	set @EmployeeName = isnull(@EmployeeName, @UserName)
	
	declare @Total decimal(18, 2), @Minute decimal(18,4), @Hour decimal(18,4);

	set @Minute = 60;
	set @Hour = 3600;

	with [A] ([Total]) as
	(
		select
			cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18,2))
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				on ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				on [Process].[ModuleID] = [Module].[ID]
		where
			cast([Created] as date) = @Date			
		and
			cast([Closed] as date) = @Date
		and
			[Process].[UserName] = @UserName AND [Session].[MachineName] = @MachineName
		AND
		(
			@HasViewAllActivity = 1
			OR
			(	@HasViewOwnActivity = 1 
				AND 
				[Process].[UserName] = @CurrentUserName
			)
		)
		group by
			isnull([Module].[Description], [Module].[Name])
	)
	select
		@Total = max([Total])
	from
		[A];
		
	select
		@EmployeeName as [EmployeeName],
		@Date as [Date],
		@UserName as [UserName],
		@MachineName as [MachineName];

	with [A] ([Name], [CategoryID], [SortKey], [Total], [Idle]) as
	(
		select
			isnull([Module].[Description], [Module].[Name]), 
			[Module].[CategoryID],
			case 
				when [Module].[CategoryID] = 2 then 0 
				when [Module].[CategoryID] = 3 then 1
				else 2
			end,
			sum(datediff(second, [Created], [Closed]) - [Idle]),
			sum([Idle])
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				ON ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				ON [Process].[ModuleID] = [Module].[ID]
		where
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		and
			[Process].[UserName] = @UserName AND [Session].[MachineName] = @MachineName
		AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			)
		group by
			isnull([Module].[Description], [Module].[Name]),
			[Module].[CategoryID]
	)
	select top 20
		row_number() over (order by [SortKey] asc, [Total] desc) as [Row],
		[Name], 
		[CategoryID],
		cast([Total] / @Hour as decimal(18,2)) as [Duration],
		cast([Idle] / @Hour as decimal(18,2)) as [Idle],
		cast([Total] * 100.0 / @Minute / @Total as decimal(18,2)) as [RelativeDuration],
		[Name] as [Module]
	from 
		[A]
	where
		[Total] >= 60
	order by
		[SortKey] asc, [Total] desc;

	return 0;
go

grant execute on [pjmams].[UserActivityBrowse] to [activitymonitorsite];
go
  

--**************UserModuleActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[UserModuleActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[UserModuleActivityBrowse]'
      DROP PROCEDURE [pjmams].[UserModuleActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[UserModuleActivityBrowse]'
GO
       

create procedure [pjmams].[UserModuleActivityBrowse] 
	@Date date,	
	@MachineName nvarchar(128),
	@Module nvarchar(max),
	@UserName nvarchar(128),
	@CurrentUserName nvarchar(128)
	as
	set nocount on;


	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END

	
	DECLARE @EmployeeName nvarchar(256);
	
	select
		@EmployeeName = [EmployeeName]
	from
		[pjmams].[EmployeeView]
	where
		[WindowsLogin] = @UserName
	
	SET @EmployeeName = isnull(@EmployeeName, @UserName)
	
	SELECT
		@EmployeeName as [EmployeeName],
		@MachineName as [MachineName],
		@Module as [Module],
		@Date as [Date]
	
	select
		isnull([Module].[Description], [Module].[Name]) as [ModuleName],
		[Window].[Caption] as [Window],
		MIN([Window].[Created]) as [Created],
		SUM([Window].[Duration]) as [DurationSeconds]
	from 
		[pjmams].[Session] inner join [pjmams].[Process] 
			on ([Session].[ID] = [Process].[SessionID])
		inner join [pjmams].[Module]
			on ([Process].[ModuleID] = [Module].[ID])
		inner join [pjmams].[Window]
			on ([Window].[ProcessID] = [Process].[ID])
	where
		cast([Window].[Created] as date) = @Date
	and
		[Process].[UserName] = @UserName AND [Session].[MachineName]=@MachineName
	and
		(@Module is null or isnull([Module].[Description], [Module].[Name]) = @Module)
	and
		[Window].[Caption] is not null and [Window].[Caption] != ''
	AND
		(
			@HasViewAllActivity = 1
			OR
			(	@HasViewOwnActivity = 1 
				AND 
				[Process].[UserName] = @CurrentUserName
			)
		)
	group by
		isnull([Module].[Description], [Module].[Name]),
		[Window].[Caption]
	order by
		3 asc;			

	return 0;
go

grant execute on [pjmams].[UserModuleActivityBrowse] to [activitymonitorsite]
go

   

--**************UsersActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[UsersActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[UsersActivityBrowse]'
      DROP PROCEDURE [pjmams].[UsersActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[UsersActivityBrowse]'
GO
       

create procedure [pjmams].[UsersActivityBrowse] @Date date, @CurrentUserName nvarchar(128)
as
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END
	
	select @Date as [Date];
		
	declare @Total decimal(18, 2), @Hour decimal(18, 4), @Minute decimal(18, 4);
	
	set @Hour = 3600;
	set @Minute = 60;
	
	with [A] ([Total]) as
	(
		select
			cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18, 2))
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				ON ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				ON [Process].[ModuleID] = [Module].[ID]
			left outer join [pjmams].[EmployeeView]
				on ([Process].[UserName] = [EmployeeView].[WindowsLogin])
		where
			[Process].[UserName] != '\'
			AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			) 
		and
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		group by
			isnull([EmployeeView].[EmployeeName], [Process].[UserName]),
			[Process].[UserName],
			[Session].[MachineName]
	)
	select
		@Total = max([Total])
	from
		[A];

	with [A] ([EmployeeName], [UserName], [MachineName], [Category1], [Category2], [Category3], [Total], [Idle]) as
	(
		select
			isnull([EmployeeView].[EmployeeName], [Process].[UserName]) as [EmployeeName],
			[Process].[UserName],
			[Session].[MachineName],
			sum((case when [Module].[CategoryID] = 1 then datediff(second, [Created], [Closed]) - [Idle] else 0 end)),
			sum((case when [Module].[CategoryID] = 2 then datediff(second, [Created], [Closed]) - [Idle] else 0 end)),
			sum((case when [Module].[CategoryID] = 3 then datediff(second, [Created], [Closed]) - [Idle] else 0 end)),
			sum(datediff(second, [Created], [Closed]) - [Idle]),
			sum([Idle])
		from 
			[pjmams].[Session] inner join [pjmams].[Process] 
				on ([Session].[ID] = [Process].[SessionID])
			inner join [pjmams].[Module]
				on [Process].[ModuleID] = [Module].[ID]
			left outer join [pjmams].[EmployeeView]
				on ([Process].[UserName] = [EmployeeView].[WindowsLogin])
		where
			[Process].[UserName] != '\'
			AND
			(
				@HasViewAllActivity = 1
				OR
				(	@HasViewOwnActivity = 1 
					AND 
					[Process].[UserName] = @CurrentUserName
				)
			) 
		and
			cast([Created] as date) = @Date
		and
			cast([Closed] as date) = @Date
		group by
			isnull([EmployeeView].[EmployeeName], [Process].[UserName]),
			[Process].[UserName],
			[Session].[MachineName]
	)
	select
		row_number() over (order by [Total] desc) as [Row],
		[EmployeeName],
		[UserName],
		[MachineName],
		cast([Category1] * 100.0 / [Total] as decimal(18,2)) as [Category1],
		cast([Category2] * 100.0 / [Total] as decimal(18,2)) as [Category2],
		cast([Category3] * 100.0 / [Total] as decimal(18,2)) as [Category3],
		cast([Idle] / @Hour as decimal(18,2)) as [Idle],
		cast([Total] / @Hour as decimal(18,2)) as [Duration],
		cast([Total] * 100 / @Minute / @Total as decimal(18,2)) as [RelativeDuration]
	from
		[A]
	where
		[Total] > 60
	order by
		[Total] desc;

	return 0;
go

grant execute on [pjmams].[UsersActivityBrowse] to [activitymonitorsite];
go


--**************WorkstationActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[WorkstationActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[WorkstationActivityBrowse]'
      DROP PROCEDURE [pjmams].[WorkstationActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[WorkstationActivityBrowse]'
GO
       

create procedure [pjmams].[WorkstationActivityBrowse] @Date date, @MachineName nvarchar(128),@CurrentUserName nvarchar(128)
as
	set nocount on;


	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END

	
	select @Date as [Date], @MachineName as [MachineName];
		
	declare @Total decimal(18, 2), @Minute decimal(18,4), @Hour decimal(18,4);

	set @Minute = 60;
	set @Hour = 3600;
	
	select top 1
		@Total = cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18, 2))
	from 
		[pjmams].[Session] inner join [pjmams].[Process] 
			on ([Session].[ID] = [Process].[SessionID])
		inner join [pjmams].[Module]
			on [Process].[ModuleID] = [Module].[ID]
	where
		[Process].[UserName] != '\'
	AND
		(
			@HasViewAllActivity = 1
			OR
			(	@HasViewOwnActivity = 1 
				AND 
				[Process].[UserName] = @CurrentUserName
			)
		) 
	and
		cast([Created] as date) = @Date
	and
		cast([Closed] as date) = @Date;

	select
		row_number() over (order by sum(datediff(second, [Created], [Closed]) - [Idle]) desc) as [Row],
		isnull([EmployeeView].[EmployeeName], [Process].[UserName]) as [EmployeeName],
		[Process].[UserName],
		cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Hour as decimal(18,2)) as [Duration],
		cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18,2)) * 100 / @Total as [RelativeDuration]
	from 
		[pjmams].[Session] inner join [pjmams].[Process] 
			on ([Session].[ID] = [Process].[SessionID])
		inner join [pjmams].[Module]
			on [Process].[ModuleID] = [Module].[ID]
		left outer join [pjmams].[EmployeeView]
			on ([Process].[UserName] = [EmployeeView].[WindowsLogin])
	where
		[Process].[UserName] != '\'
	AND
		(
			@HasViewAllActivity = 1
			OR
			(	@HasViewOwnActivity = 1 
				AND 
				[Process].[UserName] = @CurrentUserName
			)
		) 
	and
		cast([Created] as date) = @Date
	and
		cast([Closed] as date) = @Date
	and
		[Session].[MachineName] = @MachineName
	group by
		isnull([EmployeeView].[EmployeeName], [Process].[UserName]),
		[Process].[UserName]
	order by 
		 5 desc;
	
	return 0;
go

grant execute on [pjmams].[WorkstationActivityBrowse] to [activitymonitorsite];
go

    

--**************WorkstationsActivityBrowse**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[WorkstationsActivityBrowse]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[WorkstationsActivityBrowse]'
      DROP PROCEDURE [pjmams].[WorkstationsActivityBrowse]
END
GO

PRINT 'Creating procedure [pjmams].[WorkstationsActivityBrowse]'
GO
       

create procedure [pjmams].[WorkstationsActivityBrowse] @Date date, @CurrentUserName nvarchar(128)
as
	set nocount on;

	DECLARE
		@HasViewOwnActivity int,
		@HasViewAllActivity int

	SET @HasViewOwnActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewOwnActivity', @CurrentUserName)
	SET @HasViewAllActivity = [pjmams].fnHasAccessRightByNameForUserLogin('ViewAllActivity', @CurrentUserName)

	IF(@HasViewOwnActivity = 0 AND @HasViewAllActivity = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END


	
	select @Date as [Date]
		
	declare @Total decimal(18, 2), @Minute decimal(18,4), @Hour decimal(18,4);

	set @Minute = 60;
	set @Hour = 3600;
	
	select top 1
		@Total = cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18,2))
	from 
		[pjmams].[Session] inner join [pjmams].[Process] 
			on ([Session].[ID] = [Process].[SessionID])
		inner join [pjmams].[Module]
			on [Process].[ModuleID] = [Module].[ID]
	where
		[Process].[UserName] != '\'
	AND
		(
			@HasViewAllActivity = 1
			OR
			(	@HasViewOwnActivity = 1 
				AND 
				[Process].[UserName] = @CurrentUserName
			)
		) 
	and
		cast([Created] as date) = @Date
	and
		cast([Closed] as date) = @Date;

	select
		row_number() over (order by sum(datediff(second, [Created], [Closed]) - [Idle]) desc) as [Row],
		[Session].[MachineName],
		cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Hour as decimal(18,2)) as [Duration],
		cast(sum(datediff(second, [Created], [Closed]) - [Idle]) / @Minute as decimal(18,2)) * 100 / @Total as [RelativeDuration]
	FROM 
		[pjmams].[Session] inner join [pjmams].[Process] 
			on ([Session].[ID] = [Process].[SessionID])
		inner join [pjmams].[Module]
			on [Process].[ModuleID] = [Module].[ID]
	where
		[Process].[UserName] != '\'
	AND
		(
			@HasViewAllActivity = 1
			OR
			(	@HasViewOwnActivity = 1 
				AND 
				[Process].[UserName] = @CurrentUserName
			)
		) 
	and
		cast([Created] as date) = @Date
	and
		cast([Closed] as date) = @Date
	group by
		[Session].[MachineName]
	order by 
		 4 desc;
	
	return 0;
go

grant execute on [pjmams].[WorkstationsActivityBrowse] to [activitymonitorsite];
go

    

--//////////////Service scripts////////////////////
--**************CloseProcessEntry**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[CloseProcessEntry]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[CloseProcessEntry]'
      DROP PROCEDURE [pjmams].[CloseProcessEntry]
END
GO

PRINT 'Creating procedure [pjmams].[CloseProcessEntry]'
GO
       

create procedure [pjmams].[CloseProcessEntry] @ID int, @MessageTime datetime
as
begin
	set nocount on
	declare @SessionID int;

	update [pjmams].[Process] 
	set 
		@SessionID = [SessionID],
		[Closed] = @MessageTime
	where 
		[ID] = @ID;	

	return 0;
end
GO

grant execute on [pjmams].[CloseProcessEntry] to [activitymonitorservice];
GO


--**************CreateProcessEntry**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[CreateProcessEntry]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[CreateProcessEntry]'
      DROP PROCEDURE [pjmams].[CreateProcessEntry]
END
GO

PRINT 'Creating procedure [pjmams].[CreateProcessEntry]'
GO
       

create procedure [pjmams].[CreateProcessEntry] 
	@SessionID INT, 
	@Pid INT, 
	@ModuleName nvarchar(max), 
	@ModuleDescription nvarchar(max), 
	@Version varchar(32),
	@Lang varchar(32),
	@UserName nvarchar(128),
	@MessageTime datetime
as
begin
	set nocount on;
	
	declare @ModuleID int,
			@CategoryID int,
			@EntryID int,
			@EntryCreated datetime,
			@PrevEntryClosed datetime

	if rtrim(ltrim(@ModuleDescription)) = ''
	begin
		set @ModuleDescription = null;
	end


	select top 1
		@CategoryID = [CategoryID]
	from
		[pjmams].[Module]
	where
		[Module].[Description] = @ModuleDescription
	and
		[CategoryID] <> 1;

	if @CategoryID is null
		set @CategoryID = 1;

	--Защищенные системные процессы относятся к категории "Прочее"
	IF @Pid = 0
		SET @CategoryID = 3

	select
		@ModuleID = [ID]
	from
		[pjmams].[Module]
	where
		[Name] = @ModuleName 
		and ([Description] = @ModuleDescription or ([Description] is null and @ModuleDescription is null))
		and ([Version] = @Version or ([Version] is null and @Version is null))
		and ([Lang] = @Lang or ([Lang] is null and @Lang is null))
	
	begin transaction;
	
	if @ModuleID is null
	begin
		insert into [pjmams].[Module]
			([Name], [Description], [Version], [Lang], [CategoryID])
		values
			(@ModuleName, @ModuleDescription, @Version, @Lang, @CategoryID)
		if @@error != 0
		begin
			rollback transaction;
			return 1;
		end	
		set @ModuleID = scope_identity();
	end
	
	insert into [pjmams].[Process] 
		([SessionID], [ModuleID], [PID], [UserName], [Created], [Closed]) 
	values
		(@SessionID, @ModuleID, @Pid, @UserName, @MessageTime, @MessageTime); 

	if @@error != 0
	begin
		rollback transaction;
		return 1;
	end

	set @EntryID = scope_identity();


	update [pjmams].[Session] 
	set 
		[SessionEnd] = @MessageTime,
		[ActiveProcessID] = @EntryID
	where 
		[ID] = @SessionID;

	if @@error != 0
	begin
		rollback transaction;
		return 1;
	end

	commit transaction;

	select cast(@EntryID as decimal(10));
	
	return 0;
end
go

grant execute on [pjmams].[CreateProcessEntry] to [activitymonitorservice];
go


--**************CreateWindowEntry**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[CreateWindowEntry]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[CreateWindowEntry]'
      DROP PROCEDURE [pjmams].[CreateWindowEntry]
END
GO

PRINT 'Creating procedure [pjmams].[CreateWindowEntry]'
GO
       

create procedure [pjmams].[CreateWindowEntry] 
	@ProcessID int,
	@Hwnd int,
	@Caption nvarchar(max),
	@MessageTime datetime
as
begin
	set nocount on;

	declare
		@NewEntryID int

	
	insert into [pjmams].[Window] 
		([ProcessID], [HWND], [Caption], [Created]) 
	values
		(@ProcessID, @Hwnd, @Caption, @MessageTime);
		
	set @NewEntryID = SCOPE_IDENTITY()
	 

	update [pjmams].[Process]
	set
		[Closed] = @MessageTime
	where
		[ID] = @ProcessID;

	declare @SessionID int;

	select
		@SessionID = [SessionID]
	from
		[pjmams].[Process]
	where
		[Process].[ID] = @ProcessID;

	if @SessionID is not null
	begin
		update [pjmams].[Session] 
		set 
			[SessionEnd] = @MessageTime,
			[ActiveProcessID] = @ProcessID,
			[ActiveWindowID] = @NewEntryID
		where 
			[ID] = @SessionID;
	end

	return 0;
end
go

grant execute on [pjmams].[CreateWindowEntry] to [activitymonitorservice];
GO
  

--**************GetClientSettings**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[GetClientSettings]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[GetClientSettings]'
      DROP PROCEDURE [pjmams].[GetClientSettings]
END
GO

PRINT 'Creating procedure [pjmams].[GetClientSettings]'
GO
       

CREATE PROCEDURE [pjmams].[GetClientSettings] 
AS
BEGIN
	SELECT 
		[dbo].[F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53].[Name],
		[dbo].[F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53].[Value]
	FROM
		[dbo].[F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53]
	WHERE
		[dbo].[F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53].[Group] = 'ActivityMonitorClientSettings'
END
GO

GRANT EXECUTE ON [pjmams].[GetClientSettings] TO PUBLIC;
GO


--**************HeartBeat**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[HeartBeat]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[HeartBeat]'
      DROP PROCEDURE [pjmams].[HeartBeat]
END
GO

PRINT 'Creating procedure [pjmams].[HeartBeat]'
GO
       

CREATE PROCEDURE [pjmams].[HeartBeat] 
	@SessionID int,
	@MessageTime datetime
as
begin
	set nocount on;
	
	declare 
		@ActiveProcessID int,
		@ActiveWindowID int;

	update [pjmams].[Session] 
	set 
		@ActiveProcessId = [ActiveProcessID],
		@ActiveWindowID = [ActiveWindowID],
		[SessionEnd] = @MessageTime
	where 
		[ID] = @SessionID;
	
	if @ActiveProcessID is not null
	begin
		update [pjmams].[Process]
		set
			[Closed] = @MessageTime
		where
			[ID] = @ActiveProcessID;
	end

	if @ActiveWindowID is not null
	begin
		update [pjmams].[Window]
		set
			[Duration] = datediff(second, [Created], @MessageTime)
		where
			[ID] = @ActiveWindowID;
	end

	return 0;
end
go

grant execute on [pjmams].[HeartBeat] to [activitymonitorservice];
GO
    

--**************OpenSession**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[OpenSession]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[OpenSession]'
      DROP PROCEDURE [pjmams].[OpenSession]
END
GO

PRINT 'Creating procedure [pjmams].[OpenSession]'
GO
       

CREATE PROCEDURE [pjmams].[OpenSession] 
	@MachineName nvarchar(128),
	@WinDir nvarchar(256),
	@ProgramFiles nvarchar(256),
	@RemoteSession tinyint,
	@UserName nvarchar(128),
	@MessageTime datetime
as
begin
	set nocount on;
	
	update [pjmams].[Session]
	set
		[UserStatus] = 3
	where
		[MachineName] = @MachineName and [UserName] = @UserName;

	insert into [pjmams].[Session] 
		([MachineName], [WinDir], [ProgramFiles], [UserStatus], [RemoteSession], [UserName], [SessionStart], [SessionEnd], [UserStatusTime]) 
	values
		(@MachineName, @WinDir, @ProgramFiles, 1, @RemoteSession, @UserName, @MessageTime, @MessageTime, @MessageTime); 

	select scope_identity();
	
	return 0;
end
go

grant execute on [pjmams].[OpenSession] to [activitymonitorservice];
go
   

--**************UpdateClientSetting**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[UpdateClientSetting]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[UpdateClientSetting]'
      DROP PROCEDURE [pjmams].[UpdateClientSetting]
END
GO

PRINT 'Creating procedure [pjmams].[UpdateClientSetting]'
GO
       

CREATE PROCEDURE [pjmams].[UpdateClientSetting] 
	@Name nvarchar(256),
	@Value sql_variant,
	@CurrentUserName nvarchar(128)
AS
BEGIN

	DECLARE
		@HasManageActivityMonitor int

	SET @HasManageActivityMonitor = [pjmams].fnHasAccessRightByNameForUserLogin('ManageActivityMonitor', @CurrentUserName)

	IF(@HasManageActivityMonitor = 0)
	BEGIN
		RAISERROR('#AccessDenied', 18, -1)
		RETURN 1
	END


	DECLARE
		@Group nvarchar(256) = 'ActivityMonitorClientSettings'

	IF EXISTS(SELECT * FROM [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53] WHERE [Name]=@Name AND [Group]=@Group)
	BEGIN
		UPDATE [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53] 
        SET 
            Value=@Value
        WHERE 
            [Name]=@Name AND [Group]=@Group
	END
	ELSE
	BEGIN
		INSERT INTO [F41AB0F8-FDF0-4FAD-9B71-8F17B4A51B53] 
			([Value], [Name], [Group])
		VALUES
			(@Value, @Name, @Group)
	END
	RETURN @@error
RETURN 0
END
GO

GRANT EXECUTE ON [pjmams].[UpdateClientSetting] TO [activitymonitorsite];
GO


--**************UpdateUserState**************
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[pjmams].[UpdateUserState]') AND type in (N'P', N'PC'))
BEGIN
      PRINT 'Dropping procedure [pjmams].[UpdateUserState]'
      DROP PROCEDURE [pjmams].[UpdateUserState]
END
GO

PRINT 'Creating procedure [pjmams].[UpdateUserState]'
GO
       

create procedure [pjmams].[UpdateUserState] 
	@SessionID int,
	@UserStatus int,
	@MessageTime datetime
as
begin
	set nocount on;
	
	declare 
		@UserStatusTime datetime, 
		@PrevStatus int,
		@ActiveProcessID int, 
		@Idle int, 
		@CurrentTime datetime,
		@ActiveWindowID int,
		@Duration int;

	set @CurrentTime = @MessageTime;

	update [pjmams].[Session] 
	set 
		[SessionEnd] = @CurrentTime,
		@PrevStatus = [UserStatus],
		[UserStatus] = @UserStatus,
		@UserStatusTime = [UserStatusTime],
		@ActiveProcessID = [ActiveProcessID],
		@ActiveWindowID = [ActiveWindowID],
		[UserStatusTime] = @CurrentTime
	where 
		[ID] = @SessionID;

	if @ActiveProcessID is not null
	begin
		if @UserStatus = 1 and @PrevStatus != @UserStatus
		begin
			set @Idle = datediff(second, @UserStatusTime, @CurrentTime);

			update [pjmams].[Process]
			set
				[Closed] = @CurrentTime,
				[Idle] = [Idle] + @Idle
			where
				[ID] = @ActiveProcessID;
		end
		else
		begin
			update [pjmams].[Process]
			set
				[Closed] = @CurrentTime
			where
				[ID] = @ActiveProcessID;
		end
	end


	if @ActiveWindowID is not null and @UserStatus = 1
	begin
		update [pjmams].[Window]
		set
			[Duration] = datediff(second, [Created], @CurrentTime)
		where
			[ID] = @ActiveWindowID
	end
	
	return 0;
end
go

grant execute on [pjmams].[UpdateUserState] to [activitymonitorservice];
GO
    

