IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[adfBusinessCardFilling]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure adfBusinessCardFilling'
		DROP PROCEDURE [dbo].[adfBusinessCardFilling]
	END
GO

PRINT 'Creating procedure adfBusinessCardFilling'
GO
/******************************************************************************
**		File: adfBusinessCardFilling.sql
**		Name: adfBusinessCardFilling
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Nikita.Golyshev
**		Date: 27/03/2014 16:43:25
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
*******************************************************************************/
CREATE Procedure [dbo].[adfBusinessCardFilling]
	@doc			Xml,
	@XPathRoot		nvarchar(256),
	@ID				TIdentifier,
	@Class			TClass,
	@FileAs			TName,
	@Comments		TNotes,
	@CompanyID		TIdentifier,
	@PositionID		TIdentifier
AS
	DECLARE @result INT,
			@PhoneTypeID INT,
			@AddressTypeID INT,
			@EmailTypeID INT
	
	SET NOCOUNT ON
	SET QUOTED_IDENTIFIER ON

	BEGIN TRANSACTION
	
	UPDATE [BusinessCard]
	SET 
		[FileAs] = 		CASE WHEN [FileAs] IS NULL THEN @FileAs ELSE [FileAs] END,
		[Comments] = 	CASE WHEN [Comments] IS NULL THEN @Comments ELSE [Comments] END,
		[CompanyID] = 	CASE WHEN [CompanyID] IS NULL THEN @CompanyID ELSE [CompanyID] END,
		[PositionID] = 	CASE WHEN [PositionID] IS NULL THEN @PositionID ELSE [PositionID] END,
		[Modified] = 	CASE WHEN [Modified] IS NULL THEN CURRENT_TIMESTAMP ELSE [Modified] END,
		[ModifiedBy] = 	CASE WHEN [ModifiedBy] IS NULL THEN dbo.adfGetCurrentUserID() ELSE [ModifiedBy] END
	WHERE 
		[ID] = @ID AND [Class] = @Class
	
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	
	

	IF @XPathRoot = '/adfEmployeeGet'
	BEGIN
	
		;MERGE [Phone] AS target
		USING (	SELECT
					@ID AS ID,
					@Class AS Class,
					T.c.value('PhoneTypeID[1]','int') AS PhoneTypeID,
					T.c.value('Phone[1]','varchar(50)') AS Phone
				FROM
					@doc.nodes('/adfEmployeeGet/Phone') AS T(c)	) AS source
		ON (target.BusinessCardID = source.ID) AND (target.BusinessCardClass = source.Class) AND (target.PhoneTypeID = source.PhoneTypeID)
		WHEN MATCHED THEN 
			UPDATE 
			SET [Phone] = CASE 
					WHEN target.[Phone] IS NULL OR target.[Phone] = '' THEN source.Phone
					ELSE target.[Phone] END
		WHEN NOT MATCHED THEN	
			INSERT ([BusinessCardID], [BusinessCardClass], [PhoneTypeID], [Phone])
			VALUES (source.ID, source.Class, source.PhoneTypeID, source.Phone);
	
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
			
			
		;MERGE [Address] AS target
		USING (	SELECT
					@ID AS ID,
					@Class AS Class,
					T.c.value('AddressTypeID[1]','int') AS AddressTypeID,
					T.c.value('Street[1]','nvarchar(256)') AS Street,
					T.c.value('City[1]','nvarchar(50)') AS City,
					T.c.value('Region[1]','nvarchar(256)') AS Region,
					T.c.value('PostalCode[1]','varchar(50)') AS PostalCode,
					T.c.value('Country[1]','nvarchar(256)') AS Country,
					(CASE 
						WHEN T.c.value('Mail[1]','varchar(5)')='false' THEN 0 
						ELSE 1 
					END) AS Mail
				FROM
					@doc.nodes('/adfEmployeeGet/Address') AS T(c)	) AS source
		ON (target.BusinessCardID = source.ID) AND (target.BusinessCardClass = source.Class) AND (target.AddressTypeID = source.AddressTypeID)
		WHEN MATCHED THEN 
			UPDATE 
			SET [BusinessCardID] = CASE 
					WHEN target.[BusinessCardID] IS NULL OR target.[BusinessCardID] = '' THEN source.ID
					ELSE target.[BusinessCardID] END,
				[BusinessCardClass] = CASE 
					WHEN target.[BusinessCardClass] IS NULL OR target.[BusinessCardClass] = '' THEN source.Class
					ELSE target.[BusinessCardClass] END,
				[AddressTypeID] = CASE 
					WHEN target.[AddressTypeID] IS NULL OR target.[AddressTypeID] = '' THEN source.AddressTypeID
					ELSE target.[AddressTypeID] END,
				[Street] = CASE 
					WHEN target.[Street] IS NULL OR target.[Street] = '' THEN source.Street
					ELSE target.[Street] END,
				[City] = CASE 
					WHEN target.[City] IS NULL OR target.[City] = '' THEN source.City
					ELSE target.[City] END,
				[Region] = CASE 
					WHEN target.[Region] IS NULL OR target.[Region] = '' THEN source.Region
					ELSE target.[Region] END,
				[PostalCode] = CASE 
					WHEN target.[PostalCode] IS NULL OR target.[PostalCode] = '' THEN source.PostalCode
					ELSE target.[PostalCode] END,
				[Country] = CASE 
					WHEN target.[Country] IS NULL OR target.[Country] = '' THEN source.Country
					ELSE target.[Country] END,
				[Mail] = CASE 
					WHEN target.[Mail] IS NULL THEN source.Mail
					ELSE target.[Mail] END
		WHEN NOT MATCHED THEN	
			INSERT ([BusinessCardID], [BusinessCardClass], [AddressTypeID], [Street], [City], [Region], 
				[PostalCode], [Country], [Mail])
			VALUES (source.ID, source.Class, source.AddressTypeID, source.Street, 
				source.City, source.Region, source.PostalCode, source.Country, source.Mail);
	
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
			

		;MERGE [Email] AS target
		USING (	SELECT
					@ID AS ID,
					@Class AS Class,
					T.c.value('EmailTypeID[1]','int') AS EmailTypeID,
					T.c.value('Email[1]','nvarchar(256)') AS Email,
					T.c.value('Email[1]','nvarchar(256)') AS DisplayAs
				FROM
					@doc.nodes('/adfEmployeeGet/Email') AS T(c)	) AS source
		ON (target.BusinessCardID = source.ID) AND (target.BusinessCardClass = source.Class) AND (target.EmailTypeID = source.EmailTypeID)
		WHEN MATCHED THEN 
			UPDATE 
			SET [Email] = CASE 
					WHEN target.[Email] IS NULL OR target.[Email] = '' THEN source.Email
					ELSE target.[Email] END,
				[DisplayAs] = CASE 
					WHEN target.[DisplayAs] IS NULL OR target.[DisplayAs] = '' THEN source.DisplayAs
					ELSE target.[DisplayAs] END
		WHEN NOT MATCHED THEN	
			INSERT ([BusinessCardID], [BusinessCardClass], [EmailTypeID], [Email], [DisplayAs])
			VALUES (source.ID, source.Class, source.EmailTypeID, source.Email, source.DisplayAs);
	
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END

	END
	ELSE IF @XPathRoot = '/adfCompanyGet'
		BEGIN
	
		;MERGE [Phone] AS target
		USING (	SELECT
					@ID AS ID,
					@Class AS Class,
					T.c.value('PhoneTypeID[1]','int') AS PhoneTypeID,
					T.c.value('Phone[1]','varchar(50)') AS Phone
				FROM
					@doc.nodes('/adfCompanyGet/Phone') AS T(c)	) AS source
		ON (target.BusinessCardID = source.ID) AND (target.BusinessCardClass = source.Class) AND (target.PhoneTypeID = source.PhoneTypeID)
		WHEN MATCHED THEN 
			UPDATE 
			SET [Phone] = CASE 
					WHEN target.[Phone] IS NULL OR target.[Phone] = '' THEN source.Phone
					ELSE target.[Phone] END
		WHEN NOT MATCHED THEN	
			INSERT ([BusinessCardID], [BusinessCardClass], [PhoneTypeID], [Phone])
			VALUES (source.ID, source.Class, source.PhoneTypeID, source.Phone);
	
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END

			
		;MERGE [Address] AS target
		USING (	SELECT
					@ID AS ID,
					@Class AS Class,
					T.c.value('AddressTypeID[1]','int') AS AddressTypeID,
					T.c.value('Street[1]','nvarchar(256)') AS Street,
					T.c.value('City[1]','nvarchar(50)') AS City,
					T.c.value('Region[1]','nvarchar(256)') AS Region,
					T.c.value('PostalCode[1]','varchar(50)') AS PostalCode,
					T.c.value('Country[1]','nvarchar(256)') AS Country,
					(CASE 
						WHEN T.c.value('Mail[1]','varchar(5)')='false' THEN 0 
						ELSE 1 
					END) AS Mail
				FROM
					@doc.nodes('/adfCompanyGet/Address') AS T(c)	) AS source
		ON (target.BusinessCardID = source.ID) AND (target.BusinessCardClass = source.Class) AND (target.AddressTypeID = source.AddressTypeID)
		WHEN MATCHED THEN 
			UPDATE 
			SET [BusinessCardID] = CASE 
					WHEN target.[BusinessCardID] IS NULL OR target.[BusinessCardID] = '' THEN source.ID
					ELSE target.[BusinessCardID] END,
				[BusinessCardClass] = CASE 
					WHEN target.[BusinessCardClass] IS NULL OR target.[BusinessCardClass] = '' THEN source.Class
					ELSE target.[BusinessCardClass] END,
				[AddressTypeID] = CASE 
					WHEN target.[AddressTypeID] IS NULL OR target.[AddressTypeID] = '' THEN source.AddressTypeID
					ELSE target.[AddressTypeID] END,
				[Street] = CASE 
					WHEN target.[Street] IS NULL OR target.[Street] = '' THEN source.Street
					ELSE target.[Street] END,
				[City] = CASE 
					WHEN target.[City] IS NULL OR target.[City] = '' THEN source.City
					ELSE target.[City] END,
				[Region] = CASE 
					WHEN target.[Region] IS NULL OR target.[Region] = '' THEN source.Region
					ELSE target.[Region] END,
				[PostalCode] = CASE 
					WHEN target.[PostalCode] IS NULL OR target.[PostalCode] = '' THEN source.PostalCode
					ELSE target.[PostalCode] END,
				[Country] = CASE 
					WHEN target.[Country] IS NULL OR target.[Country] = '' THEN source.Country
					ELSE target.[Country] END,
				[Mail] = CASE 
					WHEN target.[Mail] IS NULL THEN source.Mail
					ELSE target.[Mail] END
		WHEN NOT MATCHED THEN	
			INSERT ([BusinessCardID], [BusinessCardClass], [AddressTypeID], [Street], [City], [Region], 
				[PostalCode], [Country], [Mail])
			VALUES (source.ID, source.Class, source.AddressTypeID, source.Street, 
				source.City, source.Region, source.PostalCode, source.Country, source.Mail);
	
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
		
				
		;MERGE [Email] AS target
		USING (	SELECT
					@ID AS ID,
					@Class AS Class,
					T.c.value('EmailTypeID[1]','int') AS EmailTypeID,
					T.c.value('Email[1]','nvarchar(256)') AS Email,
					T.c.value('Email[1]','nvarchar(256)') AS DisplayAs
				FROM
					@doc.nodes('/adfCompanyGet/Email') AS T(c)	) AS source
		ON (target.BusinessCardID = source.ID) AND (target.BusinessCardClass = source.Class) AND (target.EmailTypeID = source.EmailTypeID)
		WHEN MATCHED THEN 
			UPDATE 
			SET [Email] = CASE 
					WHEN target.[Email] IS NULL OR target.[Email] = '' THEN source.Email
					ELSE target.[Email] END,
				[DisplayAs] = CASE 
					WHEN target.[DisplayAs] IS NULL OR target.[DisplayAs] = '' THEN source.DisplayAs
					ELSE target.[DisplayAs] END
		WHEN NOT MATCHED THEN	
			INSERT ([BusinessCardID], [BusinessCardClass], [EmailTypeID], [Email], [DisplayAs])
			VALUES (source.ID, source.Class, source.EmailTypeID, source.Email, source.DisplayAs);
	
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END

		COMMIT TRANSACTION	
		
	END

	RETURN 0
GO

GRANT EXECUTE ON adfBusinessCardFilling TO PUBLIC
GO