IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'adfDocumentCategoryUpdate')
	BEGIN
		PRINT 'Dropping Procedure adfDocumentCategoryUpdate'
		DROP  Procedure  adfDocumentCategoryUpdate
	END

GO

PRINT 'Creating Procedure adfDocumentCategoryUpdate'
GO
/******************************************************************************
**		File: 
**		Name: adfDocumentCategoryUpdate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		19.02.13	SAP				  OPENXML
*******************************************************************************/
CREATE Procedure adfDocumentCategoryUpdate
	@DocumentID TIdentifier,
	@DocumentClass TClass,
	@Xml XML
AS
	DECLARE @ID TIdentifier
	SET NOCOUNT ON

	DECLARE categories CURSOR LOCAL FAST_FORWARD READ_ONLY FOR
	SELECT 
		tab.col.value('Name[1]','nvarchar(256)'),
		tab.col.value('State[1]','int')
	FROM @Xml.nodes('//Item') tab(col)

	OPEN categories
	DECLARE @name TName, @state INT
	
	FETCH NEXT FROM categories INTO @name, @state
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @ID = dbo.adfGetCategoryID(@name, @DocumentClass)
		PRINT @name
		PRINT @ID
		IF @state = 2 -- added
		BEGIN
			IF NOT EXISTS(SELECT * FROM [DocumentCategory] WHERE [DocumentID] = @DocumentID AND [DocumentClass] = @DocumentClass AND [CategoryID] = @ID)
			BEGIN
				INSERT [DocumentCategory] ([DocumentID], [DocumentClass], [CategoryID])
				VALUES(@DocumentID, @DocumentClass, @ID)
			END
		END
		ELSE IF @state = 3 -- deleted
		BEGIN
			IF EXISTS(SELECT * FROM [DocumentCategory] WHERE [DocumentID] = @DocumentID AND [DocumentClass] = @DocumentClass AND [CategoryID] = @ID)
			BEGIN
				DELETE [DocumentCategory] WHERE [DocumentID] = @DocumentID AND [DocumentClass] = @DocumentClass AND [CategoryID] = @ID
			END
		END
		IF @@error != 0 GOTO ERROR_HANDLER
		FETCH NEXT FROM categories INTO @name, @state
	END
	
	CLOSE categories
	DEALLOCATE categories
	
	RETURN 0
ERROR_HANDLER:
	CLOSE categories
	DEALLOCATE categories
	
	RETURN 1
GO

GRANT EXEC ON adfDocumentCategoryUpdate TO PUBLIC
GO
 