IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'adftrDocumentUpdate')
	BEGIN
		PRINT 'Dropping Trigger adftrDocumentUpdate'
		DROP  Trigger adftrDocumentUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: adftrDocumentUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    04.12.09			SYV				   @ModifiedBy   AS OWNER   
*******************************************************************************/

PRINT 'Creating Trigger adftrDocumentUpdate'
GO
CREATE TRIGGER adftrDocumentUpdate ON [Document] FOR UPDATE
AS
	IF @@rowcount = 0 
		RETURN
	
	SET NOCOUNT ON
	
	IF UPDATE(AttachmentCount) OR UPDATE(NoteCount)
		RETURN
		
	DECLARE updatedDocuments CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
		SELECT 
			[inserted].[ID], 
			[inserted].[Class], 
			[deleted].[State] as [FromState],
			[inserted].[State] as [ToState],
			[deleted].[Flag] as [OldFlag],
			[inserted].[Flag] as [NewFlag],
			[inserted].[ModifiedBy] as [ModifiedBy]
		FROM 
			inserted INNER JOIN deleted
				ON ([inserted].[Class] = [deleted].[Class]
					AND [inserted].[ID] = [deleted].[ID])
					
	DECLARE @ID TIdentifier, @Class TClass, @FromState TState, @ToState TState,
			@TransitionTemplateID TIdentifier, @RightID TIdentifier,
			@UserID TIdentifier, @Created TDateTime, @result INT,
			@OldFlag TFlag, @NewFlag TFlag, @ModifiedBy TIdentifier,
			@IssueClass TClass, @StickerClass TClass, @AnnouncementClass TClass, @MailItemClass TClass
			
	SET @UserID = dbo.adfGetCurrentUserID()
	SET @Created = GETDATE()
	SET @IssueClass = dbo.adfGetClassByName('Pjm5.Issue')
	SET @StickerClass = dbo.adfGetClassByName('Adf.Sticker')
	SET @AnnouncementClass = dbo.adfGetClassByName('Adf.Announcement')
	SET @MailItemClass = dbo.adfGetClassByName('Adf.MailItem')

	OPEN updatedDocuments
	FETCH NEXT FROM updatedDocuments INTO @ID, @Class, @FromState, @ToState, @OldFlag, @NewFlag, @ModifiedBy
	WHILE @@FETCH_STATUS = 0
	BEGIN
		IF @OldFlag = @NewFlag		
		BEGIN
			SET @TransitionTemplateID = NULL
			SELECT
				@TransitionTemplateID = [TransitionTemplate].[ID],
				@RightID = [TransitionTemplate].[RightID]
			FROM
				[TransitionTemplate]
			WHERE
				[FromState] = @FromState AND [ToState] = @ToState AND [Class] = @Class
			
			IF @TransitionTemplateID IS NULL
			BEGIN	
				RAISERROR('#TransitionTemplateCouldNotBeFound', 16, -1)
				ROLLBACK TRANSACTION
				RETURN
			END
			
			EXEC @result = adfAccessRightCheckInternal @RightID, @UserID
			IF @result != 0
			BEGIN
				ROLLBACK TRANSACTION
				RETURN
			END

			INSERT INTO [Transition]
				([TransitionTemplateID], [Class], [DocumentID], [UserID], [Created])
			VALUES
				(@TransitionTemplateID, @Class, @ID, @UserID, @Created)
			IF @@error != 0
			BEGIN
				ROLLBACK TRANSACTION
				RETURN
			END

			IF (@Class IN (@IssueClass, @StickerClass, @AnnouncementClass, @MailItemClass))
			BEGIN
				IF EXISTS(SELECT * FROM [DocumentUserState] WHERE [DocumentClass] = @Class AND [DocumentID] = @ID  AND [UserID] = @UserID)
				BEGIN
					UPDATE [DocumentUserState] SET [Read] = 1
					WHERE [DocumentClass] = @Class AND [DocumentID] = @ID AND [UserID] = @UserID
				END
				ELSE
				BEGIN
					INSERT INTO [DocumentUserState] ([UserID], [DocumentClass], [DocumentID], [Read]) 
					VALUES(@UserID, @Class, @ID, 1)
				END
			END
		END
		FETCH NEXT FROM updatedDocuments INTO @ID, @Class, @FromState, @ToState, @OldFlag, @NewFlag, @ModifiedBy
	END
	
	CLOSE updatedDocuments
	DEALLOCATE updatedDocuments
	
	RETURN
GO

