﻿if exists (select * from dbo.sysobjects where id = object_id(N'[exch].[ContactGet]') and objectproperty(id, N'IsProcedure') = 1)
begin
	print 'Dropping procedure ContactGet'
	drop procedure [exch].[ContactGet]
end
go

print 'Creating procedure ContactGet'
go

create procedure [exch].[ContactGet] @UserID TIdentifier, @ID TIdentifier with execute as 'pjmexchangesync'
as
begin
	set nocount on;

	with [Contact] ([Class], [ID], [FileAs], [Title], [FirstName], [MiddleName], [LastName], [Suffix], [Initials], 
					[BirthDate], [BirthPlace], [LegalCode], [Position], [Company], [Division], [Qualification],
					[HireDate], [FireDate], [Comments], [Created], [Modified], [RowVersion], [ExchangeID], [SyncTime]) as
	(
		select
			[E].[Class],
			[E].[ID],
			[E].[FileAs],
			[E].[Title],
			[E].[FirstName],
			[E].[MiddleName],
			[E].[LastName],
			[E].[Suffix],
			[E].[Initials],
			[E].[BirthDate],
			[E].[BirthPlace],
			[E].[LegalCode],
			[P].[FileAs] as [Position],
			[C].[FileAs] as [Company],
			[D].[FileAs] as [Division],
			[Q].[FileAs] as [Qualification],
			[E].[HireDate],
			[E].[FireDate],
			[E].[Comments],
			[E].[Created],
			[E].[Modified],
			cast([E].[Timestamp] as bigint) as [RowVersion],
			[L].[ExchangeID],
			[L].[SyncTime]
		from 
			[dbo].[Employee] as [E] left outer join [dbo].[Position] as [P]
				on ([E].[PositionID] = [P].[ID])
			left outer join [dbo].[Company] as [C]
				on ([E].[CompanyID] = [C].[ID])
			left outer join [dbo].[Division] as [D]
				on ([E].[DivisionID] = [D].[ID])
			left outer join [dbo].[Qualification] as [Q]
				on ([E].[QualificationID] = [Q].[ID])
			left outer join [exch].[ExchangeLink] as [L]
				on ([E].[ID] = [L].[DocumentID] and [E].[Class] = [L].[DocumentClass] and [L].[UserID] = @UserID)
		where
			[E].[ID] = @ID
	),
	[Category] ([ID], [Category]) as
	(
		select
			[E].[ID],
			[C].[FileAs]
		from
			[dbo].[Employee] as [E] left outer join [dbo].[DocumentCategory] as [DC]
				on([E].[Class] = [DC].[DocumentClass] and [E].[ID] = [DC].DocumentID)
			left outer join [dbo].[Category] as [C]
				on ([DC].[CategoryID] = [C].[ID])
		where
			[E].[ID] = @ID
	),
	[Reminder] ([ID], [EventTime], [RowVersion]) as
	(
		select
			[R].[DocumentID],
			min([R].[EventTime]),
			max(cast([UR].[Timestamp] as bigint))
		from
			[dbo].[Employee] as [E] inner join [dbo].[Reminder] as [R] 
				on ([R].[DocumentClass] = [E].[Class] and [R].[DocumentID] = [E].[ID])
			inner join [UserReminder] as [UR]
				ON ([UR].[ReminderID] = [R].[ID] and [UR].[UserID] = @UserID)
		where
			[UR].[State] = 1
		and
			[R].[EventTime] > getdate()
		group by
			[R].[DocumentID]
	),
	[ExtendedProperties] ([ID], [Picture], [PictureHash], [Gender]) as
	(
		select
			[EP].[ID],
			[EP].[ExtendedProperties].value('(/ExtendedProperties/Values/Picture)[1]', 'varbinary(max)'),
			[EP].[ExtendedProperties].value('(/ExtendedProperties/Values/PictureHash)[1]', 'varbinary(max)'),
			[EP].[ExtendedProperties].value('(/ExtendedProperties/Values/Gender)[1]', 'varchar(2)')
		from
			[dbo].[Employee] as [E]	inner join [dbo].[ExtendedProperties] as [EP]
				on ([E].[Class] = [EP].[Class] and [E].[ID] = [EP].[ID])
		where
			[E].[ID] = @ID
	),
	[Address] ([ID], [TypeID], [Street], [City], [Region], [PostalCode], [Country], [Mail]) as 
	(
	select
		[A].[BusinessCardID],
		[A].[AddressTypeID],
		[A].[Street],
		[A].[City],
		[A].[Region],
		[A].[PostalCode],
		[A].[Country],
		[A].[Mail]
	from
		[dbo].[Employee] as [E] inner join [dbo].[Address] as [A]
			on ([E].[Class] = [A].[BusinessCardClass] and [E].[ID] = [A].[BusinessCardID])
	where
		[E].[ID] = @ID and [A].[AddressTypeID] in (1, 3, 4)
	),
	[Phone] ([ID], [TypeID], [Phone]) as 
	(
		select
			[P].[BusinessCardID] as [ID],
			[P].[PhoneTypeID],
			[P].[Phone]
		from
			[dbo].[Employee] as [E] inner join [dbo].[Phone] as [P]
				on ([E].[Class] = [P].[BusinessCardClass] and [E].[ID] = [P].[BusinessCardID])
		where
			[E].[ID] = @ID
	),
	[Email] ([ID], [TypeID], [Email], [DisplayAs]) as 
	(
		select
			[EM].[BusinessCardID],
			[EM].[EmailTypeID],
			[EM].[Email],
			[EM].[DisplayAs]
		from
			[dbo].[Employee] as [E] inner join [dbo].[Email] as [EM]
				on ([E].[Class] = [EM].[BusinessCardClass] and [E].[ID] = [EM].[BusinessCardID])
		where
			[E].[ID] = @ID
	)
	select
		[Contact].[ID], 
		[Contact].[FileAs], 
		[Contact].[Title], 
		[Contact].[FirstName], 
		[Contact].[MiddleName], 
		[Contact].[LastName], 
		[Contact].[Suffix], 
		[Contact].[Initials], 
		[Contact].[BirthDate], 
		[Contact].[Position], 
		[Contact].[Company], 
		[Contact].[Division], 
		[Contact].[Qualification],
		[Contact].[Comments], 
		[Contact].[Created], 
		[Contact].[Modified], 
		(case when [Contact].[RowVersion] > isnull([Reminder].[RowVersion], 0) then [Contact].[RowVersion] else [Reminder].[RowVersion] end) as [RowVersion],
		[Contact].[ExchangeID],
		[Contact].[SyncTime],
		(select	
			[Phone].[TypeID],
			[Phone].[Phone]
		from
			[Phone]
		where
			[Phone].[ID] = [Contact].[ID]
		for xml auto, type),
		(select	
			[Email].[TypeID],
			[Email].[Email]
		from
			[Email]
		where
			[Email].[ID] = [Contact].[ID]
		for xml auto, type),
		(select	
			[Address].[TypeID],
			[Address].[Street],
			[Address].[City], 
			[Address].[Region], 
			[Address].[PostalCode], 
			[Address].[Country], 
			[Address].[Mail]
		from
			[Address]
		where
			[Address].[ID] = [Contact].[ID]
		for xml auto, type),
		(select 
			[Category].[Category]
		from
			[Category]
		where	
			[Category].[ID] = [Contact].[ID]
		for xml auto, type),
		(select
			[Reminder].[EventTime]
		from
			[Reminder]
		where
			[Reminder].[ID] = [Contact].[ID]
		for xml auto, type),
		(select	
			[ExtendedProperties].[Picture],
			[ExtendedProperties].[PictureHash],
			[ExtendedProperties].[Gender]
		from	
			[ExtendedProperties]
		where
			[ExtendedProperties].[ID] = [Contact].[ID]
		for xml auto, type, binary base64, elements)
	from	
		[Contact] left outer join [ExtendedProperties]	
			on ([Contact].[ID] = [ExtendedProperties].[ID])
		left outer join [Reminder]
			on ([Contact].[ID] = [Reminder].[ID])
	for xml auto;

	return 0;
end
go

grant execute on [exch].[ContactGet] to [projectmateservice]

go

