IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnPjm5RecurrenceGetNextDateTime]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnPjm5RecurrenceGetNextDateTime'
		DROP FUNCTION [dbo].[fnPjm5RecurrenceGetNextDateTime]
	END

GO

PRINT 'Creating function fnPjm5RecurrenceGetNextDateTime'
GO
/******************************************************************************
**		File: 
**		Name: fnPjm5RecurrenceGetNextDateTime
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		28.07.10	SYV				       @@DATEFIRST
**									 @RecurrenceType = 1  
*******************************************************************************/
CREATE FUNCTION fnPjm5RecurrenceGetNextDateTime(
	@RecurrenceID	TIdentifier,
	@Datetime		TDatetime,
	@CurrentDate	TDatetime
	)
RETURNS TDatetime
AS
BEGIN
		
	DECLARE 
		@Result TDatetime
		
	SET @Datetime = ISNULL(@Datetime, @CurrentDate)
	DECLARE -- RECURRENCE
		@StartDate TDate, 
		@EndDate TDate, 
		@RecurrenceType TIdentifier, 
		@Occurrences TCount,
		@Interval TCount, 
		@DayOfWeekMask TIdentifier, 
		@Instance TIdentifier, 
		@MonthOfYear TCount, 
		@DayOfMonth TCount
		
	SELECT
		@StartDate = [StartDate], 
		@EndDate = [EndDate], 
		@RecurrenceType = [RecurrenceType], 
		@Occurrences = [Occurrences],
		@Interval = [Interval], 
		@DayOfWeekMask = [DayOfWeekMask], 
		@Instance = [Instance], 
		@MonthOfYear = [MonthOfYear], 
		@DayOfMonth = [DayOfMonth]
	FROM
		[RecurrencePattern]
	WHERE
		[ID] = @RecurrenceID
		
	IF @StartDate > @CurrentDate
		RETURN NULL
	IF @Occurrences IS NOT NULL AND @Occurrences = 0
		RETURN NULL
	IF @EndDate IS NOT NULL
	BEGIN
		IF @EndDate <= @CurrentDate
			RETURN NULL
		IF @EndDate <= @Datetime
			RETURN NULL
	END
		
	DECLARE
		@weekday int,
		@w int,
		@p int,
		@d datetime
	DECLARE 
		@delta int,
		@count int
		
	IF @RecurrenceType = 0
	BEGIN
		IF @Interval != 0
			SET @Result = DATEADD(day, @Interval, @Datetime)
		ELSE
		BEGIN
			SET @Result = DATEADD(day, 1, @Datetime)
			SET @Result = dbo.fnPjm5AddWorkingDays(@Result, 1)
			SET @Result = DATEADD(hour, DATEPART(hour, @Datetime), @Result)
			SET @Result = DATEADD(minute, DATEPART(minute, @Datetime), @Result)
		END
	END
	ELSE IF @RecurrenceType = 1
	BEGIN
		IF @DayOfWeekMask = 0
			RETURN NULL
			
		SET @weekday = DATEPART(weekday, @Datetime)
		
		IF (@@DATEFIRST != 7)
		BEGIN
			SET @weekday = (@weekday + @@DATEFIRST) % 7
		END
		
		SET @w = @weekday
		
		IF @weekday = 7
			SET @weekday = 1
		ELSE
			SET @weekday = @weekday + 1
		WHILE @DayOfWeekMask != @DayOfWeekMask | POWER(2, @weekday - 1)
		BEGIN
			IF @weekday = 7
				SET @weekday = 1
			ELSE
				SET @weekday = @weekday + 1
		END

		SET @Result = @Datetime
		IF @w = 1 
		BEGIN
			SET @Result = DATEADD(week, @Interval, @Result)
			IF @weekday != 1
				SET @Result = DATEADD(day, @weekday - 8, @Result)
		END
		ELSE IF @w < @weekday OR @weekday = 1
		BEGIN
			IF @weekday = 1
				SET @Result = DATEADD(day, 8 - @w, @Result)
			ELSE
				SET @Result = DATEADD(day, @weekday - @w, @Result)
		END
		ELSE
		BEGIN
			SET @Result = DATEADD(week, @Interval, @Result)
			IF @weekday != 1
				SET @Result = DATEADD(day, @weekday - @w, @Result)
		END
	END
	ELSE IF @RecurrenceType = 2
	BEGIN
		SET @Result = DATEADD(day, 1 - DAY(@Datetime), @Datetime)
		SET @Result = DATEADD(month, @Interval, @Result)
		IF @DayOfMonth > DAY(DATEADD(day, -1, DATEADD(month, 1, @Result)))
			SET @DayOfMonth = DAY(DATEADD(day, -1, DATEADD(month, 1, @Result)))
		SET @Result = DATEADD(day, @DayOfMonth - 1, @Result)
	END
	ELSE IF @RecurrenceType = 3
	BEGIN
		SET @Result = DATEADD(day, 1 - DAY(@Datetime), @Datetime)
		SET @Result = DATEADD(month, @Interval, @Result)
		
		IF @Instance = 5 
		BEGIN
			SET @delta = -1 
			SET @count = 1
			SET @Result = DATEADD(day, -1, DATEADD(month, 1, @Result))
		END
		ELSE 
		BEGIN
			SET @delta = 1
			SET @count = @Instance
		END

		SET @weekday = DATEPART(weekday, @Result)
		WHILE @count != 0
		BEGIN
			WHILE @DayOfWeekMask != @DayOfWeekMask | POWER(2, @weekday - 1) 
			BEGIN
				SET @Result = DATEADD(day, @delta, @Result)
				SET @weekday = DATEPART(weekday, @Result)
			END
			IF @count > 1
			BEGIN
				SET @Result = DATEADD(day, @delta, @Result)
				SET @weekday = DATEPART(weekday, @Result)
			END
			SET @count = @count -1
		END
	END
	ELSE IF @RecurrenceType = 5
	BEGIN
		SET @Result = DATEADD(day, 1 - DAY(@Datetime), @Datetime)
		SET @Result = DATEADD(month, @MonthOfYear - MONTH(@Datetime), @Result)
		
		IF @DayOfMonth > DAY(DATEADD(day, -1, DATEADD(month, 1, @Result)))
			SET @DayOfMonth = DAY(DATEADD(day, -1, DATEADD(month, 1, @Result)))
		SET @Result = DATEADD(day, @DayOfMonth - 1, @Result)
		IF @Result <= @Datetime
			SET @Result = DATEADD(year, 1, @Result)
	END
	ELSE IF @RecurrenceType = 6
	BEGIN
		SET @Result = DATEADD(year, -1, @Datetime)
		WHILE (@Result <= @Datetime)
		BEGIN	
			SET	@Result = DATEADD(year, 1, @Result)		
			SET @Result = DATEADD(day, 1 - DAY(@Result), @Result)
			SET @Result = DATEADD(month, @MonthOfYear - MONTH(@Result), @Result)
			IF @Instance = 5 
			BEGIN
				SET @delta = -1 
				SET @count = 1
				SET @Result = DATEADD(day, -1, DATEADD(month, 1, @Result))
			END
			ELSE 
			BEGIN
				SET @delta = 1
				SET @count = @Instance
			END
			SET @weekday = DATEPART(weekday, @Result)
						
			WHILE @count != 0
				BEGIN
					WHILE @DayOfWeekMask != @DayOfWeekMask | POWER(2, @weekday - 1) 
				BEGIN
					SET @Result = DATEADD(day, @delta, @Result)
					SET @weekday = DATEPART(weekday, @Result)
				END
				IF @count > 1
				BEGIN
					SET @Result = DATEADD(day, @delta, @Result)
					SET @weekday = DATEPART(weekday, @Result)
				END
				SET @count = @count -1
			END
		END
	END
	RETURN @Result
END
