IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnReportGetDocumentInFinance]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnReportGetDocumentInFinance'
		DROP FUNCTION [dbo].[fnReportGetDocumentInFinance]
	END
GO

PRINT 'Creating function fnReportGetDocumentInFinance'
GO

/******************************************************************************
**		File: fnReportGetDocumentInFinance.sql
**		Name: fnReportGetDocumentInFinance
**
**              
**		Return values:
**
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAN
**		Date: 26/08/2010 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		26.08.2010	MAN				First Version
*******************************************************************************/
CREATE FUNCTION dbo.fnReportGetDocumentInFinance
(
	@DocumentClass TIdentifier,
	@PeriodStart TDateTime,
	@PeriodEnd TDateTime,
	@ProjectXML XML
)
RETURNS @Result TABLE
(
	DocumentID TIdentifier,
	LinkClass	   TClass,
	LinkDocumentID TIdentifier,
	State	TState,
	Date	TDateTime,
	ThroughCalculation TFlag
)
BEGIN
	DECLARE
		@CalculationClass TClass,
		@StatementClass TClass,
		@InvoiceClass TClass,
		@TimeEntryClass TClass,
		@ExpenseClass TClass,
		@ApprovedState TState
		
	SET	@CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET	@StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
	SET	@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET	@TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	SET	@ExpenseClass = dbo.adfGetClassByName('Pjm5.Expense')

	SET @ApprovedState = 
		CASE 
			WHEN @DocumentClass =  @TimeEntryClass OR @DocumentClass = @ExpenseClass THEN 6
		END
		
	DECLARE @DocumentApproved TABLE
	(
		ID TIdentifier
	)

	DECLARE @DocumentInCalculation TABLE
	(
		DocumentID		 TIdentifier,
		CalculationID	 TIdentifier,
		CalculationState TState,
		CalculationDate  TDateTime
	)
	DECLARE @DocumentInStatement TABLE
	(
		DocumentID		TIdentifier,
		StatementID		TIdentifier,
		StatementState	TState,
		StatementDate   TDateTime
	)
	DECLARE @DocumentInInvoice TABLE
	(
		DocumentID		TIdentifier,
		InvoiceID		TIdentifier,
		InvoiceState	TState,
		InvoiceDate		TDateTime
	)
	DECLARE @DocumentInCalculationThroughCalculation TABLE
	(
		DocumentID		 TIdentifier,
		CalculationID	 TIdentifier,
		CalculationState TState,
		CalculationDate  TDateTime
	)
	DECLARE @DocumentInStatementThroughCalculation TABLE
	(
		DocumentID		TIdentifier,
		StatementID		TIdentifier,
		StatementState	TState,
		StatementDate   TDateTime
	)
	DECLARE @DocumentInInvoiceThroughCalculation TABLE
	(
		DocumentID		TIdentifier,
		InvoiceID		TIdentifier,
		InvoiceState	TState,
		InvoiceDate		TDateTime
	)
	IF @DocumentClass = @TimeEntryClass

		INSERT INTO @DocumentApproved( ID )
		SELECT 
			[vwTimeEntry].[ID]
		FROM
			[vwTimeEntry]
		WHERE
			[vwTimeEntry].[State] = @ApprovedState
			AND
			[vwTimeEntry].[StartTime] >= @PeriodStart
			AND
			[vwTimeEntry].[StartTime] < @PeriodEnd
			AND
			[vwTimeEntry].[ProjectID] IN
			(
				SELECT
					Projects.value('@ProjectID','TIdentifier') AS [ProjectID]
				FROM
					@ProjectXML.nodes('/row') AS Projects(Projects)
			)
	ELSE IF @DocumentClass = @ExpenseClass
		INSERT INTO @DocumentApproved( ID )
		SELECT 
			[Expense].[ID]
		FROM
			[Expense]
		WHERE
			[Expense].[State] = @ApprovedState
			AND
			[Expense].[Date] > @PeriodStart
			AND
			[Expense].[Date] < @PeriodEnd
			AND
			[Expense].[ProjectID] IN
			(
				SELECT
					Projects.value('@ProjectID','TIdentifier') AS [ProjectID]
				FROM
					@ProjectXML.nodes('/row') AS Projects(Projects)
			)

	INSERT INTO @DocumentInCalculation ( DocumentID, CalculationID,	CalculationState, CalculationDate )
	SELECT
		[CalculationLine].[DocumentID],
		[Calculation].[ID],
		[Calculation].[State],
		[Calculation].[Date]
	FROM
		[CalculationLine]
		INNER JOIN [Calculation]
			ON [Calculation].[ID] = [CalculationLine].[CalculationID]
	WHERE
		[CalculationLine].[DocumentClass] = @DocumentClass
		AND
		[CalculationLine].[DocumentID] IN
		(
			SELECT
				[@DocumentApproved].[ID]
			FROM
				@DocumentApproved
			WHERE
				[@DocumentApproved].[ID] = [CalculationLine].[DocumentID]
		)

	INSERT INTO @DocumentInStatement( DocumentID, StatementID, StatementState, StatementDate )
	SELECT
		[StatementLine].[DocumentID],
		[Statement].[ID],
		[Statement].[State],
		[Statement].[Date]
	FROM
		[StatementLine]
		INNER JOIN [Statement]
			ON [Statement].[ID] = [StatementLine].[StatementID]
	WHERE
		[StatementLine].[DocumentClass] = @DocumentClass
		AND
		[StatementLine].[DocumentID] IN
		(
			SELECT
				[@DocumentApproved].[ID]
			FROM
				@DocumentApproved
		)
		
	INSERT INTO @DocumentInInvoice ( DocumentID,InvoiceID,InvoiceState, InvoiceDate )
	SELECT
		[InvoiceLine].[DocumentID],
		[Invoice].[ID],
		[Invoice].[State],
		[Invoice].[Date]
	FROM
		[InvoiceLine]
		INNER JOIN [Invoice]
			ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
	WHERE
		[InvoiceLine].[DocumentClass] = @DocumentClass
		AND
		[InvoiceLine].[DocumentID] IN
		(
			SELECT
				[@DocumentApproved].[ID]
			FROM
				@DocumentApproved
			WHERE
				[@DocumentApproved].[ID] = [InvoiceLine].[DocumentID]
		)

	INSERT INTO @DocumentInCalculationThroughCalculation ( DocumentID, CalculationID, CalculationState, CalculationDate)
	SELECT
		[@DocumentInCalculation].[DocumentID],
		[Calculation].[ID],
		[Calculation].[State],
		[Calculation].[Date]
	FROM
		[CalculationAdvance]
		INNER JOIN [Calculation]
			ON [CalculationAdvance].[CalculationID] = [Calculation].[ID]
		INNER JOIN @DocumentInCalculation
			ON [@DocumentInCalculation].[CalculationID] = [CalculationAdvance].[AdvanceCalculationID]
	WHERE
		[@DocumentInCalculation].[CalculationID] IN
		(	
			SELECT
				[CalculationLine].[CalculationID]
			FROM
				[CalculationLine]
			WHERE
				[CalculationLine].[DocumentClass] = @DocumentClass
		)
		
	INSERT INTO @DocumentInInvoiceThroughCalculation ( DocumentID, InvoiceID, InvoiceState, InvoiceDate )
	SELECT
		[@DocumentInCalculation].[DocumentID],
		[Invoice].[ID],
		[Invoice].[State],
		[Invoice].[Date]
	FROM
		[InvoiceLine]
		INNER JOIN @DocumentInCalculation
			ON [@DocumentInCalculation].[CalculationID] = [InvoiceLine].[DocumentID] AND [InvoiceLine].[DocumentClass] = @CalculationClass
		INNER JOIN [Invoice]
			ON [Invoice].[ID] = [InvoiceLine].[InvoiceID]
			
	INSERT INTO @DocumentInInvoiceThroughCalculation( DocumentID, InvoiceID, InvoiceState,InvoiceDate )
	SELECT
		[@DocumentInCalculationThroughCalculation].[DocumentID],
		[Invoice].[ID],
		[Invoice].[State],
		[Invoice].[Date]
	FROM
		[InvoiceLine]
		INNER JOIN @DocumentInCalculationThroughCalculation
			ON [@DocumentInCalculationThroughCalculation].[CalculationID] = [InvoiceLine].[DocumentID] AND [InvoiceLine].[DocumentClass] = @CalculationClass
		INNER JOIN [Invoice]
			ON 	[Invoice].[ID] = [InvoiceLine].[InvoiceID]
			
	INSERT INTO @DocumentInStatementThroughCalculation ( DocumentID,StatementID,StatementState,StatementDate )
	SELECT
		[@DocumentInCalculation].[DocumentID],
		[Statement].[ID],
		[Statement].[State],
		[Statement].[Date]
	FROM
		[StatementLine]
		INNER JOIN @DocumentInCalculation
			ON [@DocumentInCalculation].[CalculationID] = [StatementLine].[DocumentID] AND [StatementLine].[DocumentClass] = @CalculationClass
		INNER JOIN [Statement]
			ON [Statement].[ID] = [StatementLine].[StatementID]

	INSERT INTO @DocumentInStatementThroughCalculation(	DocumentID,StatementID,StatementState,StatementDate )
	SELECT
		[@DocumentInCalculationThroughCalculation].[DocumentID],
		[Statement].[ID],
		[Statement].[State],
		[Statement].[Date]
	FROM
		[StatementLine]
		INNER JOIN @DocumentInCalculationThroughCalculation
			ON [@DocumentInCalculationThroughCalculation].[CalculationID] = [StatementLine].[DocumentID] AND [StatementLine].[DocumentClass] = @CalculationClass
		INNER JOIN [Statement]
			ON [Statement].[ID] = [StatementLine].[StatementID]
	--  
	INSERT INTO @Result
	(
		DocumentID,
		LinkClass,
		LinkDocumentID,
		State,
		Date,
		ThroughCalculation
	)
	SELECT
		[@DocumentInCalculation].[DocumentID] AS [DocumentID], 
		@CalculationClass AS [LinkClass],
		[@DocumentInCalculation].[CalculationID] AS [LinkDocumentID],
		[@DocumentInCalculation].[CalculationState] AS [State],
		[@DocumentInCalculation].[CalculationDate] AS [Date],
		0 AS [ThroughCalculation]
	FROM
		@DocumentInCalculation
	UNION
	SELECT
		[@DocumentInStatement].[DocumentID],
		@StatementClass,
		[@DocumentInStatement].[StatementID],
		[@DocumentInStatement].[StatementState],
		[@DocumentInStatement].[StatementDate],
		0
	FROM
		@DocumentInStatement
	UNION
	SELECT
		[@DocumentInInvoice].[DocumentID],
		@InvoiceClass,
		[@DocumentInInvoice].[InvoiceID],
		[@DocumentInInvoice].[InvoiceState],
		[@DocumentInInvoice].[InvoiceDate],
		0
	FROM
		@DocumentInInvoice
	UNION
	SELECT
		[@DocumentInCalculationThroughCalculation].[DocumentID],
		@CalculationClass,
		[@DocumentInCalculationThroughCalculation].[CalculationID],
		[@DocumentInCalculationThroughCalculation].[CalculationState],
		[@DocumentInCalculationThroughCalculation].[CalculationDate],
		1
	FROM
		@DocumentInCalculationThroughCalculation
	UNION
	SELECT
		[@DocumentInStatementThroughCalculation].[DocumentID],
		@StatementClass,
		[@DocumentInStatementThroughCalculation].[StatementID],
		[@DocumentInStatementThroughCalculation].[StatementState],
		[@DocumentInStatementThroughCalculation].[StatementDate],
		1
	FROM
		@DocumentInStatementThroughCalculation
	UNION
	SELECT
		[@DocumentInInvoiceThroughCalculation].[DocumentID],
		@InvoiceClass,
		[@DocumentInInvoiceThroughCalculation].[InvoiceID],
		[@DocumentInInvoiceThroughCalculation].[InvoiceState],
		[@DocumentInInvoiceThroughCalculation].[InvoiceDate],
		1
	FROM
		@DocumentInInvoiceThroughCalculation

RETURN
END
GO