﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[fnReportTimeSellAnalysisByEmployeesGetEmployeesList]') AND xtype IN (N'FN', N'IF', N'TF'))
	BEGIN
		PRINT 'Dropping function fnReportTimeSellAnalysisByEmployeesGetEmployeesList'
		DROP FUNCTION [dbo].[fnReportTimeSellAnalysisByEmployeesGetEmployeesList]
	END
GO

PRINT 'Creating function fnReportTimeSellAnalysisByEmployeesGetEmployeesList'
GO

/******************************************************************************
**		File: fnReportTimeSellAnalysisByEmployeesGetEmployeesList.sql
**		Name: fnReportTimeSellAnalysisByEmployeesGetEmployeesList
**
**              
**		Return values:
**
**		Called by:   
**              ReportTimeSellAnalysisByEmployees
**		Parameters:
**		Input							Output
**      ----------						-----------
**		
**		
**		Auth: Sergey Pelevin
**		Date: 25.07.14 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		
*******************************************************************************/
CREATE FUNCTION dbo.fnReportTimeSellAnalysisByEmployeesGetEmployeesList
(
	@OwnCompanyID int, --ID собственной компании (NULL-все)
	@DivisionID int, --ID подразделения (NULL - все)
	@QualificationID int, --ID квалификации (NULL - все)
	@EmployeeState int, --Статус сотрудников (0 - работающие, 1 - все, вкл. уволенных)
	@EmployeeSelector int, --Сотрудники (0 - все, 1 - Вкл. в список, 2 - Кроме вкл. в список)
	@EmployeeXml	xml, --Список сотрудников	
	@PeriodStart date, -- Начало периода
	@PeriodEnd date, --Конец периода
	@IncludingSentToApprove bit -- Включать отправленное на утверждение (0 - не включать, 1 - включать)	
)
RETURNS TABLE
AS
RETURN
(	
	SELECT
		[E].[ID] AS [EmployeeID],
		[D].[ID] AS [DivisionID],
		[Q].[ID] AS [QualificationID]
	FROM
		[Company] AS [C]
		--?????
		--INNER JOIN [Project] AS [P]
		--	ON [C].[ID] = [P].[SupplierCompanyID]		
		INNER JOIN [Employee] AS [E]
			ON [E].[CompanyID] = [C].[ID]
		INNER JOIN [dbo].[fnReportEmployeesWithTimeInBillableProjectsInPeriod]
		(
			@OwnCompanyID,
			@DivisionID,
			@QualificationID,
			@PeriodStart,
			@PeriodEnd,
			@IncludingSentToApprove
		) AS [EWTIBPIP]
			ON [E].[ID] = [EWTIBPIP].[ID]
		LEFT JOIN @EmployeeXml.nodes('/Employees/ID') AS T(c)
			ON T.c.value('.[1]', 'int') = [E].[ID]
		INNER JOIN [Division] AS [D]
			ON [D].[ID] = [E].[DivisionID]
		INNER JOIN [Qualification] AS [Q]
			ON [Q].[ID] = [E].[QualificationID]
	WHERE		
		--Статус сотрудника
		(
			(@EmployeeState = 0 AND [E].[State] = 1)
			OR
			@EmployeeState = 1
		)
		AND
		--Фильтр по списку сотрудников
		(
			@EmployeeSelector = 0 
			OR 
			(@EmployeeSelector = 1 AND T.c.value('.[1]', 'int') IS NOT NULL)
			OR 
			(@EmployeeSelector = 2 AND T.c.value('.[1]', 'int') IS NULL)
		)
		AND 
		--Флаг "Исключать из отчетов"
		(
			[E].[HideInReports] = 0
		)	
)
GO