IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5AssignmentCreateInternal]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5AssignmentCreateInternal'
		DROP PROCEDURE [dbo].[pjm5AssignmentCreateInternal]
	END
GO

PRINT 'Creating procedure pjm5AssignmentCreateInternal'
GO

/******************************************************************************
**		File: pjm5AssignmentCreateInternal.sql
**		Name: pjm5AssignmentCreateInternal
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Andrew.Maklakov
**		Date: 01/20/2005 13:18:25
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		28.01.10	SYV				 Rate     1,2,5
**		03.03.10	SYV				 Rate   pjm6RateAssignmentUpdate
**		29.03.13	SOV				  -   ,
**									   : "   /" 
**									 "     "
**		02.04.13	SOV				  -  ,
**									   : " "
**		08.05.13	SOV				   " ", 
**									     ,
**									       
*******************************************************************************/
CREATE PROCEDURE pjm5AssignmentCreateInternal
	@ID TIdentifier OUTPUT,
	@Flag int,
	@State TState,
	@FileAs TName,
	@EmployeeID TIdentifier,
	@ProjectID TIdentifier,
	@QualificationID TIdentifier,
	@StandardRate TMoney,
	@OvertimeRate TMoney,
	@RateVisible TBool,
	@Approver TBool,
	@Administrator TBool,
	@FinanceManagement TBool,
	@TaskReport TBool,
	@IssueReport TBool,
	@Comments TNotes,
	@AlwaysUseRate TBool = 0
AS
	SET NOCOUNT ON
	DECLARE @ActivityTypeID TIdentifier,
			@Billable TIdentifier,
			@Actively TIdentifier,
			@prevStandardRate TMoney,
			@prevOvertimeRate TMoney,
			@prevAlwaysUseRate TBool,
			@ViewFinance TBool,
			@ViewAllFinance TBool,
			@UserID TIdentifier
	
	SET @UserID = dbo.fnPjm5GetEmployeeUserID(@EmployeeID)
	SET @ViewFinance = [dbo].[adfHasAccessRightByNameForUser]('ViewFinance', @UserID)
	SET @ViewAllFinance = [dbo].[adfHasAccessRightByNameForUser]('ViewAllFinance', @UserID)
	
	IF(@ViewFinance=0 AND @ViewAllFinance = 0 AND @FinanceManagement=1)
	BEGIN
		SET @FinanceManagement=0
	END

	SELECT TOP(1)
		@prevStandardRate = [Assignment].[StandardRate],
		@prevOvertimeRate = [Assignment].[OvertimeRate],
		@prevAlwaysUseRate = [Assignment].[AlwaysUseRate]
	FROM 
		[Assignment]
		INNER JOIN [AssignmentEntry]
			ON [Assignment].[ID] = [AssignmentEntry].[AssignmentID]	
	WHERE
		[AssignmentEntry].[EmployeeID] = @EmployeeID
		AND [Assignment].[ProjectID] = @ProjectID
		AND [Assignment].[State] = 2
	ORDER BY [Assignment].[Modified] DESC
		
	IF (dbo.fnPjm7IsManagerAssistant(@ProjectID, @EmployeeID) = 1)
	BEGIN
		SET @Approver=1
		SET @Administrator=1
		SET @TaskReport = 1
		SET @IssueReport = 1
	END

	IF (dbo.fnPjm7IsFinanceManager(@ProjectID, @EmployeeID) = 1)
		SET @FinanceManagement = 1

	INSERT INTO [Assignment]
		(
		[Flag], 
		[State], 
		[FileAs], 
		[ProjectID], 
		[QualificationID], 
		[StandardRate], 
		[OvertimeRate], 
		[RateVisible],
		[Approver], 
		[Administrator],
		[FinanceManagement],
		[ReceiveTaskStatusReports],
		[ReceiveIssueStatusReports],
		[Comments],
		[AlwaysUseRate]
		)
	VALUES
		(
		@Flag, 
		@State, 
		@FileAs, 
		@ProjectID, 
		@QualificationID, 
		ISNULL(@StandardRate, 0), 
		ISNULL(@OvertimeRate, 0), 
		@RateVisible,
		@Approver, 
		@Administrator,
		@FinanceManagement,
		@TaskReport,
		@IssueReport, 
		@Comments,
		@AlwaysUseRate
		)		
	IF @@ERROR != 0
	BEGIN
		RETURN 1
	END
	SET @ID = SCOPE_IDENTITY()
	
	IF @EmployeeID IS NOT NULL
	BEGIN
		INSERT INTO [AssignmentEntry]
			([EmployeeID], [AssignmentID], [ValidFrom])
		VALUES
			(@EmployeeID, @ID, CAST('17530101' AS datetime))
		IF @@ERROR != 0
		BEGIN
			RETURN 1
		END
	END
	
	IF @QualificationID IS NOT NULL
	BEGIN
		IF @ActivityTypeID IS NOT NULL AND
			NOT EXISTS (SELECT * FROM [ProjectActivityType] WHERE [ProjectID] = @ProjectID AND [ActivityTypeID] = @ActivityTypeID)
		BEGIN
			INSERT INTO [ProjectActivityType] 
				([ProjectID], [ActivityTypeID], [Billable], [Actively])
			VALUES 
				(@ProjectID, @ActivityTypeID, ISNULL(@Billable, 0), @Actively)
			IF @@ERROR != 0
			BEGIN
				RETURN 1
			END
		END
	END
	
	IF @StandardRate != @prevStandardRate OR @OvertimeRate != @prevOvertimeRate OR @AlwaysUseRate != @prevAlwaysUseRate
	BEGIN
		EXEC pjm6RateAssignmentUpdate @ID, @prevStandardRate, @prevOvertimeRate, @prevAlwaysUseRate
	END
	
	RETURN 0
GO

GRANT EXECUTE ON pjm5AssignmentCreateInternal TO PUBLIC
GO
