﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm6BudgetBrowse]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm6BudgetBrowse'
		DROP PROCEDURE [dbo].[pjm6BudgetBrowse]
	END
GO

PRINT 'Creating procedure pjm6BudgetBrowse'
GO

/******************************************************************************
**		File: BudgetBrowse.sql
**		Name: pjm6BudgetBrowse
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Anton.Paramonov
**		Date: 11/01/2011 11:01:32
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**      03.12.12	MAB				Добавление дат по проекту
**		22.03.13	MAB				Добавление новых полей по закупкам
**		26.03.13	MAB				Добавление поля "Плановая рентабельность"
*******************************************************************************/
CREATE PROCEDURE [pjm6BudgetBrowse]
	@BudgetTypeSelector			TIdentifier,
	@StateSelector				TIdentifier,
	@ProjectManagerEmployeeID	TIdentifier,
	@DateSelector				TIdentifier,
	@PeriodStart				TDate,
	@PeriodEnd					TDate,
	@CostCenterSelector			TIdentifier,
	@CostCenterID				TIdentifier
AS
	DECLARE
		@Lang			TLanguage,
		@Now			TDate

	SET @Now = GETDATE()
	IF @DateSelector = 1
		SET @PeriodStart = CAST(CAST(YEAR(@Now) AS varchar(4)) + '0101' AS datetime)
	ELSE IF @DateSelector = 2
		SET @PeriodStart = CAST(CAST(YEAR(@Now)-1 AS varchar(4)) + '0101' AS datetime)
	ELSE IF @DateSelector = 3
		SET @PeriodStart = DATEADD(day, 1-DATEPART(dayofyear, @PeriodStart), @PeriodStart)
	SET @PeriodStart = CONVERT(datetime, CONVERT(varchar(8), @PeriodStart, 112), 112)
	SET @PeriodEnd = DATEADD(year, 1, @PeriodStart)

	SET @Lang = dbo.adfGetLanguage()

	SELECT
		[Budget].[Class],
		[Budget].[ID],
		[Budget].[State],
		[Budget].[Flag],
		CASE @Lang
			WHEN 'en' THEN 'Budget for project ''' + [Project].[Number] + '''' ELSE
			'Бюджет по проекту ''' + [Project].[Number] + '''' END as [FileAs],
		[Budget].[WarningFlagID],
		[BudgetWarningFlag].[WarningType],
		[Project].[Number] AS [ProjectCode],
		[Project].[FileAs] AS [ProjectFileAs],
		[Project].[ProjectManagerEmployeeID],
		[Budget].[BudgetTypeID],
		dbo.fnPjm6GetBudgetPeriodName([Budget].[StartDate], [Budget].[EndDate], [BudgetTypeID], @Lang, 1) AS [Period],
		[Project].[Date],
		[ProjectDetail].[DueDate],
		[Project].[EndDate],
		[Budget].[LastCalculationDate],
		[Budget].[NextCalculationDate],
		[Budget].[CurrencyCode],
		[Budget].[WorkBudgetBottom],
		[Budget].[WorkBudgetHigh],
		[Budget].[PlanPurchases],
		[Budget].[PlanProfitability] ,
		[Budget].[ExpenseBudgetBottom],
		[Budget].[ExpenseBudgetHigh],
		[Budget].[PlannedManHour],
		[Budget].[ActualManHour],
		[Budget].[PlannedManHourWithSub],
		[Budget].[ActualManHourWithSub],

		[Budget].[TotalPCPlannedManHour] + dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour]) 
			AS [TotalWDPlannedManHour],--Работа без НДС с учетом скидки, план
		[Budget].[TotalPCActualManHour] + dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour]) 
			AS [TotalWDActualManHour],--Работа без НДС с учетом скидки, факт
		
		dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour])
			AS [TotalWithVATPCWDPlannedManHour],--Работы с НДС, план
		dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour])
			AS [TotalWithVATPCWDActualManHour],--Работа С НДС, факт

		[Budget].[TotalPCPlannedExpense] + dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense])
			AS [TotalWDPlannedExpense],--Затраты без НДС с учетом скидки, план
		[Budget].[TotalPCActualExpense] + dbo.fnPjm6GetBudgetTotalDiscount([DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense])
			AS [TotalWDActualExpense],--Затраты без НДС со скидкой, факт

		dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalWithVATPCPlannedExpense])
			AS [TotalWithVATPCWDPlannedExpense],--Затраты с НДС, план		
		dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalWithVATPCActualExpense])		
			AS [TotalWithVATPCWDActualExpense],--Затраты с НДС, факт
		
		[Budget].[TotalPCPlanPurchase]	AS	TotalPurchases,--Закупки без НДС
		([Budget].[TotalPCPlanPurchase] + [Budget].[VATPCPlanPurchase])	AS	TotalWithVATPurchases,--Закупки с НДС
		
		[Budget].[TotalPCWDPlannedManHourWithSub] AS [TotalWDPlannedManHourWithSub],--Работа без НДС с  учетом скидки с п/п, план
		[Budget].[TotalPCWDActualManHourWithSub]  AS [TotalWDActualManHourWithSub],--Работа без НДС с  учетом скидки с п/п, факт

		[Budget].[TotalWithVATPCWDPlannedManHourWithSub] AS [TotalWithVATPCWDPlannedManHourWithSub],--Работа с НДС с п/п, план
		[Budget].[TotalWithVATPCWDActualManHourWithSub] AS [TotalWithVATPCWDActualManHourWithSub],--Работа с НДС с п/п, факт

		[Budget].[TotalPCWDPlannedExpenseWithSub] AS [TotalWDPlannedExpenseWithSub],--Затраты без НДС с п/п, план
		[Budget].[TotalPCWDActualExpenseWithSub] AS [TotalWDActualExpenseWithSub],--Затраты без НДС с учетом скидки с п/п, факт

		[Budget].[TotalWithVATPCWDPlannedExpenseWithSub] AS [TotalWithVATPCWDPlannedExpenseWithSub],--Затраты с НДС  с п/п, план
		[Budget].[TotalWithVATPCWDActualExpenseWithSub] AS [TotalWithVATPCWDActualExpenseWithSub],--Затраты с НДС с п/п, факт

		[Budget].[TotalPCPlanPurchaseWithSub]	AS	[TotalWithVATPurchasesWithSub],--Закупки без НДС c п/п
		([Budget].[VATPCPlanPurchaseWithSub] + [Budget].[TotalPCPlanPurchaseWithSub])	AS [TotalPurchasesWithSub],--Закупки с НДС c п/п
		
		dbo.fnPjm7GetBudgetSemaphoreState([Budget].[WorkBudgetBottom], [Budget].[WorkBudgetHigh], dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour]), [Budget].[LastCalculationDate], [Budget].[State], [Budget].[UseWorkPlanning])
			AS [SemaphoreTypeManHour],
		dbo.fnPjm7GetBudgetSemaphoreState([Budget].[ExpenseBudgetBottom], [Budget].[ExpenseBudgetHigh], dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1)), [Budget].[LastCalculationDate], [Budget].[State], [Budget].[UseExpensePlanning])
			AS [SemaphoreTypeExpense],

		--Процент исполнения планового времени
		(CASE WHEN [Budget].[PlannedManHour] > 0 THEN 
			ROUND(([ActualManHour]) / [Budget].[PlannedManHour], 2)
			ELSE
				0
		END) AS [PercentOverdraftManHour],
		--Процент исполнения планового времени с подбюджетами
		(CASE WHEN [PlannedManHourWithSub] > 0 THEN 
			ROUND([ActualManHourWithSub] / [PlannedManHourWithSub], 2) 
			ELSE
				0
		END) AS [PercentOverdraftManHourWithSub],
		--Процент исполнения плана по работам
		(CASE WHEN dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour]) > 0 THEN 
			ROUND( dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCActualManHour], [TotalWithVATPCActualManHour]) 
			/ dbo.fnPjm6GetBudgetTotalWithDiscount([VATRate], [VATIsIncluded], [DiscountViewWork], [DiscountPercentWork], [DiscountTotalWork], [TotalPCPlannedManHour], [TotalWithVATPCPlannedManHour]), 2) 
			ELSE
				0
		END) AS [PercentOverdraftTotalWDManHour],
		--Процент исполнения плана по затратам
		(CASE WHEN dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1)) > 0 THEN
			ROUND( dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCActualExpense], [TotalPCActualExpense] + dbo.fnPjm6GetVATForTotal([TotalPCActualExpense], [VATRate], 0, 1)) 
			/ dbo.fnPjm6GetBudgetTotalWithVATWD([VATRate], [VATIsIncluded], [DiscountViewExpense], [DiscountPercentExpense], [DiscountTotalExpense], [TotalPCPlannedExpense], [TotalPCPlannedExpense] + dbo.fnPjm6GetVATForTotal([TotalPCPlannedExpense], [VATRate], 0, 1)), 2) 
			ELSE
				0
		END) AS [PercentOverdraftTotalWDExpense],
		--Процент исполнения плана по работам с подбюджетами
		(CASE WHEN (CASE WHEN [VATRate] IS NOT NULL AND [VATIsIncluded] = 1 THEN [Budget].[TotalWithVATPCWDPlannedManHourWithSub] ELSE [Budget].[TotalPCWDPlannedManHourWithSub] END) > 0 THEN
			ROUND((CASE WHEN [VATRate] IS NOT NULL AND [VATIsIncluded] = 1 THEN [Budget].[TotalWithVATPCWDActualManHourWithSub] ELSE [Budget].[TotalPCWDActualManHourWithSub] END) 
			/ (CASE WHEN [VATRate] IS NOT NULL AND [VATIsIncluded] = 1 THEN [Budget].[TotalWithVATPCWDPlannedManHourWithSub] ELSE [Budget].[TotalPCWDPlannedManHourWithSub] END), 2) 
			ELSE
				0
		END) AS [PercentOverdraftTotalWDManHourWithSub],
		--Процент исполнения плана по затратам с подбюджетами
		(CASE WHEN (CASE WHEN [VATRate] IS NOT NULL THEN [Budget].[TotalWithVATPCWDPlannedExpenseWithSub] ELSE [Budget].[TotalPCWDPlannedExpenseWithSub] END) > 0 THEN
			ROUND((CASE WHEN [VATRate] IS NOT NULL THEN [Budget].[TotalWithVATPCWDActualExpenseWithSub] ELSE [Budget].[TotalPCWDActualExpenseWithSub] END)
			/ (CASE WHEN [VATRate] IS NOT NULL THEN [Budget].[TotalWithVATPCWDPlannedExpenseWithSub] ELSE [Budget].[TotalPCWDPlannedExpenseWithSub] END), 2) 
			ELSE
			 0
		END) AS [PercentOverdraftTotalWDExpenseWithSub],
		ISNULL([BudgetWarningFlagGlobal].[FileAs], [BudgetWarningFlag].[FileAs]) AS [WarningText],
		[Budget].[Comments],
		[Budget].[Created],
		[Budget].[CreatedBy],
		[Budget].[Modified],
		[Budget].[ModifiedBy],
		[Budget].[Timestamp],
		[Project].[CostCenterID],
		[Project].[CustomerCompanyID],
		[Project].[SupplierCompanyID]
	FROM
		[Budget]
		INNER JOIN [Project]
			ON ([Budget].[DocumentID] = [Project].[ID])
		INNER JOIN [ProjectDetail]
			ON	[ProjectDetail].[ID] = [Budget].[DocumentID]
		LEFT OUTER JOIN [BudgetWarningFlag]
			ON ([Budget].[WarningFlagID] = [BudgetWarningFlag].[ID])
		LEFT OUTER JOIN [BudgetWarningFlagGlobal] 
			ON ([BudgetWarningFlag].[ID] = [BudgetWarningFlagGlobal].[ID]
				AND [BudgetWarningFlagGlobal].[Language] = @Lang)
	WHERE
		((@BudgetTypeSelector = 0) OR ([Budget].[BudgetTypeID] = @BudgetTypeSelector)) AND
		((@StateSelector = 0) OR ([Budget].[State] = @StateSelector)) AND
		(@ProjectManagerEmployeeID IS NULL OR [Project].[ProjectManagerEmployeeID] = @ProjectManagerEmployeeID) AND 
		((@CostCenterSelector = 0) OR (@CostCenterSelector = 1 AND ([Project].[CostCenterID] = @CostCenterID OR (@CostCenterID IS NULL AND [Project].[CostCenterID] IS NULL)))) AND
		(
			(
				[Budget].[BudgetTypeID] = 1 
				AND
				(
					@DateSelector = 0 
					OR 
					(
						[Project].[Date] < @PeriodEnd AND ISNULL([Project].[EndDate], @Now)  > @PeriodStart
					)
				)
			)
			OR
			(
				[Budget].[BudgetTypeID] > 1 
				AND
				(
					@DateSelector = 0 
					OR 
					(
						[Budget].[StartDate] < @PeriodEnd AND ISNULL([Budget].[EndDate], @Now)  > @PeriodStart
					)
				)
			)
		)
		AND
		(dbo.fnPjm6CanEditBudget([Budget].[ID]) = 1 OR dbo.fnPjm6CanViewBudget([Budget].[ID]) = 1)
		AND
		[Budget].[State] <> 6

RETURN 0
GO

GRANT EXECUTE ON [pjm6BudgetBrowse] TO PUBLIC
GO