﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm7DocumentChangeBillableFlagOnCalculationApprove]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm7DocumentChangeBillableFlagOnCalculationApprove'
		DROP PROCEDURE [dbo].[pjm7DocumentChangeBillableFlagOnCalculationApprove]
	END
GO

PRINT 'Creating procedure pjm7DocumentChangeBillableFlagOnCalculationApprove'
GO

/******************************************************************************
**		File: pjm7DocumentChangeBillableFlagOnCalculationApproves.sql
**		Name: pjm7DocumentChangeBillableFlagOnCalculationApproves
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Sergey Pelevin
**		Date: 26.06.14
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-------------------------------------------
**		01.07.14	SAP			Доработка условий выбора ЗОВ и затрат
**		02.07.14	SAP			Обработка записей из других проектов / подпроектов
**		04.07.14	SAP			Фильтрация списка проектов, в кторых можно снимать флаги "К оплате"
*******************************************************************************/
CREATE PROCEDURE pjm7DocumentChangeBillableFlagOnCalculationApprove
	@CalculationID TIdentifier,
	@BillableFlagNewValue bit
AS
	SET NOCOUNT ON

	DECLARE 
		@ProjectID TIdentifier,
		@Date TDate,
		@DocumentClass TClass,
		@PeriodStart TDate,
		@PeriodEnd TDate,
		@TimeEntryClass TClass = [dbo].[adfGetClassByName]('Pjm5.TimeEntry'),
		@ExpenseClass TClass = [dbo].[adfGetClassByName]('Pjm5.Expense')
		
	SELECT	
		@ProjectID = [Calculation].[ProjectID],		
		@DocumentClass = [Calculation].[DocumentClass],
		@PeriodStart = [Calculation].[PeriodStart],
		@PeriodEnd = DATEADD(DAY,1,[Calculation].[PeriodEnd])
	FROM
		[Calculation]
	WHERE
		[Calculation].[ID] = @CalculationID
				

	IF @DocumentClass <> @TimeEntryClass AND @DocumentClass <> @ExpenseClass
	BEGIN
		RAISERROR('#DocumentHasNoBillableFlag', 18, -1)
		RETURN 1
	END

	

	BEGIN TRANSACTION

	--добавление необходимых прав
	INSERT INTO [Right] ([Name],[DisplayName],[Group]) VALUES('RemoveBillableFlag', '', '')
	EXEC pjm6SetRightForMasters 'RemoveBillableFlag'

	--Установка значений параметров для вызова функции fnPjm7GetAvailableProjectsForCalculationProjectSelectorDialog,
	--которая возвращает список доступных проектов для сотрудника (используется в диалоге выбора проекта при добавлении линии в калькуляцию)
	DECLARE
		@EmployeeID int = NULL,--Текущий пользователь
		@OnlyActive bit = 1,--Только проекты в состоянии 2 (Открыт)
		@ContractOnly bit = 1,-- Только коммерческие проекты (тип Работа по договорам)
		@OnlyProjectMember bit = 0,--Проекты, в которых текущий пользователь является участником, либо РП, либо ФМП
		@ProjectState int = 2,--Служебный фильтр (не состояние проекта), 2 = фильтр отключен
		@ProjectSelector int = 0,--Служебный фильтр (относится к использованию запросов в проекте), 0 = фильтр отключен
		@SupportsRequestManagement bit = 0--Использование запросов в проекте 0 = не важно

	IF @DocumentClass = @TimeEntryClass
	BEGIN
		--Подготовка списка проектов, в которых будем искать ЗОВ	
		;
		WITH [Projects]([ID]) 
		AS
		(
			SELECT DISTINCT 
				[ProjectID] 
			FROM 
				[TimeEntry] 
			WHERE 
				[ID] IN (
							SELECT 
								[DocumentID]
							FROM 
								[CalculationLine] 
							WHERE 
								[CalculationID] = @CalculationID
						)
		),

		[TimeEntriesInRange]([ID])
		AS
		(
			SELECT
				[ID]
			FROM
				[TimeEntry]
			WHERE
				[ProjectID] IN
				(
					SELECT 
						[Projects].[ID] 
					FROM 
						[Projects]
						INNER JOIN dbo.fnPjm7GetAvailableProjectsForCalculationProjectSelectorDialog 
								(
									@EmployeeID,
									@OnlyActive,
									@ContractOnly,
									@OnlyProjectMember,
									@ProjectState,
									@ProjectSelector,
									@SupportsRequestManagement
								) AS [AvialableProjects]
							ON [Projects].[ID] = [AvialableProjects].[ID]
				)
				AND
				[StartTime] >= @PeriodStart
				AND
				[StartTime] < @PeriodEnd
				AND
				[Billable] = 1
				AND 
				([State] <> 6 AND [State] <> 4)
			EXCEPT
			SELECT
				[DocumentID]
			FROM
				[CalculationLine]
			WHERE
				[CalculationID] = @CalculationID
		)
		UPDATE [TimeEntry]
		SET [Billable] = @BillableFlagNewValue
		WHERE [ID] IN (SELECT [ID] FROM [TimeEntriesInRange])

		IF @@error <> 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	ELSE IF @DocumentClass = @ExpenseClass
	BEGIN
			
		;
		WITH [Projects]([ID]) 
		AS
		(
			SELECT DISTINCT 
				[ProjectID] 
			FROM 
				[Expense] 
			WHERE 
				[ID] IN (
							SELECT 
								[DocumentID]
							FROM 
								[CalculationLine] 
							WHERE 
								[CalculationID] = @CalculationID
						)
		),

		[ExpensesInRange]([ID])
		AS
		(
			SELECT
				[ID]
			FROM
				[Expense]
			WHERE
				[ProjectID] IN 
					(
						SELECT 
							[Projects].[ID] 
						FROM 
							[Projects]
							INNER JOIN dbo.fnPjm7GetAvailableProjectsForCalculationProjectSelectorDialog 
								(
									@EmployeeID,
									@OnlyActive,
									@ContractOnly,
									@OnlyProjectMember,
									@ProjectState,
									@ProjectSelector,
									@SupportsRequestManagement
								) AS [AvialableProjects]
							ON [Projects].[ID] = [AvialableProjects].[ID]
					)
				AND
				[Date] >= @PeriodStart
				AND
				[Date] < @PeriodEnd
				AND
				[Billable] = 1
				AND 
				([State] <> 6 AND [State] <> 4)
			EXCEPT
			SELECT
				[DocumentID]
			FROM
				[CalculationLine]
			WHERE
				[CalculationID] = @CalculationID
		)
		UPDATE [Expense]
		SET [Billable] = @BillableFlagNewValue
		WHERE [ID] IN (SELECT [ID] FROM [ExpensesInRange])

		IF @@error <> 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END

	--удаление дарованных в ходе процедуры прав
	EXEC pjm6DelRightForMasters 'RemoveBillableFlag'
	DELETE [Right] WHERE [Name] = 'RemoveBillableFlag'

	IF @@error <> 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	COMMIT TRANSACTION

	RETURN 0
GO

GRANT EXECUTE ON pjm7DocumentChangeBillableFlagOnCalculationApprove TO PUBLIC
GO