IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'pjm6DocumentNumberFindTemplate')
	BEGIN
		PRINT 'Dropping Procedure pjm6DocumentNumberFindTemplate'
		DROP  Procedure  pjm6DocumentNumberFindTemplate
	END
GO

PRINT 'Creating Procedure pjm6DocumentNumberFindTemplate'
GO
/******************************************************************************
**		File: DocumentNumberFindTemplate
**		Name: pjm6DocumentNumberFindTemplate
**		Desc:        
**			     
**              
**		Auth: SPV
**		Date: 17.07.11
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		17.07.11	SPV				first version
*******************************************************************************/
CREATE Procedure pjm6DocumentNumberFindTemplate
	@DocumentClass TClass,
	@DocumentID	TIdentifier
AS
	
	DECLARE
		@TemplateClass TClass,
		@CompanyClass TClass,
		@ProjectClass TClass,
		@Own TBool,
		@Flag tinyint
		
	SET @CompanyClass = dbo.adfGetClassByName('Adf.Company')
	SET @ProjectClass = dbo.adfGetClassByName('Pjm5.Project')
	
	SELECT @Own = [Own] FROM [Company] WHERE [Class] = @DocumentClass AND [ID] = @DocumentID
	if @Own = 1
	BEGIN
		SELECT dbo.adfGetClassByName('Pjm5.Statement') as [TemplateClass]
		RETURN 1
	END

	SET @Flag = CASE WHEN @DocumentClass = @CompanyClass AND ISNULL(@Own, 0) = 0 THEN 1
					 WHEN @DocumentClass = @ProjectClass AND ISNULL(@Own, 0) = 0 THEN 2
					 ELSE 0 END
					 
	IF @Flag IN (1, 2)
	BEGIN
		SET @TemplateClass = (
			SELECT 
				TOP(1) [NumberTemplate].[TemplateClass]
			FROM
				[Project]
				INNER JOIN [NumberTemplate]
					ON [SupplierCompanyID] = [NumberTemplate].[CompanyID]
			WHERE
				(@Flag = 2 AND [Project].[ID] = @DocumentID OR @Flag = 1 AND [Project].[CustomerCompanyID] = @DocumentID) AND
				[NumberTemplate].[ApplyLevel] = @Flag)
	END
		
	IF @TemplateClass IS NULL
	BEGIN
		RAISERROR('#numbertemplatenotfound', 18, -1)
		RETURN 1
	END
	
	SELECT @TemplateClass as [TemplateClass]
	RETURN 1
GO

GRANT EXEC ON pjm6DocumentNumberFindTemplate TO PUBLIC
GO