﻿IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm7EmployeeNormBrowse]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm7EmployeeNormBrowse'
		DROP PROCEDURE [dbo].[pjm7EmployeeNormBrowse]
	END
GO

PRINT 'Creating procedure pjm7EmployeeNormBrowse'
GO

/******************************************************************************
**		File: pjm7EmployeeNormBrowse.sql
**		Name: pjm7EmployeeNormBrowse
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: Sergey Pelevin
**		Date: 20.06.14
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-------------------------------------------
**		21.07.14	SAP			Переработка процедуры (из-за создания документа "Норматив сотрудника")
**		24.07.14	GNV			Добавление поля [BaseCurrency]
**		13.08.14	BAO			Нормативы с пустыми значениями должны попадать в выборку
*******************************************************************************/
CREATE PROCEDURE [dbo].[pjm7EmployeeNormBrowse]
	@EmployeeID TIdentifier	
AS
	SET NOCOUNT ON

	SELECT
		[ENE].[Class],
		[ENE].[ID],
		[ENE].[State],
		[ENE].[Flag],
		[ENE].[EmployeeID],
		[ENE].[StartDate],
		[ENE].[NormID],
		[NV].[UtilizationHours],
		[NV].[RealizationHours],
		CAST(ROUND((CAST([NV].[RealizationHours] as float) / [NV].[UtilizationHours]) * 100,2) as decimal(18,4)) as [RealizationPercent],
		[NV].[Revenue],		
		[ENE].[Created],
		[ENE].[CreatedBy],
		[ENE].[Modified],
		[ENE].[ModifiedBy],
		[ENE].[Timestamp],
		[dbo].[fnPjm5GetBaseCurrency]() as [BaseCurrency],
		[ENE].[Comments]
	FROM
		[EmployeeNormEntry] AS [ENE]
		INNER JOIN [Norm] AS [N]
			ON [N].[ID] = [ENE].[NormID]
		LEFT OUTER JOIN (
			SELECT 
				[NormID], MAX([YEAR]) AS [Year]
			FROM [NormValue]
			GROUP BY [NormID]
		) AS [NVY]
			ON [ENE].[NormID] = [NVY].[NormID]
		LEFT OUTER JOIN [NormValue] AS [NV]
			ON [NVY].[NormID] = [NV].[NormID]
			AND [NVY].[Year] = [NV].[Year]
	WHERE 
		[ENE].[EmployeeID] = @EmployeeID	
	ORDER BY 
		[ENE].[StartDate] DESC

	RETURN 0
GO

GRANT EXECUTE ON [pjm7EmployeeNormBrowse] TO PUBLIC
GO
