IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm6GenerateInvoiceFromServiceAssignment]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm6GenerateInvoiceFromServiceAssignment'
		DROP PROCEDURE [dbo].[pjm6GenerateInvoiceFromServiceAssignment]
	END
GO

PRINT 'Creating procedure pjm6GenerateInvoiceFromServiceAssignment'
GO

/******************************************************************************
**		File: pjm6GenerateInvoiceFromServiceAssignment.sql
**		Name: pjm6GenerateInvoiceFromServiceAssignment
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: yuriy.soldatkin
**		Date: 14.09.09
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		26.03.13	SOV				 OpenXml  Xml
*******************************************************************************/
CREATE PROCEDURE [pjm6GenerateInvoiceFromServiceAssignment]
	@ID TIdentifier OUTPUT,
	@Xml XML,
	@DocumentClass TClass,
	@Number TName,
	@Date TDate,
	@FileAs TName,
	@BankAccountID TIdentifier,
	@CurrencyCode TCode3,
	@CurrencyRate TMoney,
	@Advance TBool, 
	@ServiceTypeID TIdentifier,
	@Description TNotes
AS
	SET NOCOUNT ON
	DECLARE @DocumentID TIdentifier,
			@State TState,
			@ProjectID TIdentifier,
			@DocumentCurrencyCode TCode3,
			@ProjectCurrencyCode TCode3,
			@DocumentCurrencyRate TMoney,
			@InvoiceBaseCurrencyRate TMoney,
			@ApplyVAT TBool,
			@PaymentCurrencyPoint TIdentifier,
			@InvoiceExpirationPeriod TCount,
			@InvoiceExpirationPeriodType TIdentifier,
			@VATRate TPercent,
			@VATIsIncluded TBool,
			@Amount TAmount,
			@Price TMoney,
			@VAT TMoney,
			@rv int,
			@value TVariant,
			@SupplierCompanyID TIdentifier,
			@CustomerCompanyID TIdentifier,
			@Scale TCount,
			@ServiceID TIdentifier
	
	DECLARE
		@TotalPC TMoney,
		@TotalWithVATPC TMoney,
		@PricePC decimal(18, 4), 
		@VATPC decimal(18, 4)

	DECLARE 
		@BaseCurrencyCode TCode3,
		@NationalCurrencyCode TCode3

	SET @State = dbo.adfwfGetInitialState('98b8f49e-202e-4b46-8074-421734078da6')
	IF @State IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END

	EXEC adfParameterGet @Name='BaseCurrency', @Group='CommonBusiness', @Value = @value OUTPUT
	SET @BaseCurrencyCode = CAST(@value as varchar(3))
	EXEC adfParameterGet @Name='NationalCurrency', @Group='CommonBusiness', @Value = @value OUTPUT
	SET @NationalCurrencyCode = CAST(@value as varchar(3))

	EXEC adfParameterGet @Name='VATRate', @Group='CommonBusiness', @Value = @value OUTPUT
	SET @VATRate = CAST(@value as decimal(18,4))
		
	SELECT
		@DocumentID = [XmlTable].value('ID[1]', 'int'),
		@ProjectID = [XmlTable].value('ProjectID[1]', 'int'),
		@VATIsIncluded = CASE [XmlTable].value('VATIsIncluded[1]', 'varchar(5)') WHEN 'false' THEN 0 ELSE 1 END,
		@TotalPC = [XmlTable].value('Sum[1]', 'money'),
		@TotalWithVATPC = [XmlTable].value('SumWithVAT[1]', 'money'),
		@VATPC = [XmlTable].value('VAT[1]', 'money'),
		@DocumentCurrencyCode =  [XmlTable].value('CurrencyCode[1]', 'varchar(5)') ,
		@ProjectCurrencyCode = [Project].[CurrencyCode],
		@PaymentCurrencyPoint = [Project].[PaymentCurrencyPoint],
		@InvoiceExpirationPeriod = [Project].[InvoiceExpirationPeriod],
		@InvoiceExpirationPeriodType = [Project].[InvoiceExpirationPeriodType],
		@ApplyVAT = [Project].[ApplyVAT],
		@SupplierCompanyID = [Project].[SupplierCompanyID],
		@CustomerCompanyID = [Project].[CustomerCompanyID],
		@Scale = [Project].[Scale],
		@ServiceID = [XmlTable].value('ServiceID[1]', 'int')
	FROM 
		@Xml.nodes('/pjm5ServiceAssignmentGet/ServiceAssignment') AS List(XmlTable)
		INNER JOIN [Project]
			ON [XmlTable].value('ProjectID[1]', 'int') = [Project].[ID]
	
	DECLARE @EmployeeID TIdentifier
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	IF dbo.adfHasAccessRight(dbo.adfGetRightID('ManageAllInvoices')) = 0 AND
		dbo.fnPjm5HasFinansDataAccess(@ProjectID, @EmployeeID, NULL, NULL) = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
	
	IF @ProjectCurrencyCode = @BaseCurrencyCode
		SET @InvoiceBaseCurrencyRate = 1.0
	ELSE IF @DocumentCurrencyCode = @CurrencyCode AND @DocumentCurrencyCode = @BaseCurrencyCode
		SET @InvoiceBaseCurrencyRate = @CurrencyRate

	IF @ProjectCurrencyCode = @CurrencyCode
		SET @CurrencyRate = 1.0
	IF @ProjectCurrencyCode = @BaseCurrencyCode
		SET @InvoiceBaseCurrencyRate = 1.0
	DECLARE @c1 TCode3, @c2 TCode3
	IF @CurrencyRate IS NULL
	BEGIN
		IF @NationalCurrencyCode = @CurrencyCode
		BEGIN
			SET @c1 = @ProjectCurrencyCode
			SET @c2 = @CurrencyCode
		END
		ELSE
		BEGIN
			SET @c2 = @ProjectCurrencyCode
			SET @c1 = @CurrencyCode
		END

		SELECT
			@CurrencyRate = [Rate]
		FROM
			[CurrencyRate]
		WHERE
			[FromCurrencyCode] = @c1 AND 
			[ToCurrencyCode] = @c2 AND
			@Date BETWEEN [ValidFrom] AND [ValidTo]
	END
	IF @InvoiceBaseCurrencyRate IS NULL
	BEGIN
		IF @NationalCurrencyCode = @ProjectCurrencyCode
		BEGIN
			SET @c1 = @BaseCurrencyCode
			SET @c2 = @ProjectCurrencyCode
		END
		ELSE
		BEGIN
			SET @c2 = @BaseCurrencyCode
			SET @c1 = @ProjectCurrencyCode
		END

		SELECT
			@InvoiceBaseCurrencyRate = [Rate]
		FROM
			[CurrencyRate]
		WHERE
			[FromCurrencyCode] = @c1 AND 
			[ToCurrencyCode] = @c2 AND
			@Date BETWEEN [ValidFrom] AND [ValidTo]
	END

	IF @VATIsIncluded != 0
		SET @PricePC = @TotalWithVATPC
	ELSE
		SET @PricePC = @TotalPC

	IF @ProjectCurrencyCode = @CurrencyCode
		SET @Price = @PricePC
	ELSE
	BEGIN
		IF @CurrencyCode != @NationalCurrencyCode
			SET @Price = ROUND(@PricePC / @CurrencyRate, 2)
		ELSE
			SET @Price = ROUND(@PricePC * @CurrencyRate, 2)
	END
	
	DECLARE @divider decimal(18, 4)
	IF @VATIsIncluded = 1
		SET @divider = 1.0 + @VATRate
	ELSE 
		SET @divider = 1.0
		
	IF @ApplyVAT != 0
	BEGIN
		IF @ProjectCurrencyCode = @CurrencyCode
			SET @VAT = @VATPC
		ELSE 
		BEGIN
			IF @CurrencyCode != @NationalCurrencyCode
				SET @VAT = ROUND(@Price * @VATRate / @divider, 2)
			ELSE
				SET @VAT = ROUND(@Price * @VATRate / @divider, 2)
		END
	END
	ELSE 
	BEGIN
		SET @VAT = 0
		SET @VATPC = 0
		SET @VATRate = 0.0
	END
		
	IF @ApplyVAT = 0 SET @VATIsIncluded = 0
	
	DECLARE @InvoiceDueDate TDate
	
	IF @InvoiceExpirationPeriodType = 1
		SET @InvoiceDueDate = dbo.fnPjm5AddWorkingDays(@Date, @InvoiceExpirationPeriod)
	ELSE
		SET @InvoiceDueDate = DATEADD(day, @InvoiceExpirationPeriod - 1, @Date)

	INSERT INTO [Invoice]
		([Flag], [State], [Date], [Number], [FileAs],
		 [SupplierBankAccountID], [ProjectID],
		 [CurrencyRate], [BaseCurrencyRate], [CurrencyCode], 
		 [PaymentCurrencyPoint], [DueDate], [InvoiceExpirationPeriodType],
		 [SupplierCompanyID], [CustomerCompanyID],
		 [Scale])
	VALUES(0, @State, @Date, @Number, @FileAs,
		@BankAccountID, @ProjectID,
		@CurrencyRate, @InvoiceBaseCurrencyRate, @CurrencyCode,
		@PaymentCurrencyPoint, @InvoiceDueDate, @InvoiceExpirationPeriodType,
		@SupplierCompanyID, @CustomerCompanyID,
		@Scale)
	IF @@ERROR != 0
		RETURN 1
	SET @ID = SCOPE_IDENTITY()

	DECLARE @TotalBC TMoney, @TotalWithVATBC TMoney
	SET @Amount = 1.0
	SET @TotalBC = ROUND(CASE 
						WHEN @ProjectCurrencyCode = @BaseCurrencyCode
							THEN @PricePC * @Amount - @VATIsIncluded * @VATPC
						WHEN @ProjectCurrencyCode = @NationalCurrencyCode
							THEN (@PricePC * @Amount - @VATIsIncluded * @VATPC) / @InvoiceBaseCurrencyRate
						ELSE (@PricePC * @Amount - @VATIsIncluded * @VATPC) * @InvoiceBaseCurrencyRate
					END, 2)
	SET @TotalWithVATBC = ROUND(CASE 
						WHEN @ProjectCurrencyCode = @BaseCurrencyCode
							THEN @PricePC * @Amount + (1 - @VATIsIncluded) * @VATPC
						WHEN @ProjectCurrencyCode = @NationalCurrencyCode
							THEN (@PricePC * @Amount + (1 - @VATIsIncluded) * @VATPC) / @InvoiceBaseCurrencyRate
						ELSE (@PricePC * @Amount + (1 - @VATIsIncluded) * @VATPC) * @InvoiceBaseCurrencyRate
					END, 2)
	
	DECLARE	@UOMID TIdentifier
	SELECT 
		@UOMID=[UOM].[ID] 
	FROM 
		[UOM], [Service]
	WHERE
		[Service].[ID]=@ServiceID AND [Service].[UOMID]=[UOM].[ID] 
					
					

	INSERT INTO [InvoiceLine] (
		[InvoiceID], [DocumentClass], [DocumentID], [Description], [Amount], [VATIsIncluded], [VATRate], 
		[Price], [VAT], [PricePC], [VATPC], 
		[UOMID], [ServiceTypeID], [TotalBC], [TotalWithVATBC])
	VALUES (
		@ID, @DocumentClass, @DocumentID, @Description, 1, @VATIsIncluded, @VATRate, 
		@Price, @VAT, @PricePC, @VATPC, @UOMID, @ServiceTypeID, @TotalBC, @TotalWithVATBC)
	IF @@ERROR != 0
		RETURN 1
		
	DECLARE 
		@UserID TIdentifier,
		@InvoiceClass TClass,
		@ReminderPublishState TState,
		@ReminderPublishInterval TTimeSpan,
		@ReminderState TState,
		@ReminderInterval TTimeSpan
	
	SELECT
		@UserID = dbo.adfGetCurrentUserID(),
		@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice'),
		@ReminderPublishState = dbo.fnPjm5GetUserInitialReminderState(@InvoiceClass, 'VA', @UserID),
		@ReminderPublishInterval = dbo.fnPjm5GetReminderInterval(@InvoiceClass, 'VA', @UserID),
		@ReminderState = dbo.fnPjm5GetUserInitialReminderState(@InvoiceClass, 'VP', @UserID),
		@ReminderInterval = dbo.fnPjm5GetReminderInterval(@InvoiceClass, 'VP', @UserID)

	EXEC @rv = pjm7InvoiceReminderCreate @ID, 'VP',
		@InvoiceDueDate, @ReminderState, @ReminderInterval
	IF @rv != 0
	BEGIN
		RETURN @rv
	END
	
	EXEC @rv = pjm7InvoiceReminderCreate @ID, 'VA',
		@Date, @ReminderPublishState, @ReminderPublishInterval
	IF @rv != 0
	BEGIN
		RETURN @rv
	END

	RETURN 0
GO

GRANT EXECUTE ON pjm6GenerateInvoiceFromServiceAssignment TO PUBLIC
GO
