IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'pjm5IssueImport')
	BEGIN
		PRINT 'Dropping Procedure pjm5IssueImport'
		DROP  Procedure  pjm5IssueImport
	END

GO

PRINT 'Creating Procedure pjm5IssueImport'
GO
/******************************************************************************
**		File: 
**		Name: pjm5IssueImport
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**     ----------							-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure pjm5IssueImport
	@ProjectCode TCode,
	@PriorityID TIdentifier,
	@DueDate TDate,
	@Subject TName,
	@Comments TNotes,
	@From THref,
	@Number TCode OUTPUT,
	@ID TIdentifier OUTPUT,
	@Class TClass OUTPUT
AS
	DECLARE @ProjectID TIdentifier,
			@State TState
	
	SET NOCOUNT ON
	
	SELECT @ProjectID = [ID] FROM [Project] WHERE [Number] = @ProjectCode

	IF @ProjectID IS NULL
	BEGIN
		RAISERROR('#invalidprojectcode', 18, -1)
		RETURN 1
	END

	SET @State = dbo.adfwfGetInitialState('1acd1e8f-3c88-43b1-81ac-0dcabc871562')
	IF @State IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END

	IF NOT EXISTS(SELECT * FROM [Project] WHERE [ID] = @ProjectID AND [SupportsRequestManagement] = 1)
	BEGIN
		RAISERROR('#doesnotsupportsrequestmanagement', 18, -1)
		RETURN 1
	END


	IF NOT EXISTS(SELECT * FROM [DocumentIndex] WHERE [ProjectID] = @ProjectID)
	BEGIN
		INSERT INTO [DocumentIndex] ([ProjectID]) VALUES(@ProjectID)
	END
	
	DECLARE @Index TCount, @Prefix TCode, @N varchar(10)
	
	BEGIN TRANSACTION

	UPDATE
		[DocumentIndex]
	SET
		@Index = [IssueIndex] + 1,
		[IssueIndex] = [IssueIndex] + 1
	WHERE
		[ProjectID] = @ProjectID
	
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
		
	SELECT
		@Prefix = LEFT(UPPER([Number]), 8)
	FROM
		[Project] 
	WHERE
		[ID] = @ProjectID
		
	SET @N = CAST(@Index as varchar(10))
	
	SET @Number = @Prefix + REPLICATE('0', 8 - LEN(@N)) + @N

	INSERT INTO [Issue]
		([Flag], [State], [Number], [FileAs], [ProjectID], [SeverityID], [PriorityID], [DueDate], [Comments],
		[SendStatusReport])
	VALUES
		(1, @State, @Number, @Subject, @ProjectID, 2, @PriorityID, @DueDate, @Comments, 0)
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	SET @ID = SCOPE_IDENTITY()
	SET @Class = dbo.adfGetClassByName('Pjm5.Issue')
	
	DECLARE @PropertyID TIdentifier
	
	SELECT @PropertyID = [ID] FROM [Property] WHERE [Class]=@Class AND [Guid] = CAST('{00000001-0000-0000-0000-000000000000}' as uniqueidentifier)
	
	EXEC adfPropertyValueUpdate @Class, @ID, @PropertyID ,@From
	
	DECLARE @BusinessCardClass TClass,
			@BusinessCardID TIdentifier
	
	SELECT
		@BusinessCardClass = [BusinessCardClass],
		@BusinessCardID = [BusinessCardID]
	FROM
		[Email]
	WHERE
		[Email] = @From AND [EmailTypeID] = 1
		
	IF @BusinessCardID IS NOT NULL
	BEGIN
		
		INSERT INTO [DocumentContact]
			([DocumentClass], [DocumentID], [BusinessCardClass], [BusinessCardID])
		VALUES
			(@Class, @ID, @BusinessCardClass, @BusinessCardID)
			
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	COMMIT TRANSACTION

	EXEC pjm5IssueChangeState @ID = @ID, @ToState = 2

	RETURN 0

GO

GRANT EXEC ON pjm5IssueImport TO PUBLIC
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'pjm5IssueImportFromSite')
	BEGIN
		PRINT 'Dropping Procedure pjm5IssueImportFromSite'
		DROP  Procedure  pjm5IssueImportFromSite
	END

GO

PRINT 'Creating Procedure pjm5IssueImportFromSite'
GO
/******************************************************************************
**		File: 
**		Name: pjm5IssueImportFromSite
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**     ----------							-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE Procedure pjm5IssueImportFromSite
	@ProjectCode TCode,
	@PriorityID TIdentifier,
	@DueDate TDate,
	@Subject TName,
	@Comments xml,
	@From THref,
	@Number TCode OUTPUT,
	@ID TIdentifier OUTPUT,
	@Class TClass OUTPUT
AS
	DECLARE @ProjectID TIdentifier,
			@State TState
	
	SET NOCOUNT ON
	
	SELECT @ProjectID = [ID] FROM [Project] WHERE [Number] = @ProjectCode

	IF @ProjectID IS NULL
	BEGIN
		RAISERROR('#invalidprojectcode', 18, -1)
		RETURN 1
	END

	SET @State = dbo.adfwfGetInitialState('1acd1e8f-3c88-43b1-81ac-0dcabc871562')
	IF @State IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END

	IF NOT EXISTS(SELECT * FROM [Project] WHERE [ID] = @ProjectID AND [SupportsRequestManagement] = 1)
	BEGIN
		RAISERROR('#doesnotsupportsrequestmanagement', 18, -1)
		RETURN 1
	END


	IF NOT EXISTS(SELECT * FROM [DocumentIndex] WHERE [ProjectID] = @ProjectID)
	BEGIN
		INSERT INTO [DocumentIndex] ([ProjectID]) VALUES(@ProjectID)
	END
	
	DECLARE @Index TCount, @Prefix TCode, @N varchar(10)
	
	BEGIN TRANSACTION

	UPDATE
		[DocumentIndex]
	SET
		@Index = [IssueIndex] + 1,
		[IssueIndex] = [IssueIndex] + 1
	WHERE
		[ProjectID] = @ProjectID
	
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
		
	SELECT
		@Prefix = LEFT(UPPER([Number]), 8)
	FROM
		[Project] 
	WHERE
		[ID] = @ProjectID
		
	SET @N = CAST(@Index as varchar(10))
	
	SET @Number = @Prefix + REPLICATE('0', 8 - LEN(@N)) + @N

	-- 
	DECLARE @PreferredContactType	TName,
			@NotificationRequested	TName,
			@Name					TName,
			@SurName				TName,
			@Email					THref,
			@TextComments			TNotes

	SELECT
		@PreferredContactType = '  : ' + 
			CASE 
				WHEN @Comments.value('(/UserRequestData/PersonalData/PreferredContactType)[1]','nvarchar(256)') = 'Email' THEN 
					' ' 
				ELSE 
					'' 
			END,
		@NotificationRequested = '     ProjectMate: ' + 
			CASE 
				WHEN @Comments.value('(/UserRequestData/PersonalData/NotificationRequested)[1]','varchar(32)') = 'true' THEN 
					'' 
				ELSE 
					'' 
			END,
		@Name = ': ' + 
			CASE 
				WHEN @Comments.value('(/UserRequestData/PersonalData/FirstName)[1]','nvarchar(256)') IS NOT NULL THEN 
					@Comments.value('(/UserRequestData/PersonalData/FirstName)[1]','nvarchar(256)') 
				ELSE 
					' '
			END,
		@SurName = ': ' + 
			CASE 
				WHEN @Comments.value('(/UserRequestData/PersonalData/LastName)[1]','nvarchar(256)') IS NOT NULL THEN 
					@Comments.value('(/UserRequestData/PersonalData/LastName)[1]','nvarchar(256)')
				ELSE 
					' ' 
			END,
		@Email = 'Email: ' + 
			CASE 
				WHEN @Comments.value('(/UserRequestData/PersonalData/Email)[1]','nvarchar(256)') IS NOT NULL THEN 
					@Comments.value('(/UserRequestData/PersonalData/Email)[1]','nvarchar(256)')
				ELSE 
					' ' 
			END,
		@TextComments = ': ' + 
			CASE
				WHEN @Comments.value('(/UserRequestData/PersonalData/Comments)[1]','nvarchar(max)') IS NOT NULL THEN 
					@Comments.value('(/UserRequestData/PersonalData/Comments)[1]','nvarchar(max)')
				ELSE 
					' ' 
			END
	
	DECLARE @FullComment TNotes

	SET @FullComment = '  : ' + char(13) + @Name + char(13) + @SurName + char(13) + @Email +
		+ char(13) + char(13) + @TextComments + char(13) + char(13) + ' .' + char(13) + @PreferredContactType + char(13) + @NotificationRequested

	INSERT INTO [Issue]
		([Flag], [State], [Number], [FileAs], [ProjectID], [SeverityID], [PriorityID], [DueDate], [Comments],
		[SendStatusReport])
	VALUES
		(1, @State, @Number, @Subject, @ProjectID, 2, @PriorityID, @DueDate, @FullComment, 0)
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	SET @ID = SCOPE_IDENTITY()
	SET @Class = dbo.adfGetClassByName('Pjm5.Issue')
	
	DECLARE @PropertyID TIdentifier
	
	SELECT @PropertyID = [ID] FROM [Property] WHERE [Class]=@Class AND [Guid] = CAST('{00000001-0000-0000-0000-000000000000}' as uniqueidentifier)
	
	EXEC adfPropertyValueUpdate @Class, @ID, @PropertyID ,@From
	
	DECLARE @BusinessCardClass TClass,
			@BusinessCardID TIdentifier
	
	SELECT
		@BusinessCardClass = [BusinessCardClass],
		@BusinessCardID = [BusinessCardID]
	FROM
		[Email]
	WHERE
		[Email] = @From AND [EmailTypeID] = 1
		
	IF @BusinessCardID IS NOT NULL
	BEGIN
		
		INSERT INTO [DocumentContact]
			([DocumentClass], [DocumentID], [BusinessCardClass], [BusinessCardID])
		VALUES
			(@Class, @ID, @BusinessCardClass, @BusinessCardID)
			
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	COMMIT TRANSACTION

	EXEC pjm5IssueChangeState @ID = @ID, @ToState = 2

	RETURN 0

GO

GRANT EXEC ON pjm5IssueImportFromSite TO PUBLIC
GO