IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'NamedListUpdate')
	BEGIN
		PRINT 'Dropping Procedure NamedListUpdate'
		DROP  Procedure  NamedListUpdate
	END

GO

PRINT 'Creating Procedure NamedListUpdate'
GO
/******************************************************************************
**		File: 
**		Name: NamedListUpdate
**		Desc: 
**
**		This template can be customized:
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		11.03.13	MAB				  OPENXML
*******************************************************************************/
CREATE Procedure NamedListUpdate
	@ProjectID TIdentifier,
	@Xml XML
AS
	DECLARE 
		@result INT			
			
	SET NOCOUNT ON
	
	IF @result != 0 OR @@error != 0
		RETURN 1
	
	BEGIN TRANSACTION
	
	DECLARE 
		NamedListCursor CURSOR LOCAL FORWARD_ONLY READ_ONLY FOR
		SELECT
			T.c.value('ID[1]','int'),
			T.c.value('Name[1]','nvarchar(256)'),
			(CASE T.c.value('SortOrderAscending[1]','varchar(5)') 
				WHEN 'false' THEN 
					0 
				ELSE 
					1 
			END) as [SortOrderAscending]
		FROM
			@Xml.nodes('/NamedListDataSet/NamedList') AS T(c)
		WHERE
			T.c IS NOT NULL
	
	DELETE [NamedList] WHERE [ProjectID] = @ProjectID

	IF @@ERROR != 0 GOTO ErrorHandler

	DECLARE @NamedListID TIdentifier, @ID TIdentifier, @Name TName, @SortOrderAscending TBool
	
	OPEN NamedListCursor
	
	FETCH NEXT FROM NamedListCursor INTO @ID, @Name, @SortOrderAscending
	WHILE @@FETCH_STATUS = 0
	BEGIN
		INSERT INTO [NamedList] ([ProjectID], [Name], [SortOrderAscending]) VALUES(@ProjectID, @Name, @SortOrderAscending)
		SET @NamedListID = SCOPE_IDENTITY()

		INSERT INTO [NamedListBinding]
		SELECT
			@NamedListID,
			T.c.value('UDFIndex[1]','int'),
			T.c.value('UDFCaption[1]','nvarchar(256)')
		FROM
			@Xml.nodes('/NamedListDataSet/NamedListBinding') AS T(c)
		WHERE 
			T.c IS NOT NULL AND
			T.c.value('NamedListID[1]','int') = @ID
			
		IF @@error != 0 GOTO ErrorHandler 
			
		INSERT INTO [NamedListEntry] ([NamedListID], [Code])
		SELECT
			@NamedListID,
			T.c.value('Code[1]','varchar(32)')
		FROM
			@Xml.nodes('/NamedListDataSet/NamedListEntry') AS T(c)
		WHERE 
			T.c IS NOT NULL AND
			T.c.value('NamedListID[1]','int') = @ID

		IF @@error != 0 GOTO ErrorHandler
	
		FETCH NEXT FROM NamedListCursor INTO @ID, @Name, @SortOrderAscending
	END			
	
	COMMIT TRANSACTION

	CLOSE NamedListCursor
	DEALLOCATE NamedListCursor

	RETURN 0
	
ErrorHandler:
	CLOSE NamedListCursor
	DEALLOCATE NamedListCursor

	ROLLBACK TRANSACTION
	RETURN 1
GO

GRANT EXEC ON NamedListUpdate TO PUBLIC
GO
