IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5NotificationTemplateGet]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5NotificationTemplateGet'
		DROP PROCEDURE [dbo].[pjm5NotificationTemplateGet]
	END
GO

PRINT 'Creating procedure pjm5NotificationTemplateGet'
GO

/******************************************************************************
**		File: pjm5NotificationTemplateGet.sql
**		Name: pjm5NotificationTemplateGet
**		Desc: 
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: 
**		Date: 20/12/2005 13:00:00
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm5NotificationTemplateGet]
	@DocumentClass TClass,
	@DocumentID TIdentifier
AS
	SET NOCOUNT ON
	
	DECLARE @Lang TLanguage
	SET @Lang = dbo.adfGetLanguage()
	
	SELECT
		[SubscriptionType].[ID],
		ISNULL([SubscriptionTypeGlobal].[FileAs], [SubscriptionType].[FileAs]) AS [FileAs],
		[SubscriptionType].[DocumentClass],
		[SubscriptionType].[Aggregatable]
	FROM
		[SubscriptionType]
		LEFT OUTER JOIN [SubscriptionTypeGlobal]
				ON ([SubscriptionType].[ID] = [SubscriptionTypeGlobal].[ID]
					AND [SubscriptionTypeGlobal].[Language] = @Lang)
	
	--    
	IF @DocumentClass IS NULL OR @DocumentID IS NULL
	BEGIN		
		SELECT
			[NotificationMessageTemplate].[ID],
			[NotificationMessageTemplate].[SubscriptionTypeID],
			[NotificationMessageTemplate].[DocumentClass],
			[NotificationMessageTemplate].[DocumentID],
			[NotificationMessageTemplate].[NotificationWayID],
			ISNULL([NotificationMessageTemplateGlobal].[SimpleMessageTheme], 
				[NotificationMessageTemplate].[SimpleMessageTheme]) AS [SimpleMessageTheme],
			ISNULL([NotificationMessageTemplateGlobal].[SimpleMessageBody],
				[NotificationMessageTemplate].[SimpleMessageBody]) AS [SimpleMessageBody],
			ISNULL([NotificationMessageTemplateGlobal].[AggregatedMessageTheme],
				[NotificationMessageTemplate].[AggregatedMessageTheme]) AS [AggregatedMessageTheme],
			ISNULL([NotificationMessageTemplateGlobal].[AggregatedMessageBody],
				[NotificationMessageTemplate].[AggregatedMessageBody]) AS [AggregatedMessageBody]
		FROM
			[NotificationMessageTemplate]
			LEFT OUTER JOIN [NotificationMessageTemplateGlobal]
				ON ([NotificationMessageTemplate].[ID] = [NotificationMessageTemplateGlobal].[ID]
					AND [NotificationMessageTemplateGlobal].[Language] = @Lang)
		WHERE
			[NotificationMessageTemplate].[DocumentClass] IS NULL
			AND [NotificationMessageTemplate].[DocumentID] IS NULL
		RETURN 0
	END
	
	--    
	IF NOT EXISTS (
		SELECT * FROM [NotificationMessageTemplate]
		WHERE [DocumentClass] = @DocumentClass AND [DocumentID] = @DocumentID
 	)
	BEGIN
		--     
		BEGIN TRANSACTION
		INSERT INTO [NotificationMessageTemplate]
			([SubscriptionTypeID], [DocumentClass],	[DocumentID], [NotificationWayID],
			[SimpleMessageTheme], [SimpleMessageBody], [AggregatedMessageTheme], [AggregatedMessageBody])
		SELECT
			[SubscriptionTypeID], @DocumentClass, @DocumentID, [NotificationWayID],
			[SimpleMessageTheme], [SimpleMessageBody], [AggregatedMessageTheme], [AggregatedMessageBody]
		FROM [NotificationMessageTemplate]
		WHERE [DocumentClass] IS NULL AND [DocumentID] IS NULL
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
		INSERT INTO [NotificationMessageTemplateGlobal]
			([ID], [Language], [SimpleMessageTheme], [SimpleMessageBody],
			 [AggregatedMessageTheme], [AggregatedMessageBody])
		SELECT
			[NotificationMessageTemplate].[ID], [NotificationMessageTemplateGlobal].[Language],
			[NotificationMessageTemplateGlobal].[SimpleMessageTheme],
			[NotificationMessageTemplateGlobal].[SimpleMessageBody],
			[NotificationMessageTemplateGlobal].[AggregatedMessageTheme],
			[NotificationMessageTemplateGlobal].[AggregatedMessageBody]
		FROM
			[NotificationMessageTemplate]
			INNER JOIN [NotificationMessageTemplate] AS [GlobalTemplate]
				ON ([NotificationMessageTemplate].[SubscriptionTypeID] = [GlobalTemplate].[SubscriptionTypeID]
				AND [NotificationMessageTemplate].[NotificationWayID] = [GlobalTemplate].[NotificationWayID]
				AND [GlobalTemplate].[DocumentClass] IS NULL AND [GlobalTemplate].[DocumentID] IS NULL)
			INNER JOIN [NotificationMessageTemplateGlobal]
				ON [NotificationMessageTemplateGlobal].[ID] = [GlobalTemplate].[ID]		
		WHERE [NotificationMessageTemplate].[DocumentClass] = @DocumentClass
			AND [NotificationMessageTemplate].[DocumentID] = @DocumentID
		IF @@error != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
		COMMIT TRANSACTION
	END
	
	SELECT
		[NotificationMessageTemplate].[ID],
		[NotificationMessageTemplate].[SubscriptionTypeID],
		[NotificationMessageTemplate].[DocumentClass],
		[NotificationMessageTemplate].[DocumentID],
		[NotificationMessageTemplate].[NotificationWayID],
		ISNULL([NotificationMessageTemplateGlobal].[SimpleMessageTheme], 
			[NotificationMessageTemplate].[SimpleMessageTheme]) AS [SimpleMessageTheme],
		ISNULL([NotificationMessageTemplateGlobal].[SimpleMessageBody],
			[NotificationMessageTemplate].[SimpleMessageBody]) AS [SimpleMessageBody],
		ISNULL([NotificationMessageTemplateGlobal].[AggregatedMessageTheme],
			[NotificationMessageTemplate].[AggregatedMessageTheme]) AS [AggregatedMessageTheme],
		ISNULL([NotificationMessageTemplateGlobal].[AggregatedMessageBody],
			[NotificationMessageTemplate].[AggregatedMessageBody]) AS [AggregatedMessageBody]
	FROM
		[NotificationMessageTemplate]
		LEFT OUTER JOIN [NotificationMessageTemplateGlobal]
			ON ([NotificationMessageTemplate].[ID] = [NotificationMessageTemplateGlobal].[ID]
				AND [NotificationMessageTemplateGlobal].[Language] = @Lang)
	WHERE
		[NotificationMessageTemplate].[DocumentClass] = @DocumentClass
		AND [NotificationMessageTemplate].[DocumentID] = @DocumentID
	
	RETURN 0
GO

GRANT EXECUTE ON [pjm5NotificationTemplateGet] TO PUBLIC
GO 