IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[adfPaymentUpdate]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure adfPaymentUpdate'
		DROP PROCEDURE [dbo].[adfPaymentUpdate]
	END
GO

PRINT 'Creating procedure adfPaymentUpdate'
GO

/******************************************************************************
**		File: adfPaymentUpdate.sql
**		Name: adfPaymentUpdate
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: vladislav.dremin
**		Date: 01/21/2005 17:11:09
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		12.03.13	SOV				 OpenXml  Xml
*******************************************************************************/
CREATE PROCEDURE adfPaymentUpdate
	@Xml XML
AS
	DECLARE @ID TIdentifier,
			@FileAs TName,
			@Date TDate,
			@Number TCode,
			@PaymentTypeID TIdentifier,
			@PayeeCompanyID TIdentifier,
			@PayeeBankAccountID TIdentifier,
			@PayerCompanyID TIdentifier,
			@Value TMoney,
			@BaseCurrencyRate TMoney,
			@CurrencyCode TCode3,
			@VAT TMoney,
			@Comments TNotes,
			@Reason TName,
			@Payer TName
				
	SET NOCOUNT ON
	
	SELECT 
		@ID = XmlTable.value('ID[1]', 'int'),
		@Date = CONVERT(datetime, XmlTable.value('Date[1]', 'nvarchar(19)')),
		@FileAs = XmlTable.value('FileAs[1]', 'nvarchar(512)'),
		@Number = XmlTable.value('Number[1]', 'varchar(32)'),
		@PaymentTypeID = XmlTable.value('PaymentTypeID[1]', 'int'),
		@CurrencyCode = XmlTable.value('CurrencyCode[1]', 'char(3)'),
		@PayeeCompanyID = XmlTable.value('SupplierCompanyID[1]', 'int'),
		@PayeeBankAccountID = XmlTable.value('PayeeBankAccountID[1]', 'int'),
		@PayerCompanyID = XmlTable.value('CustomerCompanyID[1]', 'int'),
		@Value = XmlTable.value('Value[1]', 'money'),
		@VAT = XmlTable.value('VAT[1]', 'money'),
		@BaseCurrencyRate = XmlTable.value('BaseCurrencyRate[1]', 'money'),
		@Comments = XmlTable.value('Comments[1]', 'nvarchar(max)'),
		@Reason = XmlTable.value('Reason[1]', 'nvarchar(256)'),
		@Payer = XmlTable.value('Payer[1]', 'nvarchar(256)')
	FROM
		@Xml.nodes('/adfPaymentGet/Payment') AS List(XmlTable)

	SET @Date = CAST(CONVERT(varchar(8), @Date, 112) AS DATETIME)
	
	IF @PaymentTypeID = 1 --  
	BEGIN
		SET @Payer = NULL
	END
	ELSE IF @PaymentTypeID = 2 --  
	BEGIN
		SET @PayeeBankAccountID = NULL
	END
	ELSE -- Invalid payment type
	BEGIN
		RAISERROR('#internalerror', 16, -1)
		RETURN 1
	END	

	BEGIN TRANSACTION
	UPDATE 
		[Payment]
	SET
		[Payment].[FileAs] = @FileAs,
		[Payment].[Date] = @Date,
		[Payment].[Number] = @Number,
		[Payment].[PaymentTypeID] = @PaymentTypeID,
		[Payment].[CurrencyCode] = @CurrencyCode,
		[Payment].[PayeeCompanyID] = @PayeeCompanyID,
		[Payment].[PayeeBankAccountID]= @PayeeBankAccountID,
		[Payment].[PayerCompanyID] = @PayerCompanyID,
		[Payment].[Value] = @Value,
		[Payment].[VAT] = @VAT,
		[Payment].[BaseCurrencyRate] = @BaseCurrencyRate,
		[Payment].[Comments] = @Comments,
		[Payment].[Reason] = @Reason,
		[Payment].[Payer] = @Payer,
		[Payment].[Modified] = DEFAULT,
		[Payment].[ModifiedBy] = DEFAULT
	WHERE
		[Payment].[ID] = @ID

	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	COMMIT TRANSACTION

	EXEC adfPaymentGet @ID

	RETURN 0
GO

GRANT EXECUTE ON adfPaymentUpdate TO PUBLIC
GO
 