IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportFinanceResults')
	BEGIN
		PRINT 'Dropping Procedure ReportFinanceResults'
		DROP  Procedure  ReportFinanceResults
	END

GO

PRINT 'Creating Procedure ReportFinanceResults'
GO

/*******************************************************************************
**		Change History
*******************************************************************************
**		26.02.14	BAO		" "     " "
**		26.02.14	BAO		  "" ( @InvoiceSelector)
*******************************************************************************/
CREATE Procedure ReportFinanceResults
	@CompanyID			 TIdentifier,
	@DateSelector		 int,
	@PeriodStart		 TDateTime,
	@PeriodEnd			 TDateTime,
	@ServiceTypeSelector int,
	@InvoiceSelector	 int
AS
	SET NOCOUNT ON
	
	IF @CompanyID IS NULL
	BEGIN
		RAISERROR('#CompanyIsNotSelected', 16, -1)
		RETURN 1
	END
	
	DECLARE 
		@UserID TIdentifier,
		@EmployeeID TIdentifier,
		@Name TName,
		@Year int,
		@ServiceType TName,
		@InvoiceState TName
	
	IF @DateSelector = 0
		SET @Year = YEAR(GETDATE())
	ELSE IF @DateSelector = 1
		SET @Year = YEAR(GETDATE()) - 1
	ELSE
		SET @Year = YEAR(@PeriodStart)
		
	IF @ServiceTypeSelector = 0 SET @ServiceType = '    '
	IF @ServiceTypeSelector = 1 SET @ServiceType = ''
	IF @ServiceTypeSelector = 2 SET @ServiceType = '  '

	IF @InvoiceSelector = 0
		SET @InvoiceState = ' '
	ELSE 
		SET @InvoiceState = ' '
	
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	IF @EmployeeID IS NULL
	BEGIN
		SET @UserID = dbo.adfGetCurrentUserID()
		SELECT
			@Name = [Trustee].[DisplayName]
		FROM
			[Trustee]
		WHERE
			[Trustee].[ID] = @UserID
	END
	ELSE
	BEGIN
		SELECT
			@Name = [Employee].[FileAs]
		FROM
			[Employee]
		WHERE
			[Employee].[ID] = @EmployeeID
	END
	
	SELECT
		[Company].[ID],
		[Company].[LegalName],
		[Company].[FileAs],
		@Year AS [Year],
		0 AS [PaidOnly],
		@Name AS [Name],
		@ServiceType AS [ServiceType],
		@InvoiceState AS [InvoiceState]
	FROM
		[Company]
	WHERE
		[Company].[ID] = @CompanyID

	DECLARE @data TABLE([ServiceTypeID] int null, 
		[ServiceTypeCode] nvarchar(256) null,
		[ServiceTypeFileAs] nvarchar(256) null,
		[Month] int,
		[Value] money)
	INSERT INTO @data ([ServiceTypeID],[ServiceTypeCode],[ServiceTypeFileAs],[Month],[Value])
	SELECT 
		[ServiceType].[ID],
		[ServiceType].[Code],
		[ServiceType].[FileAs],
		MONTH([Invoice].[Date]),
		SUM([InvoiceLine].[TotalWithVATBC])
	FROM
		[Invoice]
		INNER JOIN [InvoiceLine]
			ON [InvoiceLine].[InvoiceID] = [Invoice].[ID]
		LEFT OUTER JOIN [ServiceType]
			ON [ServiceType].[ID] = [InvoiceLine].[ServiceTypeID]
		LEFT OUTER JOIN [AdvanceInvoice]
			ON [Invoice].[ID] = [AdvanceInvoice].[AdvanceInvoiceID]
		LEFT OUTER JOIN [Invoice] AS [BaseInvoice]
			ON [BaseInvoice].[ID] = [AdvanceInvoice].[InvoiceID] AND 
				[BaseInvoice].[State] IN (2, 5)
	WHERE
		[Invoice].[SupplierCompanyID] = @CompanyID AND
		YEAR([Invoice].[Date]) = @Year AND
		[Invoice].[State] IN (2, 5) AND
		[BaseInvoice].[ID] IS NULL
		AND
		(
			@ServiceTypeSelector = 0 OR
			@ServiceTypeSelector = 1 AND ([ServiceType].[RatesDepended] = 1 OR [ServiceType].[ID] IS NULL) OR
			@ServiceTypeSelector = 2 AND [ServiceType].[RatesDepended] = 0
		)
		AND
		(
		 @InvoiceSelector = 0 AND [Invoice].[State] = 5 OR
		 @InvoiceSelector = 1 AND [Invoice].[State] IN (2,5)
		)
	GROUP BY 
		[ServiceType].[ID],
		[ServiceType].[Code],
		[ServiceType].[FileAs],
		MONTH([Invoice].[Date])
	DECLARE @Total TMoney
	SELECT @Total = SUM([Value]) FROM @data

	SELECT
		ISNULL([ServiceTypeID], 0) AS [ServiceTypeID],
		ISNULL([ServiceTypeCode], '( )') AS [ServiceTypeCode],
		ISNULL([ServiceTypeFileAs], '( )') AS [ServiceTypeFileAs],
		[1],[2],[3],[4],[5],[6],[7],[8],[9],[10],[11],[12],
		(ISNULL([1],0)+ISNULL([2],0)+ISNULL([3],0)+ISNULL([4],0)+ISNULL([5],0)+ISNULL([6],0)+
		ISNULL([7],0)+ISNULL([8],0)+ISNULL([9],0)+ISNULL([10],0)+ISNULL([11],0)+ISNULL([12],0)) AS [Total]
	FROM
		@data
	PIVOT
	(
		SUM([Value])
		FOR [Month] IN ([1],[2],[3],[4],[5],[6],[7],[8],[9],[10],[11],[12])
	) AS pvt


	;
	WITH TABLE1 ([SortKey], [ServiceTypeID], [ServiceTypeCode], [ServiceTypeFileAs], [Value]) AS
	(
		SELECT TOP 9
			1 AS [SortKey],
			[ServiceTypeID], 
			ISNULL([ServiceTypeCode], '( )'),
			ISNULL([ServiceTypeFileAs], '( )'),
			SUM([Value]) AS [Value]
		FROM
			@data
		GROUP BY
			[ServiceTypeID], 
			[ServiceTypeCode],
			[ServiceTypeFileAs]
		HAVING
			SUM([Value]) > @Total / 20
		ORDER BY 5 DESC
	)
	SELECT
		[SortKey], [ServiceTypeID], [ServiceTypeCode], [ServiceTypeFileAs], [Value]
	FROM TABLE1
	UNION ALL
	SELECT
		2, 0, '()', '()',
		@Total - (SELECT SUM([Value]) FROM TABLE1)
	WHERE
		@Total - (SELECT SUM([Value]) FROM TABLE1) > 0
	ORDER BY
		1, 5 DESC
RETURN 0
GO

GRANT EXEC ON ReportFinanceResults TO PUBLIC

GO
