 IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportOvertimeWorkByEmployee')
	BEGIN
		PRINT 'Dropping Procedure ReportOvertimeWorkByEmployee'
		DROP  Procedure  ReportOvertimeWorkByEmployee
	END

GO

PRINT 'Creating Procedure ReportOvertimeWorkByEmployee'
GO

--------------------------------------------------------------------------------------
CREATE Procedure ReportOvertimeWorkByEmployee
				@EmployeeID TIdentifier,
				@DateSelector int,
				@StartDate TDate,
				@EndDate TDate
		 
AS
	SET NOCOUNT ON

SELECT 
		@EmployeeID = [ID]
	FROM 
		[Employee] 
	WHERE 
		([Employee].[ID] = @EmployeeID) OR
		(@EmployeeID IS NULL AND [UserID] = dbo.adfGetCurrentUserID())

DECLARE
		@UserID TIdentifier,
		@UserName TName
SET @UserID = dbo.adfGetCurrentUserID()
SELECT
		@UserName = [Trustee].[DisplayName]
	FROM
		[Trustee]
	WHERE
		[Trustee].[ID] = @UserID

	IF @DateSelector = 0 --last month
		BEGIN
			SELECT
				@StartDate = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
				@StartDate = DATEADD(day, 1-DATEPART(day, @StartDate), @StartDate),
				@StartDate = DATEADD(month, -1, @StartDate),
				@EndDate = DATEADD(day, -1, DATEADD(month, 1, @StartDate))
		END
		ELSE IF @DateSelector = 1 -- current month
			BEGIN
			SELECT
				@StartDate = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime),
				@StartDate = DATEADD(day, 1-DATEPART(day, @StartDate), @StartDate),
				@EndDate = DATEADD(day, -1, DATEADD(month, 1, @StartDate))
			END


		DECLARE @WorkTimeStart1 INT, @WorkTimeStart2 INT,
				@WorkTimeEnd1 INT, @WorkTimeEnd2 INT,
				@Value TVariant

		EXEC adfParameterGet 'WorkTimeStart1', 'Pjm5', @Value OUTPUT
		SET @WorkTimeStart1 = CAST(@Value as INT)

		EXEC adfParameterGet 'WorkTimeEnd1', 'Pjm5', @Value OUTPUT
		SET @WorkTimeEnd1 = CAST(@Value as INT)

		EXEC adfParameterGet 'WorkTimeStart2', 'Pjm5', @Value OUTPUT
		SET @WorkTimeStart2 = ISNULL(CAST(@Value as INT), 0)

		EXEC adfParameterGet 'WorkTimeEnd2', 'Pjm5', @Value OUTPUT
		SET @WorkTimeEnd2 = ISNULL(CAST(@Value as INT), 0)

SELECT
	@UserName AS [UserName],
	GETDATE() AS [Date],
	@StartDate AS [PeriodStart],
	@EndDate AS [PeriodEnd],
	[Company].[FileAs] AS [CompanyFileAs],
	[Employee].[FileAs] AS [EmployeeFileAs],
	dbo.fnPjm5GetWorkingDays(@StartDate ,@EndDate)*((@WorkTimeEnd1 - @WorkTimeStart1) + (@WorkTimeEnd2 - @WorkTimeStart2))/60.0 AS [WorkingHours],
	dbo.fnPjm5GetWorkingDays(@StartDate ,@EndDate) AS [WorkingDays]
FROM [Employee]
	LEFT OUTER JOIN [Company]
	ON [Company].[ID] = [Employee].[CompanyID]
WHERE [Employee].[ID] = @EmployeeID	

SELECT
	[Project].[ID],
	[Project].[FileAs] AS [ProjectFileAs],
	[Project].[Number] AS [ProjectNumber],
	SUM(CASE WHEN [TimeEntry].[State] = 2 THEN [TimeEntry].[Duration] ELSE 0 END)/60.0 AS [SentForApprovalDuration],
	SUM(CASE WHEN [TimeEntry].[State] = 4 THEN [TimeEntry].[Duration] ELSE 0 END)/60.0 AS [RejectedDuration],
	SUM(CASE WHEN [TimeEntry].[State] = 3 THEN [TimeEntry].[Duration] ELSE 0 END)/60.0 AS [ApprovedDuration]
FROM [TimeEntry]
	INNER JOIN [Project]
		ON [TimeEntry].[ProjectID] = [Project].[ID]
	INNER JOIN [Company]
		ON [Company].[ID]=[Project].[SupplierCompanyID]
WHERE 
	[TimeEntry].[Overtime] = 1 AND
	[TimeEntry].[EmployeeID] = @EmployeeID
	AND	CONVERT(datetime, CONVERT(varchar(8), [TimeEntry].[StartTime], 112), 112) BETWEEN @StartDate AND @EndDate 
	AND	[Project].[State] NOT IN (1,5) 
GROUP BY 
	[Project].[ID],
	[Project].[Number],
	[Project].[FileAs]
		
	RETURN 0
GO

GRANT EXEC ON ReportOvertimeWorkByEmployee TO PUBLIC
GO  