IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportEmployeeTimeAllocationByProject')
	BEGIN
		PRINT 'Dropping Procedure ReportEmployeeTimeAllocationByProject'
		DROP  Procedure  ReportEmployeeTimeAllocationByProject
	END

GO

PRINT 'Creating Procedure ReportEmployeeTimeAllocationByProject'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
   
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**	--------		--------			-------------------------------------------
**  18.01.10		SYV					
**	30.08.13		BAO					  ""  Duration  
**										   [TimeEntries].[State] = 6
**										()
**	04.09.13		BAO					  ""  Duration   
**										   ""  "
**										  "
**	09.09.13		BAO					  ""   
**										
**	16.01.14		BAO					   
**										,    
**										. -
**	20.01.13		BAO					    
**										   " .. "
**	21.07.14		BAO					 ,    
**										"  "
--------------------------------------------------------------------------------------*/
CREATE PROCEDURE [dbo].[ReportEmployeeTimeAllocationByProject]
        @CompanyID			TIdentifier,
        @DivisionSelector	int,
        @DivisionID			TIdentifier,
        @DateSelector		int,
        @Date				TDate,
        @PeriodStart		TdateTime,
		@PeriodEnd			TdateTime,
		@IncludingDrafts	int --       -?
		
AS

SET NOCOUNT ON

	DECLARE 
		@TimeEntryClass as TClass,
		@CalculationClass TClass,
		@StatementClass TClass,
		@InvoiceClass TClass
	
	SET @TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	SET	@CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET	@StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
	SET	@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	
	IF @DivisionID IS NULL AND @DivisionSelector=2 
	BEGIN
		RAISERROR('#DivisionIsNotSelected', 16, -1)
		RETURN 2
	END

EXEC ReportEmployeeTimeAllocationByProjectHeader @CompanyID,
                                                 @DivisionSelector,
                                                 @DivisionID,
                                                 @DateSelector,
                                                 @Date, 
                                                 @PeriodStart = @PeriodStart OUTPUT,
                                                 @PeriodEnd = @PeriodEnd OUTPUT 
												 
												 
DECLARE @Employees
TABLE
(
	ID	TIdentifier	
)	
INSERT INTO @Employees
SELECT
	[Employee].[ID]
FROM
	[Employee]
	LEFT OUTER JOIN [TimeEntry]
		ON [TimeEntry].[EmployeeID] = [Employee].[ID]
		AND [TimeEntry].[StartTime] BETWEEN @PeriodStart AND @PeriodEnd
	LEFT OUTER JOIN [EntryAmount]
		ON [EntryAmount].[ID] = [TimeEntry].[ID]
	LEFT OUTER JOIN [Project]
		ON [Project].[ID] = [TimeEntry].[ProjectID]
WHERE
	[TimeEntry].[ProjectID] IS NOT NULL AND	
	(
		[Project].[SupplierCompanyID] = @CompanyID OR
		@CompanyID IS NULL
	) AND
	(
		(@DivisionSelector = 2 AND [Employee].[DivisionID] = @DivisionID) OR
		(@DivisionSelector = 1 AND [Employee].[DivisionID] IS NULL) OR
		(@DivisionSelector = 0)
	) AND
	(
		[Employee].[HideInReports] = 0
	)
GROUP BY
	[Employee].[ID]												 
												 
												 

--   ,    
DECLARE @TimeEntriesInFinance 
TABLE
(
	[DocumentID]			TIdentifier,
	[LinkClass]				TClass, 
	[LinkDocumentID]		TIdentifier, 
	[State]					TState, 
	[Date]					TDateTime, 
	[ThroughCalculation]	TFlag
)


--      ID 
DECLARE 
	@tmp_employeeID				TIdentifier

DECLARE insertCursor CURSOR LOCAL FAST_FORWARD FOR
SELECT
	[@Employees].[ID]
FROM
	@Employees

OPEN insertCursor
FETCH NEXT FROM insertCursor INTO @tmp_employeeID
WHILE @@FETCH_STATUS = 0
BEGIN
	
	INSERT INTO @TimeEntriesInFinance([DocumentID], [LinkClass], [LinkDocumentID], [State], [ThroughCalculation])
	SELECT 
		[F].[DocumentID],
		[F].[LinkClass], 
		[F].[LinkDocumentID], 
		[F].[State], 
		[F].[ThroughCalculation]
	FROM
		fnReportGetTimeEntriesInFinance(@PeriodStart, @PeriodEnd, @tmp_employeeID) AS [F]

	FETCH NEXT FROM insertCursor INTO @tmp_employeeID
END
CLOSE insertCursor
DEALLOCATE insertCursor
											                          

DECLARE @Columns TABLE(Number int)


DECLARE @EmployeeTimeAllocation TABLE(
                                     TimeEntryState TIdentifier,
                                     EmployeeID TIdentifier,
                                     EmployeeName TName,
                                     ProjectID TIdentifier,
                                     ProjectName TName,
                                     Duration TMoney)

DECLARE @TimeEntries TABLE(
					[ID] TIdentifier,
					[EmployeeID] TIdentifier,
					[EmployeeFileAs] TName,
				    [Working] TState,
					[ProjectID] TIdentifier, 
					[ProjectCode] TCode, 
					[State] TState,
					[Duration] TMoney,
					[RoundedDuration] TMoney,
					[BillableDuration] TMoney)


INSERT INTO @TimeEntries(
					[ID],
					[EmployeeID],
					[EmployeeFileAs],
				    [Working],
					[ProjectID], 
					[ProjectCode], 
					[State],
					[Duration],
					[RoundedDuration],
					[BillableDuration])
SELECT
	[TimeEntry].[ID],
	[Employee].[ID],
	[Employee].[FileAs],
	[Employee].[State],
	[TimeEntry].[ProjectID],
	[Project].[Number],
	[TimeEntry].[State],
	SUM([TimeEntry].[Duration]/60.0) as [Duration],
	SUM([TimeEntry].[RoundedDuration]/60.0) as [RoundedDuration],
	CASE [TimeEntry].[Billable] WHEN 1 THEN 
		SUM([EntryAmount].[Amount]) ELSE NULL END as [BillableDuration]
FROM
	[Employee]
	LEFT OUTER JOIN [TimeEntry]
		ON [TimeEntry].[EmployeeID] = [Employee].[ID]
		AND [TimeEntry].[StartTime] BETWEEN @PeriodStart AND @PeriodEnd
	LEFT OUTER JOIN [EntryAmount]
		ON [EntryAmount].[ID] = [TimeEntry].[ID]
	LEFT OUTER JOIN [Project]
		ON [Project].[ID] = [TimeEntry].[ProjectID]
WHERE
	[TimeEntry].[ProjectID] IS NOT NULL AND	
	(
		[Project].[SupplierCompanyID] = @CompanyID OR
		@CompanyID IS NULL
	) AND
	(
		(@DivisionSelector = 2 AND [Employee].[DivisionID] = @DivisionID) OR
		(@DivisionSelector = 1 AND [Employee].[DivisionID] IS NULL) OR
		(@DivisionSelector = 0)
	) AND
	(
		[Employee].[HideInReports] = 0
	)
GROUP BY
	[Employee].[ID],
	[Employee].[FileAs],
	[Employee].[State],
	[TimeEntry].[ProjectID],
	[Project].[Number],
	[TimeEntry].[State],
	[TimeEntry].[Billable],
	[TimeEntry].[ID]


-- ,   ""
INSERT INTO @EmployeeTimeAllocation(TimeEntryState,
                                   EmployeeID, EmployeeName,
                                   ProjectID, ProjectName,
                                   Duration)
SELECT 
	4 AS [ColumnNumbers],
	[TimeEntries].[EmployeeID],
	[TimeEntries].[EmployeeFileAs],
	[TimeEntries].[ProjectID],
	[TimeEntries].[ProjectCode],
	CASE WHEN [TimeEntries].[State] IN (3,6)
		THEN ISNULL([TimeEntries].[RoundedDuration], 0) 
		ELSE 0 
	END
FROM 
	@TimeEntries AS [TimeEntries]
WHERE
	[TimeEntries].[Working] = 1 OR [TimeEntries].[RoundedDuration] > 0

-- ,   ""
INSERT INTO @EmployeeTimeAllocation(TimeEntryState,
                                   EmployeeID, EmployeeName,
                                   ProjectID, ProjectName,
                                   Duration)
SELECT 
	6 AS [ColumnNumbers],
	[TimeEntries].[EmployeeID],
	[TimeEntries].[EmployeeFileAs],
	[TimeEntries].[ProjectID],
	[TimeEntries].[ProjectCode],
	CASE WHEN [TimeEntries].[State] IN (6) 
				AND [TimeEntries].[BillableDuration] IS NOT NULL
				AND (@IncludingDrafts = 1 OR ([TimeEntriesInFinance].[State] = 2 OR 
						(@InvoiceClass = [TimeEntriesInFinance].[LinkClass] AND [TimeEntriesInFinance].[State] = 5) OR
						(@InvoiceClass != [TimeEntriesInFinance].[LinkClass] AND [TimeEntriesInFinance].[State] = 3))) --     
		THEN  ISNULL([TimeEntries].[BillableDuration], 0)
	ELSE 0 END
FROM 
	@TimeEntries AS [TimeEntries]
	LEFT OUTER JOIN
	@TimeEntriesInFinance AS [TimeEntriesInFinance]
		ON [TimeEntriesInFinance].[DocumentID] = [TimeEntries].[ID]
WHERE
	[TimeEntries].[Working] = 1 OR [TimeEntries].[Duration] > 0
	AND ([TimeEntriesInFinance].ThroughCalculation = 0 OR [TimeEntriesInFinance].ThroughCalculation IS NULL)

SELECT 
	[TE].* 
FROM 
	@EmployeeTimeAllocation AS [TE]
INNER JOIN [Project]
		ON [Project].[ID] = [TE].[ProjectID]
ORDER BY
	[TE].[TimeEntryState],
	[Project].[ProjectTypeID],
	[Project].[CostCenterID],
	[TE].[ProjectName]


SELECT 
	EmployeeID as EmpID, 
	EmployeeName as EmpName,
	TimeEntryState TimeState,
	ISNULL(SUM(Duration), 0) as TotalDuration
FROM @EmployeeTimeAllocation
GROUP BY TimeEntryState, EmployeeID, EmployeeName



RETURN 0

GO

GRANT EXEC ON ReportEmployeeTimeAllocationByProject TO PUBLIC

GO

