IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportEmployeeTimeAndRevenueByProject')
	BEGIN
		PRINT 'Dropping Procedure ReportEmployeeTimeAndRevenueByProject'
		DROP  Procedure  ReportEmployeeTimeAndRevenueByProject
	END
GO

PRINT 'Creating Procedure ReportEmployeeTimeAndRevenueByProject'
GO

/******************************************************************************
**		File: ReportEmployeeTimeAndRevenueByProject.sql
**		Name: ReportEmployeeTimeAndRevenueByProject
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: MAN
**		Date: 22/11/2010 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		22.11.2010	MAN				First Version
**		25.04.2013	SAP				    
**		16.01.2013	BAO				   
**									,    
**									. -
**		06.05.2014	BAO				     " "
**									    
**		07.05.2014	BAO				    
**									 
**		21.07.2014	BAO				 ,    
**									"  "
*******************************************************************************/

CREATE PROCEDURE [dbo].[ReportEmployeeTimeAndRevenueByProject]
	@CompanyID				TIdentifier,	-- ID 
    @CostCenterID			TIdentifier,
    @CostCenterSelector		TIdentifier,
    @ProjectID				TIdentifier,
	@DateSelector			TIdentifier,	--    (SelectedIndex  )
    @PeriodStart			TDateTime,		--   
    @PeriodEnd				TDateTime,		--   
    @EmployeeTypeSelector	TIdentifier,	--   0 - , 1 - 
    @EmployeeID				TIdentifier,
    @ConvertionType			TIdentifier,
    @CurrencySelector		TAmount,
    @DeviationType			TIdentifier,
	@IncludingDrafts		int --       -?
AS

SET NOCOUNT ON

--  
IF  dbo.adfHasAccessRightByName('ViewEmployeeTimeAndRevenueByProjectsReport') = 0 
BEGIN
      RAISERROR('#NotEnoughPermissions', 18, -1)
      RETURN 1
END

DECLARE
	@PeriodName		TName, --  
	@Number			TIdentifier,
	@MAXValue		TAmount,
	@Rate			FLOAT,
	@CalculationClass TClass,
	@StatementClass TClass,
	@InvoiceClass TClass
		
SET	@CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
SET	@StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
SET	@InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')

-- 
EXEC ReportGetPeriod 
	@DateSelector,			--   
	0,
	@PeriodStart OUTPUT,	--  
	@PeriodEnd	 OUTPUT,	--  
	@PeriodName  OUTPUT		--  
	
DECLARE
	@TimeEntryClass TClass
	SET	@TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')

DECLARE 
	@CurrencyCode TCode3
	--      	
SET @CurrencyCode = 
	CASE @CurrencySelector
		WHEN 0 THEN dbo.fnPjm5GetBaseCurrency()
		WHEN 1 THEN dbo.fnPjm5GetNationalCurrency()
	END
	
IF @CostCenterSelector = 0
	SET @CostCenterID = -1

DECLARE @AvailableProjects TABLE
(
	ProjectID TIdentifier
)
INSERT INTO @AvailableProjects (ProjectID)
	SELECT
		[ProjectID]
	FROM
		fnReportGetAvailableProjects 
		(
			@CompanyID,
			@CostCenterID,
			NULL,
			@ProjectID,
			@PeriodStart,
			@PeriodEnd,
			0,
			1
		)
	WHERE
		[ViewFinance] = 1

--   ,    
DECLARE @TimeEntriesInFinance 
TABLE
(
	[DocumentID]			TIdentifier,
	[LinkClass]				TClass, 
	[LinkDocumentID]		TIdentifier, 
	[State]					TState, 
	[Date]					TDateTime, 
	[ThroughCalculation]	TFlag
)

INSERT INTO @TimeEntriesInFinance([DocumentID], [LinkClass], [LinkDocumentID], [State], [ThroughCalculation])
SELECT 
	[F].[DocumentID],
	[F].[LinkClass], 
	[F].[LinkDocumentID], 
	[F].[State], 
	[F].[ThroughCalculation]
FROM
fnReportGetTimeEntriesInFinance(@PeriodStart, @PeriodEnd, @EmployeeID) AS [F]

DECLARE @Result TABLE
(
	EmployeeName TName,
	ProjectID TIdentifier,
	ProjectNumber TCode,
	ProjectFileAs TName,
	TimeRegistered TAmount,
	TimeBillable TAmount,
	TimeSoldTotal TAmount,
	RevenueTotal TMoney,
	RevenueDeviation TMoney
)
INSERT INTO @Result
(
	EmployeeName,
	ProjectID,
	ProjectNumber,
	ProjectFileAs,
	TimeRegistered,
	TimeBillable,
	TimeSoldTotal,
	RevenueTotal,
	RevenueDeviation
)
SELECT
	[Employee].[FileAS] AS [EmployeeName],
	[Project].[ID] AS [ProjectID],
	[Project].[Number] AS [ProjectNumber],
	[Project].[FileAS] AS [ProjectFileAS],
	SUM
	(
		CASE 
			WHEN [vwTimeEntry].[State] IN (3,6) -- , 
				THEN [vwTimeEntry].[RoundedDuration] / 60.0
			ELSE 0
		END
	) AS [TimeRegistered],
	SUM
	(
		CASE
			WHEN 
				[vwTimeEntry].[State] IN (3,6)  -- , 
				 AND 
				[vwTimeEntry].[Billable] = 1  --  
				THEN [vwTimeEntry].[RoundedDuration] / 60.0
			ELSE 0
		END
	) AS [TimeBillable],
	SUM
	(
		CASE
			WHEN ([vwTimeEntry].[State] IN (6) 
					AND (@IncludingDrafts = 1 OR ([TimeEntriesInFinance].[State] = 2 OR 
						(@InvoiceClass = [TimeEntriesInFinance].[LinkClass] AND [TimeEntriesInFinance].[State] = 5) OR
						(@InvoiceClass != [TimeEntriesInFinance].[LinkClass] AND [TimeEntriesInFinance].[State] = 3)))) --     
			THEN [vwTimeEntry].[BillableHour] 
			ELSE 0
		END
	) AS [TimeSoldTotal],
	SUM
	(
		CASE
			WHEN (([vwTimeEntry].[State] IN (6)) AND (@IncludingDrafts = 1 OR ([TimeEntriesInFinance].[State] IS NOT NULL AND [TimeEntriesInFinance].[State] <> 1))) THEN
				CASE 
						WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
							THEN ([vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] / ( 1 + [vwTimeEntry].[VATRate] ))
								/*>------------------------*/
								* 
								CASE [TimeEntriesInFinance].[LinkClass]
								WHEN @CalculationClass THEN
									CASE [C].[DiscountType]
									WHEN 1 THEN (1 - [C].[DiscountPercent])
									WHEN 2 THEN (1 + [C].[DiscountPercent])
									WHEN 3 THEN 
											CASE 
											WHEN [C].[TotalWithoutDiscountAndAdvanced] > [C].[Discount] THEN [C].[DiscountPercent]
											ELSE 1
											END
									WHEN 4 THEN [C].[DiscountPercent]
									ELSE 1 
									END
								ELSE 1
								END
								/*------------------------<*/
						WHEN
							( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0 )
							OR
							( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1 )
							THEN 
								[vwTimeEntry].[BillableHour] 
								* [vwTimeEntry].[Rate]
								/*>------------------------*/
								* 
								CASE [TimeEntriesInFinance].[LinkClass]
								WHEN @CalculationClass THEN
									CASE [C].[DiscountType]
									WHEN 1 THEN (1 - [C].[DiscountPercent])
									WHEN 2 THEN (1 + [C].[DiscountPercent])
									WHEN 3 THEN 
											CASE 
											WHEN [C].[TotalWithoutDiscountAndAdvanced] > [C].[Discount] THEN [C].[DiscountPercent]
											ELSE 1
											END
									WHEN 4 THEN [C].[DiscountPercent]
									ELSE 1 
									END
								ELSE 1
								END
								/*------------------------<*/
						WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
							THEN [vwTimeEntry].[BillableHour] * [vwTimeEntry].[Rate] * ( 1 + [vwTimeEntry].[VATRate] )
								/*>------------------------*/
								* 
								CASE [TimeEntriesInFinance].[LinkClass]
								WHEN @CalculationClass THEN
									CASE [C].[DiscountType]
									WHEN 1 THEN (1 - [C].[DiscountPercent])
									WHEN 2 THEN (1 + [C].[DiscountPercent])
									WHEN 3 THEN 
											CASE 
											WHEN [C].[TotalWithoutDiscountAndAdvanced] > [C].[Discount] THEN [C].[DiscountPercent]
											ELSE 1
											END
									WHEN 4 THEN [C].[DiscountPercent]
									ELSE 1 
									END
								ELSE 1
								END
								/*------------------------<*/
					END *
					dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
		END
	) AS [RevenueTotal],
	SUM
	(
		CASE 
			WHEN (([vwTimeEntry].[State] IN (3,6)) AND ([vwTimeEntry].[Billable] = 1))-- , 
				THEN 
					CASE 
						WHEN [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 1 
							THEN ([vwTimeEntry].[RoundedDuration] / 60.0 * [vwTimeEntry].[OriginalRate] / ( 1 + [vwTimeEntry].[VATRate] ))
						WHEN
							( [Project].[ApplyVAT] = 0 AND [Project].[RatesWithVAT] = 0  )
							OR
							( [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 1  )
							THEN ([vwTimeEntry].[RoundedDuration] / 60.0 * [vwTimeEntry].[OriginalRate])
						WHEN [Project].[ApplyVAT] = 1 AND [Project].[RatesWithVAT] = 0 
							THEN ([vwTimeEntry].[RoundedDuration] / 60.0 * [vwTimeEntry].[OriginalRate] * ( 1 + [vwTimeEntry].[VATRate] ))
					END *
					--   
						CASE 
							--      .
							WHEN [vwTimeEntry].[State] IN ( 3 )
								 THEN
									CASE @ConvertionType
										WHEN 0 --    
											THEN dbo.pjm6fnCurrencyGetRate([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[StartTime],@CurrencyCode)
										WHEN 1 --    
											THEN dbo.pjm6fnCurrencyGetRateMonth([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[StartTime],@CurrencyCode)
										WHEN 2 --   
											THEN dbo.pjm6fnCurrencyGetRate([vwTimeEntry].[CurrencyCode],DATEADD(ms,-5,DATEADD(mm, DATEDIFF(m,0,[vwTimeEntry].[StartTime])+1, 0)),@CurrencyCode)
									END
							ELSE
								--    .
								dbo.fnReportGetFinDocumentCurrency([vwTimeEntry].[CurrencyCode],[vwTimeEntry].[ID],@TimeEntryClass,@CurrencyCode)
						END
			ELSE 0
		END
	) AS [RevenueDeviation]
FROM
	[vwTimeEntry]
	INNER JOIN [Employee]
		ON [Employee].[ID] = [vwTimeEntry].[EmployeeID]
	INNER JOIN [Project]
		ON [vwTimeEntry].[ProjectID] = [Project].[ID]
	LEFT OUTER JOIN										--    ,   
	@TimeEntriesInFinance AS [TimeEntriesInFinance] 	
		ON [TimeEntriesInFinance].[DocumentID] = [vwTimeEntry].[ID]
	LEFT OUTER JOIN [Calculation] AS [C]
		ON [C].[ID] = [TimeEntriesInFinance].[LinkDocumentID] AND [LinkClass] = @CalculationClass
WHERE
	[vwTimeEntry].[State] IN (3,6)
	AND
	[vwTimeEntry].[StartTime] >= @PeriodStart
	AND
	[vwTimeEntry].[StartTime] < @PeriodEnd
	AND
	(
		[vwTimeEntry].[Duration] > 0
		OR
		[vwTimeEntry].[BillableDuration] > 0
	)
	AND
	(
		@EmployeeID IS NULL
		OR
		[vwTimeEntry].[EmployeeID] = @EmployeeID
	)
	AND
	(
		(
			@EmployeeTypeSelector = 0
			AND
			[Employee].[State] = 1
		)
		OR
		@EmployeeTypeSelector = 1
	)
	AND
	[Project].[ID] IN
	(
		SELECT
			[@AvailableProjects].[ProjectID]
		FROM
			@AvailableProjects
	) 
	AND
	(
		--    
			ISNULL([Project].[CostCenterID],-2) = ISNULL(@CostCenterID,-2)
			OR  
			@CostCenterID = -1
	)
	AND
	(
		[Employee].[HideInReports] = 0
	)
GROUP BY
	[Employee].[FileAS],
	[Project].[ID],
	[Project].[Number],
	[Project].[FileAS]

SET @MAXValue = 
	(
		SELECT
			MAX([MaxValues].[MaxVal])
		FROM
		(
			SELECT MAX(ISNULL([@Result].[RevenueTotal],0)) as MaxVal FROM @Result
			UNION
			SELECT	MAX(ISNULL([@Result].[RevenueDeviation],0) - ISNULL([@Result].[RevenueTotal],0)  )as MaxVal FROM @Result
		) AS MaxValues
)
-- SMART VALUE
SET @Rate = CASE  
                 WHEN @MAXValue < 100000 THEN 1
                 WHEN @MAXValue >= 100000 AND @MAXValue < 10000000 THEN 0.001
                 WHEN @MAXValue >= 10000000 AND @MAXValue < 10000000000 THEN 0.000001
                 WHEN @MAXValue >= 10000000000 THEN 0.000000001
            END
SELECT
	[@Result].[EmployeeName],
	[@Result].[ProjectID],
	[@Result].[ProjectNumber],
	[@Result].[ProjectFileAs],
	[@Result].[TimeRegistered],
	[@Result].[TimeBillable],
	[@Result].[TimeSoldTotal],
	[@Result].[RevenueTotal] * @Rate AS [RevenueTotal],
	[@Result].[RevenueDeviation] * @Rate AS [RevenueDeviation]
FROM
	@Result

----  
EXEC ReportEmployeeTimeAndRevenueByProjectHeader
	    @CompanyID,
        @CostCenterID,
        @ProjectID,
        @PeriodName,
        @PeriodStart,
        @PeriodEnd,
		@DeviationType,
		@ConvertionType,
		@EmployeeID,
		@CurrencyCode,
		@MaxValue
RETURN 0
GO

GRANT EXEC ON ReportEmployeeTimeAndRevenueByProject TO PUBLIC
GO

