	IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[ReportFuturePeriodsRealization]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure ReportFuturePeriodsRealization'
		DROP PROCEDURE [dbo].[ReportFuturePeriodsRealization]
	END
	GO

	PRINT 'Creating procedure ReportFuturePeriodsRealization'
	GO

	/******************************************************************************
	**		File: ReportFuturePeriodsRealization.sql
	**		Name: ReportFuturePeriodsRealization
	**		Desc: 
	**
	**              
	**		Return values:
	** 
	**		Called by:   
	**              
	**		Parameters:
	**		Input							Output
	**      ----------						-----------
	**
	**		Auth: Sergey Pelevin
	**		Date: 04.12.13
	*******************************************************************************
	**		Change History
	*******************************************************************************
	**		Date:		Author:		Description:
	**		--------	--------	-------------------------------------------
	**		22.12.13	SAP			 	
	**		09.07.14	BAO			 
	**		21.07.14	SAP			    
	**		22.07.14	SAP			  [Advance] -      
	**		04.08.14				  
	*******************************************************************************/
	CREATE PROCEDURE  [ReportFuturePeriodsRealization] 			
		@OwnCompanyID TIdentifier,-- ID   (NULL - )
		@CurrencySelector int,--  : 0 - , 1 - 
		@CompanyManagerSelector int, --   : 0 - , 1 -  
		@ManagerID TIdentifier, -- ID   (NULL - ,    )
		@CostCenterSelector int, --  : 0 -  , 1 -  
		@CostCenterID TIdentifier,-- ID   (NULL -  )
		@PeriodStartSelector int,--  : 0 -  , 1 - 
		@PeriodEndSelector int,--  : 0 - , 1 - 
		@PeriodStart TDate,--   
		@PeriodEnd TDate,--   
		@TypeSelector int, -- : 0 - , 1 - 
		@SortSelector int, -- : 0 -   , 1 -  
		@ScaleSelector	int,  -- : 0 - , 1 -   2 - 
		@CountSelector int, --    : 0 - , 1 -  5, 2 -  10, 3 - 
		@Count int --  	
	AS

	SET NOCOUNT ON

	DECLARE
        @date datetime,
        @CurrentEmployeeID TIdentifier,
        @Class TClass,
            
        @TimeEntryClass TClass,
        @ExpenseClass TClass,
        @PhoneCallClass TClass,
        @ServiceRequestClass TClass,

		@BaseCurrency TCode3,
		@NationalCurrency TCode3,

		@MaxValue int,
		@Scale int,

		@tmpManagerID TIdentifier,
		@tmpCompanyID TIdentifier,

		@ReportCurrency varchar(50),
		@RowCount int



		--********************************************************
		--     
		--
		--http://blogs.msdn.com/b/turgays/archive/2013/09/10/parameter-sniffing-problem-and-workarounds.aspx
		DECLARE
			@OwnCompanyID_loc TIdentifier,-- ID   (NULL - )
			@CurrencySelector_loc int,--  : 0 - , 1 - 
			@CompanyManagerSelector_loc int, --   : 0 - , 1 -  
			@ManagerID_loc TIdentifier, -- ID   (NULL - ,    )
			@CostCenterSelector_loc int, --  : 0 -  , 1 -  
			@CostCenterID_loc TIdentifier,-- ID   (NULL -  )
			@PeriodStartSelector_loc int,--  : 0 -  , 1 - 
			@PeriodEndSelector_loc int,--  : 0 - , 1 - 
			@PeriodStart_loc TDate,--   
			@PeriodEnd_loc TDate,--   
			@TypeSelector_loc int, -- : 0 - , 1 - 
			@SortSelector_loc int, -- : 0 -   , 1 -  
			@ScaleSelector_loc	int,  -- : 0 - , 1 -   2 - 
			@CountSelector_loc int, --    : 0 - , 1 -  5, 2 -  10, 3 - 
			@Count_loc int --  

		SELECT
			@OwnCompanyID_loc = @OwnCompanyID,
			@CurrencySelector_loc = @CurrencySelector,
			@CompanyManagerSelector_loc = @CompanyManagerSelector,
			@ManagerID_loc = @ManagerID,
			@CostCenterSelector_loc = @CostCenterSelector,
			@CostCenterID_loc = @CostCenterID,
			@PeriodStartSelector_loc = @PeriodStartSelector,
			@PeriodEndSelector_loc = @PeriodEndSelector,
			@PeriodStart_loc = @PeriodStart,
			@PeriodEnd_loc = @PeriodEnd,
			@TypeSelector_loc = @TypeSelector,
			@SortSelector_loc = @SortSelector,
			@ScaleSelector_loc = @ScaleSelector,
			@CountSelector_loc = @CountSelector,
			@Count_loc = @Count


	--  
	IF  dbo.adfHasAccessRightByName('ViewFuturePeriodsRealizationReport') = 0 
	BEGIN
		  RAISERROR('#NotEnoughPermissions', 18, -1)
		  RETURN 1
	END
            
	SET @date = CAST(CONVERT(varchar(8), GETDATE(), 112) AS datetime)

	IF(@PeriodStartSelector_loc = 0)
		SET @PeriodStart_loc = '1900-01-01 00:00:00'

	SET @PeriodEnd_loc = DATEADD(DAY,1,@PeriodEnd_loc)
	
	SET @Class = dbo.adfGetClassByName('Pjm5.Calculation')      
    SET @TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
    SET @ExpenseClass = dbo.adfGetClassByName('Pjm5.Expense')


	IF @CountSelector_loc = 1
		SET @Count_loc = 5
	IF @CountSelector_loc = 2
		SET @Count_loc = 10	

	-- 
	SELECT
		@BaseCurrency = [dbo].[fnPjm5GetBaseCurrency](),
		@NationalCurrency = [dbo].[fnPjm5GetNationalCurrency]()


	--   
	DECLARE @Chart TABLE
	(
		[Company] TName,
		[Total] TAmount
	)

	--   
	DECLARE @Rates TABLE
	(
		[FromCurrencyCode] TCode3,
		[ToCurrencyCode] TCode3,
		[Rate] TAmount
	)

	--      ,   
	DECLARE @ProjectAdvancesList TABLE
	(
		[CompanyID] TIdentifier,
		[ProjectID] TIdentifier,
		[NotCoveredByStatements] TAmount,
		[InvoiceID]	int		
	)

	
	--********************************************************
	--     .
	;
	WITH [TimeCurrencyCodes]([CurrencyCode])
	AS
	(
		SELECT DISTINCT				
			[TimeEntry].[CurrencyCode]		
		FROM
			[TimeEntry]
			INNER JOIN [EntryAmount]
				ON [EntryAmount].[ID] = [TimeEntry].[ID]
				AND [EntryAmount].[Price] > 0
			INNER JOIN [Project]
				ON [Project].[ID] = [TimeEntry].[ProjectID]
			INNER JOIN [Company]
				ON [Company].[ID] = [Project].[CustomerCompanyID]
			INNER JOIN [Employee]
				ON [Employee].[ID] = [Company].[ManagerEmployeeID]
		WHERE
			[Project].[State] NOT IN (4,5,6)
			AND
			[TimeEntry].[State] = 3
			AND
			[TimeEntry].[Billable] = 1 
			AND
			(
				@OwnCompanyID_loc IS NULL
				OR
				[Project].[SupplierCompanyID] = @OwnCompanyID_loc
			)
			AND
			(
				@CompanyManagerSelector_loc = 0
				OR
				[Company].[ManagerEmployeeID] = @ManagerID_loc
			)
			AND
			(
				@CostCenterSelector_loc = 0
				OR
				[Project].[CostCenterID] = @CostCenterID_loc 
			)
			AND
			(
				[TimeEntry].[StartTime] < @PeriodEnd_loc	
				AND
				[TimeEntry].[StartTime] >= @PeriodStart_loc			
			)
			AND
			(
				[StartTime] >= ISNULL(
										(
											SELECT 
												MAX([PeriodEnd]) + 1 
											FROM 
												[Calculation]
												LEFT OUTER JOIN [PaymentPeriod]
													ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
											WHERE 
												[Calculation].[ProjectID] = [TimeEntry].[ProjectID] 
												AND 
												[Calculation].[State] != 4
												AND
												[Calculation].[DocumentClass] = @TimeEntryClass
												AND
												(
													[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
													OR
													[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
												)
										), 
									0)
			)			
		)
	INSERT INTO @Rates
	SELECT
		[CurrencyCode] AS [FromCurrencyCode],
		@NationalCurrency AS [ToCurrencyCode],
		[dbo].[pjm6fnCurrencyGetRate]([CurrencyCode], @date, @NationalCurrency) AS [Rate]
	FROM
		[TimeCurrencyCodes]
	UNION
	SELECT
		[CurrencyCode] AS [FromCurrencyCode],
		@BaseCurrency AS [ToCurrencyCode],
		[dbo].[pjm6fnCurrencyGetRate]([CurrencyCode], @date, @BaseCurrency) AS [Rate]
	FROM
		[TimeCurrencyCodes]

	;
	WITH [ExpenseCurrencyCodes]([CurrencyCode])
	AS
	(
		SELECT DISTINCT
			[Expense].[ToCurrencyCode]
		FROM
			[Expense]				
			INNER JOIN [Project]
				ON [Project].[ID] = [Expense].[ProjectID]
			INNER JOIN [Company]
				ON [Company].[ID] = [Project].[CustomerCompanyID]
			INNER JOIN [Employee]
				ON [Employee].[ID] = [Company].[ManagerEmployeeID]
		WHERE
			[Project].[State] NOT IN (4,5,6)
			AND
			[Expense].[State] = 3
			AND
			[Expense].[Billable] = 1 
			AND
			(
				@OwnCompanyID_loc IS NULL
				OR
				[Project].[SupplierCompanyID] = @OwnCompanyID_loc
			)
			AND
			(
				@CompanyManagerSelector_loc = 0
				OR
				[Company].[ManagerEmployeeID] = @ManagerID_loc
			)
			AND
			(
				@CostCenterSelector_loc = 0
				OR
				[Project].[CostCenterID] = @CostCenterID_loc 
			)
			AND
			(
				[Expense].[Date] < @PeriodEnd_loc	
				AND
				[Expense].[Date] >= @PeriodStart_loc				
			)
			AND
			(
				[Expense].[Date] >= ISNULL(
										(
											SELECT 
												MAX([PeriodEnd]) + 1 
											FROM 
												[Calculation]
												LEFT OUTER JOIN [PaymentPeriod]
													ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
											WHERE 
												[Calculation].[ProjectID] = [Expense].[ProjectID] 
												AND 
												[Calculation].[State] != 4
												AND
												[Calculation].[DocumentClass] = @TimeEntryClass
												AND
												(
													[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
													OR
													[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
												)
										), 
									0)
			)		
	)
	INSERT INTO @Rates
	SELECT
		[CurrencyCode] AS [FromCurrencyCode],
		@NationalCurrency AS [ToCurrencyCode],
		[dbo].[pjm6fnCurrencyGetRate]([CurrencyCode], @date, @NationalCurrency) AS [Rate]
	FROM
		[ExpenseCurrencyCodes]
	UNION
	SELECT
		[CurrencyCode] AS [FromCurrencyCode],
		@BaseCurrency AS [ToCurrencyCode],
		[dbo].[pjm6fnCurrencyGetRate]([CurrencyCode], @date, @BaseCurrency) AS [Rate]
	FROM
		[ExpenseCurrencyCodes]
	EXCEPT
	SELECT * FROM @Rates

	--SELECT * FROM @Rates

	--********************************************************
	--     ,   ,      
	;
	WITH [TimeProjects]([CompanyID],[ProjectID],[NotCoveredByStatements],[InvoiceID])
	AS
	(
		SELECT DISTINCT				
			[Company].[ID],
			[TimeEntry].[ProjectID],			
			CASE
				WHEN 
					ROUND
					(
						ISNULL
						(
							[I].[Total] 
							- 
							SUM
							(
								ISNULL
								(
									CASE
										WHEN [SAI].[StatementPaymentValue] = 0 THEN 0
										ELSE [SAI].[StatementPaymentValue] - [SAI].[StatementPaymentVAT]
									END									
									,0
								)
							)
							,0
						)
						,2
					) <= 0 THEN 0
				ELSE 
					ROUND
					(
						ISNULL
						(
							[I].[Total] 
							- 
							SUM
							(
								ISNULL
								(
									CASE
										WHEN [SAI].[StatementPaymentValue] = 0 THEN 0
										ELSE [SAI].[StatementPaymentValue] - [SAI].[StatementPaymentVAT]
									END
									,0
								)
							)
							,0
						) 
						* 
						ISNULL([@Rates].[Rate],0)
						,2
					)
			END
			AS [NotCoveredByStatements],
			[I].[ID] AS [InvoiceID]			
		FROM
			[TimeEntry]
			INNER JOIN [EntryAmount]
				ON [EntryAmount].[ID] = [TimeEntry].[ID]
				AND [EntryAmount].[Price] > 0
			INNER JOIN [Project]
				ON [Project].[ID] = [TimeEntry].[ProjectID]
			INNER JOIN [Company]
				ON [Company].[ID] = [Project].[CustomerCompanyID]
			INNER JOIN [Employee]
				ON [Employee].[ID] = [Company].[ManagerEmployeeID]
			LEFT JOIN [Invoice] AS [I]
				ON [I].[ProjectID] = [TimeEntry].[ProjectID]
				AND
				[I].[State] IN(2,5)--  
				AND
				[I].[Date] < @PeriodEnd_loc
				AND
				[I].[Advance] = 1
			LEFT JOIN [StatementAdvanceInvoice] AS [SAI]
				ON [SAI].[AdvanceInvoiceID] = [I].[ID]
			LEFT JOIN @Rates
				ON [@Rates].[FromCurrencyCode] = [I].[CurrencyCode]
		WHERE
			[Project].[State] NOT IN (4,5,6)
			AND
			[TimeEntry].[State] = 3
			AND
			[TimeEntry].[Billable] = 1 
			AND
			(
				@OwnCompanyID_loc IS NULL
				OR
				[Project].[SupplierCompanyID] = @OwnCompanyID_loc
			)
			AND
			(
				@CompanyManagerSelector_loc = 0
				OR
				[Company].[ManagerEmployeeID] = @ManagerID_loc
			)
			AND
			(
				@CostCenterSelector_loc = 0
				OR
				[Project].[CostCenterID] = @CostCenterID_loc 
			)
			AND
			(
				[TimeEntry].[StartTime] < @PeriodEnd_loc	
				AND
				[TimeEntry].[StartTime] >= @PeriodStart_loc			
			)
			AND
			(
				[StartTime] >= ISNULL(
										(
											SELECT 
												MAX([PeriodEnd]) + 1 
											FROM 
												[Calculation]
												LEFT OUTER JOIN [PaymentPeriod]
													ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
											WHERE 
												[Calculation].[ProjectID] = [TimeEntry].[ProjectID] 
												AND 
												[Calculation].[State] != 4
												AND
												[Calculation].[DocumentClass] = @TimeEntryClass
												AND
												(
													[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
													OR
													[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
												)
										), 
									0)
			)
			AND
			(
				@CurrencySelector_loc = 0 AND [@Rates].[ToCurrencyCode] = @NationalCurrency
				OR
				@CurrencySelector_loc = 1 AND [@Rates].[ToCurrencyCode] = @BaseCurrency
			)
		GROUP BY
			[Company].[ID],
			[TimeEntry].[ProjectID],	
			[I].[ID],
			[I].[Total],
			[@Rates].[Rate]						
	)
	INSERT INTO @ProjectAdvancesList
	SELECT * FROM [TimeProjects]
	
	;
	WITH [ExpenseProjects]([CompanyID],[ProjectID],[NotCoveredByStatements],[InvoiceID])
	AS
	(
		SELECT DISTINCT			
			[Company].[ID],
			[Expense].[ProjectID],			
			CASE
				WHEN 
					ROUND
					(
						ISNULL
						(
							[I].[Total] 
							- 
							SUM
							(
								ISNULL
								(
									CASE
										WHEN [SAI].[StatementPaymentValue] = 0 THEN 0
										ELSE [SAI].[StatementPaymentValue] - [SAI].[StatementPaymentVAT]
									END									
									,0
								)
							)
							,0
						)
						,2
					) <= 0 THEN 0
				ELSE 
					ROUND
					(
						ISNULL
						(
							[I].[Total] 
							- 
							SUM
							(
								ISNULL
								(
									CASE
										WHEN [SAI].[StatementPaymentValue] = 0 THEN 0
										ELSE [SAI].[StatementPaymentValue] - [SAI].[StatementPaymentVAT]
									END
									,0
								)
							)
							,0
						) 
						* 
						ISNULL([@Rates].[Rate],0)
						,2
					)
			END 
			AS [NotCoveredByStatements],
			[I].[ID] AS [InvoiceID]			
		FROM
			[Expense]				
			INNER JOIN [Project]
				ON [Project].[ID] = [Expense].[ProjectID]
			INNER JOIN [Company]
				ON [Company].[ID] = [Project].[CustomerCompanyID]
			INNER JOIN [Employee]
				ON [Employee].[ID] = [Company].[ManagerEmployeeID]
			LEFT JOIN [Invoice] AS [I]
				ON [I].[ProjectID] = [Expense].[ProjectID]
				AND
				[I].[State] IN(2,5)--  
				AND
				[I].[Date] < @PeriodEnd_loc
				AND
				[I].[Advance] = 1
			LEFT JOIN [StatementAdvanceInvoice] AS [SAI]
				ON [SAI].[AdvanceInvoiceID] = [I].[ID]
			LEFT JOIN @Rates
				ON [@Rates].[FromCurrencyCode] = [I].[CurrencyCode]
		WHERE
			[Project].[State] NOT IN (4,5,6)
			AND
			[Expense].[State] = 3
			AND
			[Expense].[Billable] = 1 
			AND
			(
				@OwnCompanyID_loc IS NULL
				OR
				[Project].[SupplierCompanyID] = @OwnCompanyID_loc
			)
			AND
			(
				@CompanyManagerSelector_loc = 0
				OR
				[Company].[ManagerEmployeeID] = @ManagerID_loc
			)
			AND
			(
				@CostCenterSelector_loc = 0
				OR
				[Project].[CostCenterID] = @CostCenterID_loc 
			)
			AND
			(
				[Expense].[Date] < @PeriodEnd_loc	
				AND
				[Expense].[Date] >= @PeriodStart_loc				
			)
			AND
			(
				[Expense].[Date] >= ISNULL(
										(
											SELECT 
												MAX([PeriodEnd]) + 1 
											FROM 
												[Calculation]
												LEFT OUTER JOIN [PaymentPeriod]
													ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
											WHERE 
												[Calculation].[ProjectID] = [Expense].[ProjectID] 
												AND 
												[Calculation].[State] != 4
												AND
												[Calculation].[DocumentClass] = @TimeEntryClass
												AND
												(
													[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
													OR
													[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
												)
										), 
									0)
			)
			AND
			(
				@CurrencySelector_loc = 0 AND [@Rates].[ToCurrencyCode] = @NationalCurrency
				OR
				@CurrencySelector_loc = 1 AND [@Rates].[ToCurrencyCode] = @BaseCurrency
			)
		GROUP BY
			[Company].[ID],
			[Expense].[ProjectID],	
			[I].[ID],
			[I].[Total],
			[@Rates].[Rate]							
				
	)
	INSERT INTO @ProjectAdvancesList
	SELECT 
		* 
	FROM 
		[ExpenseProjects]
	WHERE 
		[ExpenseProjects].[ProjectID] NOT IN (SELECT [ProjectID] FROM @ProjectAdvancesList)


	--SELECT * FROM @ProjectAdvancesList ORDER by [CompanyID], [ProjectID]


	IF @TypeSelector_loc = 0
	BEGIN
		DECLARE @common TABLE
		(
			[Order] int,
			[ManagerID] TIdentifier,
			[CompanyID] TIdentifier,
			[PeriodStart] TDate,
			[PeriodEnd] TDate,
			[Time] TAmount,
			[TimeMoney] TAmount,
			[ExpenseMoney] TAmount,
			[Advance] TAmount,
			[Total] TAmount
		)

		--********************************************************
		--
		DECLARE @time TABLE
		(
			[ManagerID] TIdentifier,
			[CompanyID] TIdentifier,
			[PeriodStart] TDate,
			[PeriodEnd] TDate,
			[Time] TAmount,
			[TimeMoney] TAmount			
		)
		
		;
		WITH [Time]([ManagerID],[CompanyID],[StartDate],[EndDate],[Time],[TimeMoney])
		AS
		(
			SELECT				
				[Employee].[ID] AS [ManagerID],
				[Company].[ID] AS [CompanyID],
				MIN([TimeEntry].[StartTime]),
				MAX([TimeEntry].[StartTime]),						
				SUM(ISNULL([EntryAmount].[Amount], 0)) AS [Time],
				SUM(ISNULL([EntryAmount].[Amount] * [EntryAmount].[Price] * [@Rates].[Rate], 0)) AS [TimeMoney]					
			FROM
				[TimeEntry]
				INNER JOIN [EntryAmount]
					ON [EntryAmount].[ID] = [TimeEntry].[ID]
					AND [EntryAmount].[Price] > 0
				INNER JOIN [Project]
					ON [Project].[ID] = [TimeEntry].[ProjectID]
				INNER JOIN [Company]
					ON [Company].[ID] = [Project].[CustomerCompanyID]
				INNER JOIN [Employee]
					ON [Employee].[ID] = [Company].[ManagerEmployeeID]
				INNER JOIN @Rates
					ON [@Rates].[FromCurrencyCode] = [TimeEntry].[CurrencyCode]
			WHERE
				(
					@CurrencySelector_loc = 0 AND [@Rates].[ToCurrencyCode] = @NationalCurrency
					OR
					@CurrencySelector_loc = 1 AND [@Rates].[ToCurrencyCode] = @BaseCurrency
				)
				AND			
				[Project].[State] NOT IN (4,5,6)
				AND
				[TimeEntry].[State] = 3
				AND
				[TimeEntry].[Billable] = 1 
				AND
				(
					@OwnCompanyID_loc IS NULL
					OR
					[Project].[SupplierCompanyID] = @OwnCompanyID_loc
				)
				AND
				(
					@CompanyManagerSelector_loc = 0
					OR
					[Company].[ManagerEmployeeID] = @ManagerID_loc
				)
				AND
				(
					@CostCenterSelector_loc = 0
					OR
					[Project].[CostCenterID] = @CostCenterID_loc 
				)
				AND
				(
					[TimeEntry].[StartTime] < @PeriodEnd_loc
					AND
					[TimeEntry].[StartTime] >= @PeriodStart_loc					
				)
				AND
				(
					[StartTime] >= ISNULL(
											(
												SELECT 
													MAX([PeriodEnd]) + 1 
												FROM 
													[Calculation]
													LEFT OUTER JOIN [PaymentPeriod]
														ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
												WHERE 
													[Calculation].[ProjectID] = [TimeEntry].[ProjectID] 
													AND 
													[Calculation].[State] != 4
													AND
													[Calculation].[DocumentClass] = @TimeEntryClass
													AND
													(
														[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
														OR
														[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
													)
											), 
										0)
				)
			GROUP BY
				[Employee].[ID],
				[Company].[ID]
		)
		
		INSERT INTO @time
		SELECT * FROM [Time]

		--SELECT * FROM @time ORDER BY [ManagerID],[CompanyID]

		--********************************************************
		--
		DECLARE @expense TABLE
		(
			[ManagerID] TIdentifier,
			[CompanyID] TIdentifier,
			[PeriodStart] TDate,
			[PeriodEnd] TDate,
			[ExpenseMoney] TAmount			
		)


		;
		WITH [Expenses]([ManagerID],[CompanyID],[PeriodStart],[PeriodEnd],[ExpenseMoney])
		AS
		(
			SELECT
				[Employee].[ID] AS [ManagerID],
				[Company].[ID] AS [CompanyID],
				MIN([Expense].[Date]),
				MAX([Expense].[Date]),							
				SUM(ISNULL([Expense].[TotalWithVATPC] * [@Rates].[Rate], 0)) AS [ExpenseMoney]
			FROM
				[Expense]				
				INNER JOIN [Project]
					ON [Project].[ID] = [Expense].[ProjectID]
				INNER JOIN [Company]
					ON [Company].[ID] = [Project].[CustomerCompanyID]
				INNER JOIN [Employee]
					ON [Employee].[ID] = [Company].[ManagerEmployeeID]
				INNER JOIN @Rates
					ON [@Rates].[FromCurrencyCode] = [Expense].[ToCurrencyCode]
			WHERE
				(
					@CurrencySelector_loc = 0 AND [@Rates].[ToCurrencyCode] = @NationalCurrency
					OR
					@CurrencySelector_loc = 1 AND [@Rates].[ToCurrencyCode] = @BaseCurrency
				)
				AND			
				[Project].[State] NOT IN (4,5,6)
				AND
				[Expense].[State] = 3
				AND
				[Expense].[Billable] = 1 
				AND
				(
					@OwnCompanyID_loc IS NULL
					OR
					[Project].[SupplierCompanyID] = @OwnCompanyID_loc
				)
				AND
				(
					@CompanyManagerSelector_loc = 0
					OR
					[Company].[ManagerEmployeeID] = @ManagerID_loc
				)
				AND
				(
					@CostCenterSelector_loc = 0
					OR
					[Project].[CostCenterID] = @CostCenterID_loc 
				)
				AND
				(
					[Expense].[Date] < @PeriodEnd_loc		
					AND
					[Expense].[Date] >= @PeriodStart_loc			
				)
				AND
				(
					[Expense].[Date] >= ISNULL(
											(
												SELECT 
													MAX([PeriodEnd]) + 1 
												FROM 
													[Calculation]
													LEFT OUTER JOIN [PaymentPeriod]
														ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
												WHERE 
													[Calculation].[ProjectID] = [Expense].[ProjectID] 
													AND 
													[Calculation].[State] != 4
													AND
													[Calculation].[DocumentClass] = @TimeEntryClass
													AND
													(
														[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
														OR
														[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
													)
											), 
										0)
				)
			GROUP BY				
				[Employee].[ID],
				[Company].[ID]
		)


		INSERT INTO @expense
		SELECT	* FROM	[Expenses]

		--SELECT * FROM @expense ORDER BY [Company]




		--********************************************************
		--    time  expense    
		
		INSERT INTO @common
		SELECT
			0,
			[@time].[ManagerID],
			[@time].[CompanyID],
			CASE WHEN ISNULL([@time].[PeriodStart],@date) < ISNULL([@expense].[PeriodStart],@date)
				THEN ISNULL([@time].[PeriodStart],@date)
				ELSE ISNULL([@expense].[PeriodStart],@date)
			END,
			CASE WHEN ISNULL([@time].[PeriodEnd],0) > ISNULL([@expense].[PeriodEnd],0)
				THEN ISNULL([@time].[PeriodEnd],0)
				ELSE ISNULL([@expense].[PeriodEnd],0)
			END,
			[@time].[Time],
			[@time].[TimeMoney],
			[@expense].[ExpenseMoney],
			SUM(ISNULL([@ProjectAdvancesList].[NotCoveredByStatements],0)),
			CASE
				WHEN ((ISNULL([@time].[TimeMoney],0) + ISNULL([@expense].[ExpenseMoney],0)) - SUM(ISNULL([@ProjectAdvancesList].[NotCoveredByStatements],0)) < 0) THEN 0
				ELSE (ISNULL([@time].[TimeMoney],0) + ISNULL([@expense].[ExpenseMoney],0)) - SUM(ISNULL([@ProjectAdvancesList].[NotCoveredByStatements],0))
			END
		FROM
			@time
			LEFT JOIN @expense
				ON [@expense].[ManagerID] = [@time].[ManagerID]
					AND
					[@expense].[CompanyID] = [@time].[CompanyID]
			LEFT JOIN @ProjectAdvancesList
				ON [@time].[CompanyID] = [@ProjectAdvancesList].[CompanyID]
		GROUP BY
			[@time].[ManagerID],
			[@time].[CompanyID],
			CASE WHEN ISNULL([@time].[PeriodStart],@date) < ISNULL([@expense].[PeriodStart],@date)
				THEN ISNULL([@time].[PeriodStart],@date)
				ELSE ISNULL([@expense].[PeriodStart],@date)
			END,
			CASE WHEN ISNULL([@time].[PeriodEnd],0) > ISNULL([@expense].[PeriodEnd],0)
				THEN ISNULL([@time].[PeriodEnd],0)
				ELSE ISNULL([@expense].[PeriodEnd],0)
			END,
			[@time].[Time],
			[@time].[TimeMoney],
			[@expense].[ExpenseMoney]

		--SELECT * FROM @Common
		--********************************************************
		--       
		--    [Order]     :
		--0 -  
		--1000 -    
		--1001 -   

		DECLARE managerSubtotalCursor CURSOR LOCAL FAST_FORWARD FOR
			SELECT DISTINCT [ManagerID] FROM @common

		OPEN managerSubtotalCursor
		FETCH NEXT FROM managerSubtotalCursor INTO @tmpManagerID

		WHILE @@FETCH_STATUS = 0
		BEGIN
			INSERT INTO @common
			SELECT
				1000,
				@tmpManagerID,
				NULL,
				NULL,
				NULL,
				SUM([@common].[Time]),
				SUM([@common].[TimeMoney]),
				SUM([@common].[ExpenseMoney]),
				SUM([@common].[Advance]),
				SUM([@common].[Total])
			FROM
				@common
			WHERE
				[@common].[ManagerID] = @tmpManagerID


			FETCH NEXT FROM managerSubtotalCursor INTO @tmpManagerID
		END

		CLOSE managerSubtotalCursor
		DEALLOCATE managerSubtotalCursor


		SELECT
			@RowCount = COUNT(*)
		FROM
			@common

		IF @RowCount > 0
		BEGIN
			INSERT INTO @common
			SELECT
				1001,
				NULL,
				NULL,
				NULL,
				NULL,
				SUM([@common].[Time]),
				SUM([@common].[TimeMoney]),
				SUM([@common].[ExpenseMoney]),
				SUM([@common].[Advance]),
				SUM([@common].[Total])
			FROM
				@common
			WHERE 
				[Order] = 0
		END



		--********************************************************
		--   (SmartValue)
		
		SELECT
			@MaxValue = [Total]
		FROM
			@common
		WHERE
			[Order] = 1001
			
		IF @ScaleSelector_loc = 3
		SET @Scale = 
			CASE  
				WHEN @MaxValue < 10000 THEN 1
				WHEN @MaxValue >= 10000 AND @MaxValue < 10000000 THEN 1000
				WHEN @MaxValue >= 10000000 AND @MaxValue < 10000000000 THEN 1000000
				WHEN @MaxValue >= 10000000000 THEN 1000000000
			END
		ELSE
		SET	@Scale = 
			CASE @ScaleSelector_loc
				WHEN 0 THEN 1
				WHEN 1 THEN 1000
				WHEN 2 THEN 1000000
		END


		-- 		
		UPDATE @common
		SET
			[TimeMoney] = [TimeMoney] / @Scale,
			[ExpenseMoney] = [ExpenseMoney] / @Scale,
			[Advance] = [Advance] / @Scale,
			[Total] = [Total] / @Scale





		--********************************************************
		-- .
		IF @CountSelector_loc = 0
		BEGIN
			IF @SortSelector_loc = 1
			BEGIN
				DECLARE @sortedOut TABLE
				(
					[Order] int,
					[ManagerID] TName,
					[CompanyID] TName,
					[PeriodStart] TDate,
					[PeriodEnd] TDate,
					[Time] TAmount,
					[TimeMoney] TAmount,
					[ExpenseMoney] TAmount,
					[Advance] TAmount,
					[Total] TAmount
				)
							
				WHILE EXISTS(SELECT * FROM @common WHERE [@common].[ManagerID] NOT IN (SELECT DISTINCT [ManagerID] FROM @sortedOut))
				BEGIN
					SELECT TOP 1
						@tmpManagerID = [ManagerID]
					FROM
						@common
					WHERE
						[ManagerID] NOT IN (SELECT DISTINCT [ManagerID] FROM @sortedOut)
						AND 
						[Order] = 1000
					ORDER BY
						[Total] DESC


					INSERT INTO @sortedOut
					SELECT
						*
					FROM
						@common
					WHERE
						[ManagerID] = @tmpManagerID
						AND
						[Order] = 0
					ORDER BY 
						[Total] DESC

					INSERT INTO @sortedOut
					SELECT
						*
					FROM
						@common
					WHERE
						[ManagerID] = @tmpManagerID
						AND
						[Order] = 1000				
				END
			
				INSERT INTO @sortedOut
				SELECT
					*
				FROM
					@common
				WHERE				
					[Order] = 1001
				
				--SELECT * FROM @sortedOut		
				SELECT
					[Order],
					[Employee].[FileAs] AS [Manager],
					[Company].[FileAs] AS [Company],
					[PeriodStart],
					[PeriodEnd],
					[Time],
					[TimeMoney],
					[ExpenseMoney],
					[Advance],
					[Total],
					@SortSelector_loc AS [SortSelector]
				FROM
					@sortedOut 
					LEFT JOIN [Employee]
						ON [Employee].[ID] = [@sortedOut].[ManagerID]
					LEFT JOIN [Company]
						ON [Company].[ID] = [@sortedOut].[CompanyID]
			END
			ELSE
			BEGIN
				SELECT 
					[Order],
					[Employee].[FileAs] AS [Manager],
					[Company].[FileAs] AS [Company],
					[PeriodStart],
					[PeriodEnd],
					[Time],
					[TimeMoney],
					[ExpenseMoney],
					[Advance],
					[Total],
					@SortSelector_loc AS [SortSelector]
				FROM
					@common 
					LEFT JOIN [Employee]
						ON [Employee].[ID] = [@common].[ManagerID]
					LEFT JOIN [Company]
						ON [Company].[ID] = [@common].[CompanyID]
			END				
		END			
		ELSE
		BEGIN
			DECLARE @commonTopX TABLE
			(
				[Order] int,
				[ManagerID] TIdentifier,
				[CompanyID] TIdentifier,
				[PeriodStart] TDate,
				[PeriodEnd] TDate,
				[Time] TAmount,
				[TimeMoney] TAmount,
				[ExpenseMoney] TAmount,
				[Advance] TAmount,
				[Total] TAmount
			)

			--  @Count_loc 
			INSERT INTO @commonTopX
			SELECT TOP (@Count_loc)
				*
			FROM
				@common
			WHERE
				[Order] = 0
			ORDER BY
				[Total] DESC
			
			--    
			INSERT INTO @commonTopX
			SELECT
				1000,
				NULL,
				-1,
				NULL,
				NULL,
				SUM([Time]),
				SUM([TimeMoney]),
				SUM([ExpenseMoney]),
				SUM([Advance]),
				SUM([Total])			
			FROM
				@common
			WHERE
				[CompanyID] NOT IN (SELECT [CompanyID] FROM @commonTopX)
				AND
				[Order] = 0

			--      @common
			INSERT INTO @commonTopX
			SELECT
				*
			FROM
				@common
			WHERE
				[Order] = 1001



			SELECT 
				[Order],
				[Employee].[FileAs] AS [Manager],
				CASE WHEN [Order] = 1000
					THEN ''
					ELSE [Company].[FileAs] 
				END AS [Company],
				[PeriodStart],
				[PeriodEnd],
				[Time],
				[TimeMoney],
				[ExpenseMoney],
				[Advance],
				[Total],
				@SortSelector_loc AS [SortSelector]
			FROM
				@commonTopX 
				LEFT JOIN [Employee]
					ON [Employee].[ID] = [@commonTopX].[ManagerID]
				LEFT JOIN [Company]
					ON [Company].[ID] = [@commonTopX].[CompanyID]
			ORDER BY 
				[Employee].[FileAs], 
				[Order]
				
			
			-- ,       10 
			IF @Count_loc < 11
			BEGIN
				INSERT INTO @Chart
				SELECT
					CASE WHEN [Order] = 1000
						THEN ''
						ELSE [Company].[FileAs] 
					END AS [Company],
					[Total]
				FROM
					@commonTopX
					LEFT JOIN [Company]
						ON [Company].[ID] = [@commonTopX].[CompanyID]
				WHERE
					[Order] IN (0,1000)
			END		
		END	 

	END
	ELSE
	BEGIN
		DECLARE @commonP TABLE
		(
			[Order] int,
			[ManagerID] TIdentifier,
			[CompanyID] TIdentifier,
			[ProjectID] TIdentifier,
			[PeriodStart] TDate,
			[PeriodEnd] TDate,
			[Time] TAmount,
			[TimeMoney] TAmount,
			[ExpenseMoney] TAmount,
			[Advance] TAmount,
			[Total] TAmount
		)


		--********************************************************
		--
		DECLARE @timeP TABLE
		(
			[ManagerID] TIdentifier,
			[CompanyID] TIdentifier,
			[ProjectID] TIdentifier,
			[PeriodStart] TDate,
			[PeriodEnd] TDate,
			[Time] TAmount,
			[TimeMoney] TAmount			
		)


		;
		WITH [TimeP]([ManagerID],[CompanyID],[ProjectID],[StartDate],[EndDate],[Time],[TimeMoney])
		AS
		(
			SELECT				
				[Employee].[ID] AS [ManagerID],
				[Company].[ID] AS [CompanyID],
				[TimeEntry].[ProjectID],
				MIN([TimeEntry].[StartTime]),
				MAX([TimeEntry].[StartTime]),						
				SUM(ISNULL([EntryAmount].[Amount], 0)) AS [Time],
				SUM(ISNULL([EntryAmount].[Amount] * [EntryAmount].[Price] * [@Rates].[Rate], 0)) AS [TimeMoney]			
			FROM
				[TimeEntry]
				INNER JOIN [EntryAmount]
					ON [EntryAmount].[ID] = [TimeEntry].[ID]
					AND [EntryAmount].[Price] > 0
				INNER JOIN [Project]
					ON [Project].[ID] = [TimeEntry].[ProjectID]
				INNER JOIN [Company]
					ON [Company].[ID] = [Project].[CustomerCompanyID]
				INNER JOIN [Employee]
					ON [Employee].[ID] = [Company].[ManagerEmployeeID]
				INNER JOIN @Rates
					ON [@Rates].[FromCurrencyCode] = [TimeEntry].[CurrencyCode]
			WHERE
				(
					@CurrencySelector_loc = 0 AND [@Rates].[ToCurrencyCode] = @NationalCurrency
					OR
					@CurrencySelector_loc = 1 AND [@Rates].[ToCurrencyCode] = @BaseCurrency
				)
				AND			
				[Project].[State] NOT IN (4,5,6)
				AND
				[TimeEntry].[State] = 3
				AND
				[TimeEntry].[Billable] = 1 
				AND
				(
					@OwnCompanyID_loc IS NULL
					OR
					[Project].[SupplierCompanyID] = @OwnCompanyID_loc
				)
				AND
				(
					@CompanyManagerSelector_loc = 0
					OR
					[Company].[ManagerEmployeeID] = @ManagerID_loc
				)
				AND
				(
					@CostCenterSelector_loc = 0
					OR
					[Project].[CostCenterID] = @CostCenterID_loc 
				)
				AND
				(
					[TimeEntry].[StartTime] < @PeriodEnd_loc	
					AND
					[TimeEntry].[StartTime] >= @PeriodStart_loc				
				)
				AND
				(
					[StartTime] >= ISNULL(
											(
												SELECT 
													MAX([PeriodEnd]) + 1 
												FROM 
													[Calculation]
													LEFT OUTER JOIN [PaymentPeriod]
														ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
												WHERE 
													[Calculation].[ProjectID] = [TimeEntry].[ProjectID] 
													AND 
													[Calculation].[State] != 4
													AND
													[Calculation].[DocumentClass] = @TimeEntryClass
													AND
													(
														[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
														OR
														[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
													)
											), 
										0)
				)
			GROUP BY
				[Employee].[ID],
				[Company].[ID],
				[TimeEntry].[ProjectID]
		)
		
		INSERT INTO @timeP
		SELECT * FROM [TimeP]

		--SELECT * FROM @timeP ORDER BY [ManagerID],[CompanyID],[ProjectID]


		--********************************************************
		--
		DECLARE @expenseP TABLE
		(
			[ManagerID] TIdentifier,
			[CompanyID] TIdentifier,
			[ProjectID] TIdentifier,
			[PeriodStart] TDate,
			[PeriodEnd] TDate,
			[ExpenseMoney] TAmount			
		)


		;
		WITH [ExpensesP]([ManagerID],[CompanyID],[ProjectID],[PeriodStart],[PeriodEnd],[ExpenseMoney])
		AS
		(
			SELECT
				[Employee].[ID] AS [ManagerID],
				[Company].[ID] AS [CompanyID],
				[Expense].[ProjectID],
				MIN([Expense].[Date]),
				MAX([Expense].[Date]),							
				SUM(ISNULL([Expense].[TotalWithVATPC] * [@Rates].[Rate], 0)) AS [ExpenseMoney]		
			FROM
				[Expense]				
				INNER JOIN [Project]
					ON [Project].[ID] = [Expense].[ProjectID]
				INNER JOIN [Company]
					ON [Company].[ID] = [Project].[CustomerCompanyID]
				INNER JOIN [Employee]
					ON [Employee].[ID] = [Company].[ManagerEmployeeID]
				INNER JOIN @Rates
					ON [@Rates].[FromCurrencyCode] = [Expense].[ToCurrencyCode]
			WHERE
				(
					@CurrencySelector_loc = 0 AND [@Rates].[ToCurrencyCode] = @NationalCurrency
					OR
					@CurrencySelector_loc = 1 AND [@Rates].[ToCurrencyCode] = @BaseCurrency
				)
				AND			
				[Project].[State] NOT IN (4,5,6)
				AND
				[Expense].[State] = 3
				AND
				[Expense].[Billable] = 1 
				AND
				(
					@OwnCompanyID_loc IS NULL
					OR
					[Project].[SupplierCompanyID] = @OwnCompanyID_loc
				)
				AND
				(
					@CompanyManagerSelector_loc = 0
					OR
					[Company].[ManagerEmployeeID] = @ManagerID_loc
				)
				AND
				(
					@CostCenterSelector_loc = 0
					OR
					[Project].[CostCenterID] = @CostCenterID_loc 
				)
				AND
				(
					[Expense].[Date] < @PeriodEnd_loc		
					AND
					[Expense].[Date] >= @PeriodStart_loc			
				)
				AND
				(
					[Expense].[Date] >= ISNULL(
											(
												SELECT 
													MAX([PeriodEnd]) + 1 
												FROM 
													[Calculation]
													LEFT OUTER JOIN [PaymentPeriod]
														ON [PaymentPeriod].[ProjectID] = [Calculation].[ProjectID]
												WHERE 
													[Calculation].[ProjectID] = [Expense].[ProjectID] 
													AND 
													[Calculation].[State] != 4
													AND
													[Calculation].[DocumentClass] = @TimeEntryClass
													AND
													(
														[Calculation].[PeriodEnd] < [PaymentPeriod].[CalculationNextPaymentDate]
														OR
														[PaymentPeriod].[CalculationNextPaymentDate] IS NULL
													)
											), 
										0)
				)
			GROUP BY				
				[Employee].[ID],
				[Company].[ID],
				[Expense].[ProjectID]
		)


		INSERT INTO @expenseP
		SELECT	* FROM	[ExpensesP]

		--SELECT * FROM @expenseP ORDER BY [ManagerID],[CompanyID],[ProjectID]



		--********************************************************
		--    time  expense    
		
		INSERT INTO @commonP
		SELECT
			0,
			[@timeP].[ManagerID],
			[@timeP].[CompanyID],
			[@timeP].[ProjectID],
			CASE WHEN ISNULL([@timeP].[PeriodStart],@date) < ISNULL([@expenseP].[PeriodStart],@date)
				THEN ISNULL([@timeP].[PeriodStart],@date)
				ELSE ISNULL([@expenseP].[PeriodStart],@date)
			END,
			CASE WHEN ISNULL([@timeP].[PeriodEnd],0) > ISNULL([@expenseP].[PeriodEnd],0)
				THEN ISNULL([@timeP].[PeriodEnd],0)
				ELSE ISNULL([@expenseP].[PeriodEnd],0)
			END,
			[@timeP].[Time],
			[@timeP].[TimeMoney],
			[@expenseP].[ExpenseMoney],
			SUM(ISNULL([@ProjectAdvancesList].[NotCoveredByStatements],0)),
			CASE
				WHEN ((ISNULL([@timeP].[TimeMoney],0) + ISNULL([@expenseP].[ExpenseMoney],0)) - SUM(ISNULL([@ProjectAdvancesList].[NotCoveredByStatements],0)) < 0) THEN 0
				ELSE (ISNULL([@timeP].[TimeMoney],0) + ISNULL([@expenseP].[ExpenseMoney],0)) - SUM(ISNULL([@ProjectAdvancesList].[NotCoveredByStatements],0))
			END
		FROM
			@timeP
			LEFT JOIN @expenseP
				ON [@expenseP].[ManagerID] = [@timeP].[ManagerID]
					AND
					[@expenseP].[CompanyID] = [@timeP].[CompanyID]
					AND
					[@expenseP].[ProjectID] = [@timeP].[ProjectID]
			LEFT JOIN @ProjectAdvancesList
				ON [@timeP].[CompanyID] = [@ProjectAdvancesList].[CompanyID]
					AND
					[@timeP].[ProjectID] = [@ProjectAdvancesList].[ProjectID]
		GROUP BY
			[@timeP].[ManagerID],
			[@timeP].[CompanyID],
			[@timeP].[ProjectID],
			CASE WHEN ISNULL([@timeP].[PeriodStart],@date) < ISNULL([@expenseP].[PeriodStart],@date)
				THEN ISNULL([@timeP].[PeriodStart],@date)
				ELSE ISNULL([@expenseP].[PeriodStart],@date)
			END,
			CASE WHEN ISNULL([@timeP].[PeriodEnd],0) > ISNULL([@expenseP].[PeriodEnd],0)
				THEN ISNULL([@timeP].[PeriodEnd],0)
				ELSE ISNULL([@expenseP].[PeriodEnd],0)
			END,
			[@timeP].[Time],
			[@timeP].[TimeMoney],
			[@expenseP].[ExpenseMoney]


		--********************************************************
		--    ,    
		--    [Order]     :
		-- -1 -    
		--0 -  
		--1000 -    
		--1001 -   


		--  
		DECLARE companySubtotalCursor CURSOR LOCAL FAST_FORWARD FOR
			SELECT DISTINCT [ManagerID], [CompanyID] FROM @commonP

		OPEN companySubtotalCursor
		FETCH NEXT FROM companySubtotalCursor INTO @tmpManagerID, @tmpCompanyID

		WHILE @@FETCH_STATUS = 0
		BEGIN
			INSERT INTO @commonP
			SELECT
				-1,
				@tmpManagerID,
				@tmpCompanyID,
				NULL,
				MIN([@commonP].[PeriodStart]),
				MAX([@commonP].[PeriodEnd]),
				SUM([@commonP].[Time]),
				SUM([@commonP].[TimeMoney]),
				SUM([@commonP].[ExpenseMoney]),
				SUM([@commonP].[Advance]),
				SUM([@commonP].[Total])
			FROM
				@commonP
			WHERE
				[@commonP].[ManagerID] = @tmpManagerID
				AND
				[@commonP].[CompanyID] = @tmpCompanyID


			FETCH NEXT FROM companySubtotalCursor INTO @tmpManagerID, @tmpCompanyID
		END

		CLOSE companySubtotalCursor
		DEALLOCATE companySubtotalCursor


		--  
		DECLARE managerSubtotalCursor CURSOR LOCAL FAST_FORWARD FOR
			SELECT DISTINCT [ManagerID] FROM @commonP

		OPEN managerSubtotalCursor
		FETCH NEXT FROM managerSubtotalCursor INTO @tmpManagerID

		WHILE @@FETCH_STATUS = 0
		BEGIN
			INSERT INTO @commonP
			SELECT
				1000,
				@tmpManagerID,
				NULL,
				NULL,
				NULL,
				NULL,
				SUM([@commonP].[Time]),
				SUM([@commonP].[TimeMoney]),
				SUM([@commonP].[ExpenseMoney]),
				SUM([@commonP].[Advance]),
				SUM([@commonP].[Total])
			FROM
				@commonP
			WHERE
				[@commonP].[ManagerID] = @tmpManagerID
				AND
				[@commonP].[Order] = 0


			FETCH NEXT FROM managerSubtotalCursor INTO @tmpManagerID
		END

		CLOSE managerSubtotalCursor
		DEALLOCATE managerSubtotalCursor


		SELECT
			@RowCount = COUNT(*)
		FROM
			@commonP

		-- 
		IF @RowCount > 0
		BEGIN		
			INSERT INTO @commonP
			SELECT
				1001,
				NULL,
				NULL,
				NULL,
				NULL,
				NULL,
				SUM([@commonP].[Time]),
				SUM([@commonP].[TimeMoney]),
				SUM([@commonP].[ExpenseMoney]),
				SUM([@commonP].[Advance]),
				SUM([@commonP].[Total])
			FROM
				@commonP
			WHERE 
				[Order] = 0
		END



		--********************************************************
		--   (SmartValue)
		
		SELECT
			@MaxValue = [Total]
		FROM
			@commonP
		WHERE
			[Order] = 1001
			
		IF @ScaleSelector_loc = 3
			SET @Scale = 
				CASE  
					WHEN @MaxValue < 10000 THEN 1
					WHEN @MaxValue >= 10000 AND @MaxValue < 10000000 THEN 1000
					WHEN @MaxValue >= 10000000 AND @MaxValue < 10000000000 THEN 1000000
					WHEN @MaxValue >= 10000000000 THEN 1000000000
				END
		ELSE
			SET	@Scale = 
				CASE @ScaleSelector_loc
					WHEN 0 THEN 1
					WHEN 1 THEN 1000
					WHEN 2 THEN 1000000
				END


		-- 		
		UPDATE @commonP
		SET
			[TimeMoney] = [TimeMoney] / @Scale,
			[ExpenseMoney] = [ExpenseMoney] / @Scale,
			[Advance] = [Advance] / @Scale,
			[Total] = [Total] / @Scale



		--********************************************************
		-- .
		IF @CountSelector_loc = 0
			SELECT
				[Order],
				[Employee].[FileAs] AS [Manager],
				[Company].[FileAs] AS [Company],
				[Project].[Number] AS [ProjectNumber],
				[PeriodStart],
				[PeriodEnd],
				[Time],
				[TimeMoney],
				[ExpenseMoney],
				[Advance],
				[Total],
				@SortSelector_loc AS [SortSelector]
			FROM
				@commonP 
				LEFT JOIN [Employee]
					ON [Employee].[ID] = [@commonP].[ManagerID]
				LEFT JOIN [Company]
					ON [Company].[ID] = [@commonP].[CompanyID]
				LEFT JOIN [Project]
					ON [Project].[ID] = [@commonP].[ProjectID]
			ORDER BY 
				[Employee].[FileAs],
				[Company],
				[Order]
		ELSE
		BEGIN
			DECLARE @commonPTopX TABLE
			(
				[Order] int,
				[ManagerID] TIdentifier,
				[CompanyID] TIdentifier,
				[ProjectID] TIdentifier,
				[PeriodStart] TDate,
				[PeriodEnd] TDate,
				[Time] TAmount,
				[TimeMoney] TAmount,
				[ExpenseMoney] TAmount,
				[Advance] TAmount,
				[Total] TAmount
			)

			--  @Count_loc 
			INSERT INTO @commonPTopX
			SELECT TOP (@Count_loc)
				*
			FROM
				@commonP
			WHERE
				[Order] = -1
			ORDER BY
				[Total] DESC

			--      
			INSERT INTO @commonPTopX
			SELECT 
				*
			FROM
				@commonP
			WHERE
				[Order] = 0
				AND
				[CompanyID] IN (SELECT [CompanyID] FROM @commonPTopX)
			
			
			--    
			INSERT INTO @commonPTopX
			SELECT
				1000,
				NULL,
				-1,
				NULL,
				NULL,
				NULL,
				SUM([Time]),
				SUM([TimeMoney]),
				SUM([ExpenseMoney]),
				SUM([Advance]),
				SUM([Total])			
			FROM
				@commonP
			WHERE
				[CompanyID] NOT IN (SELECT DISTINCT [CompanyID] FROM @commonPTopX)
				AND
				[Order] = -1

			--      @common
			INSERT INTO @commonPTopX
			SELECT
				*
			FROM
				@commonP
			WHERE
				[Order] = 1001



			SELECT 
				[Order],
				[Employee].[FileAs] AS [Manager],
				CASE WHEN [Order] = 1000
					THEN ''
					ELSE [Company].[FileAs] 
				END AS [Company],
				[Project].[Number] AS [ProjectNumber],
				[PeriodStart],
				[PeriodEnd],
				[Time],
				[TimeMoney],
				[ExpenseMoney],
				[Advance],
				[Total],
				@SortSelector_loc AS [SortSelector]
			FROM
				@commonPTopX 
				LEFT JOIN [Employee]
					ON [Employee].[ID] = [@commonPTopX].[ManagerID]
				LEFT JOIN [Company]
					ON [Company].[ID] = [@commonPTopX].[CompanyID]
				LEFT JOIN [Project]
					ON [Project].[ID] = [@commonPTopX].[ProjectID]
			ORDER BY 
				[Employee].[FileAs],
				[Company], 
				[Order]
				
				
			-- ,       10 
			IF @Count_loc < 11
			BEGIN
				INSERT INTO @Chart
				SELECT
					CASE WHEN [Order] = 1000
						THEN ''
						ELSE [Company].[FileAs] 
					END AS [Company],
					[Total]
				FROM
					@commonPTopX
					LEFT JOIN [Company]
						ON [Company].[ID] = [@commonPTopX].[CompanyID]
				WHERE
					[Order] IN (-1,1000)
			END				
		END		
	END


	--********************************************************
	--    	
	SELECT
		@ReportCurrency =
			CASE @CurrencySelector_loc
				WHEN 0 THEN 
					CASE @ScaleSelector_loc
						WHEN 0 THEN ' ' + @NationalCurrency
						WHEN 1 THEN ' . ' + @NationalCurrency
						WHEN 2 THEN ' . ' + @NationalCurrency
						WHEN 3 THEN
							CASE @Scale
								WHEN 1 THEN ' ' + @NationalCurrency
								WHEN 1000 THEN ' . ' + @NationalCurrency
								WHEN 1000000 THEN ' . ' + @NationalCurrency
								WHEN 1000000000 THEN ' . ' + @NationalCurrency
							END 
					END
				WHEN 1 THEN 
					CASE @ScaleSelector_loc
						WHEN 0 THEN ' ' + @BaseCurrency
						WHEN 1 THEN ' . ' + @BaseCurrency
						WHEN 2 THEN ' . ' + @BaseCurrency
						WHEN 3 THEN
							CASE @Scale
								WHEN 1 THEN ' ' + @BaseCurrency
								WHEN 1000 THEN ' . ' + @BaseCurrency
								WHEN 1000000 THEN ' . ' + @BaseCurrency
								WHEN 1000000000 THEN ' . ' + @BaseCurrency
							END 
					END				
			END

	
	--********************************************************
	--  
	SELECT * FROM @Chart


	SET @PeriodEnd_loc = DATEADD(DAY,-1,@PeriodEnd_loc)
	--********************************************************
	--  
		EXEC ReportFuturePeriodsRealizationHeader @OwnCompanyID_loc, @ReportCurrency, @CompanyManagerSelector_loc, @ManagerID_loc, @CostCenterSelector_loc, @CostCenterID_loc,
		@PeriodStartSelector_loc, @PeriodEndSelector_loc, @PeriodStart_loc, @PeriodEnd_loc, @CountSelector_loc, @Count_loc, @SortSelector_loc, @TypeSelector_loc

	RETURN 0

GO
GRANT EXEC ON ReportFuturePeriodsRealization TO PUBLIC
GO