IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportServiceRequest')
	BEGIN
		PRINT 'Dropping Procedure ReportServiceRequest'
		DROP  Procedure  ReportServiceRequest
	END

GO

PRINT 'Creating Procedure ReportServiceRequest'
GO

--------------------------------------------------------------------------------------
-- Copyright (c) 2003 by Avicom
--------------------------------------------------------------------------------------
-- $Archive: $
-- $Author: $
-- $Revision: $
-- $Modtime: $
-- $Date: $
--
-- Description: this stored procedure ...........
-- Input:	.........
-- Output:      0 - success, negative number - otherwise
--
-- Notes:       no
--
--
/* $Log: $ 
   $NoKeywords: $ 
--------------------------------------------------------------------------------------*/
CREATE Procedure ReportServiceRequest
	@ID TIdentifier
AS

	SET NOCOUNT ON
	DECLARE
		@ViewAllFinance TBool,
		@CurrentEmployeeID TIdentifier,
		@UserID			TIdentifier,
		@UserName		TName,
		@AdvancedTimeExpenses bit,
		@value TVariant
	
	SET @ViewAllFinance = dbo.adfHasAccessRight(dbo.adfGetRightID('ViewAllFinance'))	
	SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @UserID = dbo.adfGetCurrentUserID()
	
	EXEC adfParameterGet 'AdvancedTimeExpensesForm', 'Pjm5', @value OUTPUT
	SET @AdvancedTimeExpenses = ISNULL(CAST(@value AS BIT), 0)
		
	SELECT
		@UserName = [Trustee].[DisplayName]
	FROM
		[Trustee]
	WHERE
		[Trustee].[ID] = @UserID

	SELECT
		[ServiceRequest].[FileAs],
		[ServiceRequest].[Number],
		[ServiceRequest].[Date],
		[ServiceRequest].[StartDate],
		[ServiceRequest].[DueDate],
		[ServiceRequest].[State],
		[ServiceRequest].[Billable],
		[ServiceRequest].[VATIsIncluded],
		[Project].[FileAs] AS [ProjectName],
		[Project].[Number] AS [ProjectCode],
		[Employee].[FileAs] AS [EmployeeName],
		[Priority].[FileAs] AS [Priority],
		[Project].[ContractNumber] AS [ContractNumber],
		[SupplierCompany].[FileAs] AS [SupplierCompanyName],
		[CustomerCompany].[FileAs] AS [CustomerCompanyName],
		dbo.fnPjm5GetContactList([ServiceRequest].[ID], [ServiceRequest].[Class]) AS [Contacts],
		dbo.fnPjm5GetCategoryList([ServiceRequest].[ID], [ServiceRequest].[Class]) AS [Categories],
		dbo.fnPjm5GetAttachmentList([ServiceRequest].[ID], [ServiceRequest].[Class]) AS [AttachmentList],
		CASE 
			WHEN @ViewAllFinance = 1 OR
				[Project].[ProjectManagerEmployeeID] = @CurrentEmployeeID OR
				dbo.fnPjm5IsProjectApprover([Project].[ID], @CurrentEmployeeID) = 1 OR
				dbo.fnPjm7IsManagerAssistant([Project].[ID], @CurrentEmployeeID) = 1
				THEN 1
			ELSE 0
		END AS [Flag],
		@UserName AS [UserName],
		[ServiceRequest].[Description],
		CASE 
			WHEN (@ViewAllFinance = 1 OR
				dbo.fnPjm5HasFinansDataAccess([Project].[ID], @CurrentEmployeeID, NULL, NULL) = 1 OR
				dbo.fnPjm5IsProjectApprover([Project].[ID], @CurrentEmployeeID) = 1 OR
				dbo.fnPjm7IsManagerAssistant([Project].[ID], @CurrentEmployeeID) = 1) AND [Project].[ProjectTypeID] = 1
				THEN 1
			ELSE 0
		END AS [FinanceEnable],
		[Project].[CurrencyCode],
		[ServiceRequest].[ManHour],
		[ServiceRequest].[ActualManHour],
		[ServiceRequest].[ActualBudget],
		@AdvancedTimeExpenses AS [AdvancedTimeExpenses]		
	FROM
		[ServiceRequest]
		LEFT OUTER JOIN [Project]
			ON [ServiceRequest].[ProjectID] = [Project].[ID]
		LEFT OUTER JOIN [Employee]
			ON [ServiceRequest].[EmployeeID] = [Employee].[ID]
		LEFT OUTER JOIN [Priority]
			ON [ServiceRequest].[PriorityID] = [Priority].[ID]
		LEFT OUTER JOIN [Company] AS [CustomerCompany]
			ON [Project].[CustomerCompanyID] = [CustomerCompany].[ID]
		LEFT OUTER JOIN [Company] AS [SupplierCompany]
			ON [Project].[SupplierCompanyID] = [SupplierCompany].[ID]				
	WHERE
		[ServiceRequest].[ID] = @ID
		
	SELECT
		2 AS [Table],
		[ServiceAssignment].[FileAs],
		[ServiceRequestLine].[Description],
		[UOM].[Code] AS [UOM],
		[ServiceRequestLine].[RequestAmount],
		[ServiceRequestLine].[FactAmount],
		[ServiceRequestLine].[Price],
		NULL AS [Date]		
	FROM
		[ServiceRequestLine]
		INNER JOIN [ServiceAssignment]
			ON [ServiceRequestLine].[ServiceAssignmentID] = [ServiceAssignment].[ID]
		INNER JOIN [Service]
			ON [ServiceAssignment].[ServiceID] = [Service].[ID]	
		LEFT OUTER JOIN [UOM]
			ON [UOM].[ID] = [Service].[UOMID]
	WHERE	
		[ServiceRequestLine].[ServiceRequestID] = @ID
	UNION ALL
	SELECT
		1,
		NULL,
		[ServiceRequest].[Comments],
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	FROM
		[ServiceRequest]
	WHERE
		[ServiceRequest].[ID] = @ID	
	UNION ALL
	SELECT
		3,
		[Trustee].[DisplayName],
		[Note].[Text],
		NULL,
		NULL,
		NULL,
		NULL,
		[Note].[Created]		
	FROM
		[ServiceRequest]
		INNER JOIN [Note]
			ON [ServiceRequest].[ID] = [Note].[DocumentID] AND
			   [ServiceRequest].[Class] = [Note].[DocumentClass]
		INNER JOIN [Trustee]
			ON ([Note].[CreatedBy] = [Trustee].[ID])
	WHERE
		[ServiceRequest].[ID] = @ID 
	ORDER BY 
		5 ASC		


RETURN 0
GO

GRANT EXEC ON ReportServiceRequest TO PUBLIC

GO
