IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportWordStatementTotalCalc')
	BEGIN
		PRINT 'Dropping Procedure ReportWordStatementTotalCalc'
		DROP  Procedure  ReportWordStatementTotalCalc
	END

GO

PRINT 'Creating Procedure ReportWordStatementTotalCalc'
GO
--------------------------------------------------------------------------------------
-- Copyright (c) 2012 by Avicom
-- Author: Seregy Pelevin
-- Date: 01.10.2012
-- Description: This stored procedure calculates some total values for calculation.
-- This values is used in ReportWordStatementV2 to provide info about discounts
/******************************************************************************
**		Change History
******************************************************************************
**	04.12.12	SAP				    


*/
	

CREATE PROCEDURE ReportWordStatementTotalCalc
	@ID int,
	@Language TName = NULL,
	@TotalLinesWithoutDiscount TMoney output,
	@TotalLinesVATWithoutDiscount TMoney output,
	@TotalLinesWithVATWithoutDiscount TMoney output,
	@DiscountWithVATPC TMoney output,
	@DiscountWithVAT TMoney output
	
AS
	SET NOCOUNT ON
	
	DECLARE
		@timeClass TClass,
		@expenseClass TClass,
		@phoneClass TClass,
		@requestClass TClass,
		@calculationClass TClass,
		@DocumentCalculationClass TClass,
		@TotalAmount TAmount,
		
		@TotalLines TMoney,
		@TotalLinesVAT TMoney,
		@TotalWithoutDiscount TMoney,
		@TotalLinesWithVAT TMoney,
		
		@TotalAdvanceWithoutDiscountPC TMoney,
		@TotalAdvanceVATWithoutDiscountPC TMoney,
		@TotalAdvanceWithVATWithoutDiscountPC TMoney,

		@TotalAdvance TMoney,
		@TotalAdvanceVAT TMoney,
		@TotalAdvanceWithVAT TMoney,

		@Discount TMoney,
		@DiscountType TIdentifier,

		@VATRate TMoney,
		@RatesWithVAT TBool,		
		@VATIsIncluded TBool,
		@AdvanceDiscountPC TMoney,
		@AdvanceDiscountType TIdentifier,

		@Code TName,
		@GroupTimeEntry TBool,
		@ShowActivityTypes TBool,
		@ShowPrice TBool,
		@GroupingAdvanceCalculations TCount,
		@NationalCurrencyCode TCode3,
		@val TVariant,
		@ContactID TIdentifier,
		@CustomerID TIdentifier,
		@CompanyClass TClass,
		@EmployeeClass TClass,
		@Date DateTime,
		@EmployeeID TIdentifier,
		@Price TMoney,
		@Description nvarchar(max),

		@Flag int,
		@Rate decimal(18,4),
		@TypeID int,
		@EmployeeFileAs nvarchar(256),
		@ApplyVAT int

	SET @EmployeeClass = dbo.adfGetClassByName('Adf.Employee')
	SET @CompanyClass = dbo.adfGetClassByName('Adf.Company')
	SET @Code = CASE WHEN @Language = 'en' THEN 'Calculation; ' ELSE '; ' END
	SET @timeClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	SET	@expenseClass = dbo.adfGetClassByName('Pjm5.Expense')
	SET	@phoneClass = dbo.adfGetClassByName('Pjm5.PhoneCall')
	SET	@calculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET @requestClass = dbo.adfGetClassByName('Pjm5.ServiceRequest')
		
	EXEC adfParameterGet 'NationalCurrency', 'CommonBusiness', @val OUTPUT	
	SET @NationalCurrencyCode = CAST(@val AS varchar(3))

	SELECT
		@DocumentCalculationClass = [Calculation].[DocumentClass],
		@RatesWithVAT = [Calculation].[RatesWithVAT],
		@VATRate = [Calculation].[VATRate],
		@GroupTimeEntry = [Project].[GroupCalculationLines],
		@ShowActivityTypes = [Project].[ShowActivityTypes],
		@ShowPrice = [Project].[ShowPrice],
		@GroupingAdvanceCalculations = [Project].[GroupingAdvanceCalculations],
		@ContactID = [Project].[ResponsiblePersonEmployeeID],
		@CustomerID = [Project].[CustomerCompanyID],
		@Discount = [Discount],
		@DiscountType = [DiscountType],
		@ApplyVAT = [Project].[ApplyVAT]
	FROM
		[Calculation]
		INNER JOIN [Project]
			ON [Project].[ID] = [Calculation].[ProjectID]
	WHERE
		[Calculation].[ID] = @ID
		
	IF (@DocumentcalculationClass = @TimeClass AND @ShowActivityTypes = 0)
		SET @ShowActivityTypes = 0
	ELSE
		SET @ShowActivityTypes = 1

	IF (@DocumentcalculationClass = @TimeClass AND @ShowPrice = 1)
		SET @ShowPrice = 1
	ELSE
		SET @ShowPrice = 0
			
	SET @VATIsIncluded = 0
	IF @DocumentCalculationClass = @timeClass
		SET @VATIsIncluded = @RatesWithVAT
	
	DECLARE @t TABLE(
		[ID] int,
		[Date] datetime,
		[EmployeeID] int,
		[EmployeeFileAs] nvarchar(256),
		[DivisionID] int,
		[PositionID] int,
		[QualificationID] int,
		[TypeID] int,
		[Description] nvarchar(max),
		[Price] money,
		[Amount] decimal(18,4),
		[Total] money,
		[VAT] money
	)
	
	
	IF @DocumentCalculationClass = @timeClass
	BEGIN
		INSERT INTO @t
			([ID],
			[Date],
			[EmployeeID],
			[EmployeeFileAs],
			[DivisionID],
			[PositionID],
			[QualificationID], 
			[TypeID],
			[Description],
			[Amount],
			[Price],
			[Total],
			[VAT])
		SELECT
			[CalculationLine].[ID],
			[TimeEntry].[StartTime],
			[TimeEntry].[EmployeeID],
			[Employee].[FileAs],
			[Employee].[DivisionID],
			[Employee].[PositionID],
			[Assignment].[QualificationID],
			[TimeEntry].[ActivityTypeID],
			[CalculationLine].[Description],
			[CalculationLine].[Amount],
			[CalculationLine].[Price],
			[CalculationLine].[Amount] * [CalculationLine].[Price],
			[CalculationLine].[VAT]
		FROM
			[CalculationLine]
			INNER JOIN [TimeEntry]
				ON [TimeEntry].[Class] = [CalculationLine].[DocumentClass] AND
					[TimeEntry].[ID] = [CalculationLine].[DocumentID]
			INNER JOIN [Employee]	
				ON [Employee].[ID] = [TimeEntry].[EmployeeID]
			LEFT OUTER JOIN [Assignment]
				ON [TimeEntry].[AssignmentID] = [Assignment].[ID]
		WHERE 
			[CalculationLine].[CalculationID] = @ID
	END
	ELSE
	BEGIN
		INSERT INTO @t
		(
			[ID],
			[Date],
			[EmployeeID],
			[EmployeeFileAs],
			[DivisionID],
			[PositionID],
			[QualificationID], 
			[TypeID],
			[Description],
			[Price],
			[Amount],
			[Total],
			[VAT]
		)
		SELECT
			[pjm5vwCalculationLine].[ID],
			[pjm5vwCalculationLine].[Date], 
			[Employee].[ID],
			[Employee].[FileAs],
			[Employee].[DivisionID],
			[Employee].[PositionID],
			[pjm5vwCalculationLine].[QualificationID],
			[pjm5vwCalculationLine].[TypeID],
			[pjm5vwCalculationLine].[Description],
			CASE 
				WHEN [pjm5vwCalculationLine].[VATIsIncluded] = 1 AND [pjm5vwCalculationLine].[Amount] != 0
					THEN [pjm5vwCalculationLine].[Price] - [pjm5vwCalculationLine].[VAT] / [pjm5vwCalculationLine].[Amount]
				ELSE [pjm5vwCalculationLine].[Price]
			END AS [Price],
			[pjm5vwCalculationLine].[Amount],
			CASE 
				WHEN [pjm5vwCalculationLine].[VATIsIncluded] = 1
					THEN ([pjm5vwCalculationLine].[Amount] * [pjm5vwCalculationLine].[Price] - [pjm5vwCalculationLine].[VAT])
				ELSE [pjm5vwCalculationLine].[Amount] * [pjm5vwCalculationLine].[Price]
			END AS [Total],
			[pjm5vwCalculationLine].[VAT]
		FROM
			[pjm5vwCalculationLine]
			INNER JOIN [Employee]	
				ON [Employee].[ID] = [pjm5vwCalculationLine].[EmployeeID]
		WHERE 
			[pjm5vwCalculationLine].[CalculationID] = @ID
	END
	
	--      
	SELECT
		@TotalWithoutDiscount = SUM([Total]),
		@TotalLinesWithoutDiscount = SUM([Total] - [VAT] * @VATIsIncluded),
		@TotalLinesVATWithoutDiscount = 
			SUM([Total] + [VAT] * (1 - @VATIsIncluded)) - SUM([Total] - [VAT] * @VATIsIncluded)
	FROM @t

	SET @TotalWithoutDiscount = ISNULL(@TotalWithoutDiscount, 0)
	SET @TotalLinesWithoutDiscount = ISNULL(@TotalLinesWithoutDiscount, 0)
	SET @TotalLinesVATWithoutDiscount = ISNULL(@TotalLinesVATWithoutDiscount, 0)
	
	SELECT
		--     
		@TotalLines = CASE
			WHEN [DiscountType] = 3 AND @TotalWithoutDiscount <= [Discount]
				THEN @TotalWithoutDiscount - @TotalLinesVATWithoutDiscount * @VATIsIncluded
			WHEN [DiscountType] = 3 AND @TotalWithoutDiscount > [Discount]
				THEN [Discount] - ([Discount] / (1 + [VATRate])) * @VATIsIncluded * [VATRate]
			WHEN [DiscountType] = 0
				THEN @TotalWithoutDiscount - @TotalLinesVATWithoutDiscount * @VATIsIncluded
			ELSE
				@TotalWithoutDiscount + (CASE
					WHEN [DiscountType] = 1 
						THEN -1
					ELSE 1 END) * CASE 
						WHEN [DiscountView] = 0
							THEN @TotalWithoutDiscount * [DiscountPercent]
						ELSE dbo.fnPjm6GetCalculationDiscount([ID], 0) END
				- ((@TotalWithoutDiscount + (CASE
					WHEN [DiscountType] = 1 
						THEN -1
					ELSE 1 END) * CASE 
						WHEN [DiscountView] = 0
							THEN @TotalWithoutDiscount * [DiscountPercent]
						ELSE dbo.fnPjm6GetCalculationDiscount([ID], 0) END) / (1 + [VATRate])) * @VATIsIncluded * [VATRate]
			END,
		--     
		@TotalLinesVAT = CASE
			WHEN [DiscountType] = 3 AND @TotalWithoutDiscount <= [Discount]
				THEN @TotalLinesVATWithoutDiscount
			WHEN [DiscountType] = 3 AND @TotalWithoutDiscount > [Discount]
				THEN [Discount] * [VATRate] / (1 + @VATIsIncluded * [VATRate])
			WHEN [DiscountType] = 0
				THEN @TotalLinesVATWithoutDiscount
			ELSE
				(@TotalWithoutDiscount + (CASE
					WHEN [DiscountType] = 1 
						THEN -1
					ELSE 1 END) * CASE 
						WHEN [DiscountView] = 0
							THEN @TotalWithoutDiscount * [DiscountPercent]
						ELSE dbo.fnPjm6GetCalculationDiscount([ID], 0) END) 
				* [VATRate] / (1 + @VATIsIncluded * [VATRate])
			END
	FROM
		[Calculation]
	WHERE
		[Calculation].[ID] = @ID

	SELECT @TotalAmount = ROUND(SUM([Amount]), 2) FROM @t

	SET @TotalLines = ISNULL(@TotalLines, 0)
	SET @TotalLinesVAT = ISNULL(@TotalLinesVAT, 0)
	SET @TotalAmount = ISNULL(@TotalAmount, 0)

	IF @TotalLines < 0
		SET @TotalLines = 0
	IF @TotalLinesVAT < 0
		SET @TotalLinesVAT = 0
	SET @TotalAmount = ISNULL(@TotalAmount, 0)

	DECLARE @AdvanceFlag TBool
	IF EXISTS(SELECT * FROM [CalculationAdvance] WHERE [CalculationID] = @ID)
		SET @AdvanceFlag = 1
	ELSE
		SET @AdvanceFlag = 0
		
	--  /     
	SELECT 
		@AdvanceDiscountPC = SUM(
			(CASE 
				WHEN [DiscountType] = 0
					THEN 0
				WHEN [DiscountType] = 2
					THEN 1
				ELSE -1
			END) * 
				CASE
					WHEN [DiscountType] != 3 
						THEN CASE
							WHEN [DiscountType] = 1 
								THEN CASE 
										WHEN [TotalWithoutDiscountPC] > [DiscountPC]
											THEN [DiscountPC]
										ELSE 0
									END
							ELSE [DiscountPC] END
					ELSE 
						CASE 
							WHEN [TotalWithoutDiscountPC] > [DiscountPC]
								THEN [TotalWithoutDiscountPC] - [DiscountPC]
							ELSE 0
						END	
				END),
		@TotalAdvanceWithoutDiscountPC = SUM([TotalWithoutDiscountPC]),
		@TotalAdvanceWithVATWithoutDiscountPC  = SUM([TotalWithVATWithoutDiscountPC]),
		@TotalAdvance = SUM([TotalPC]),
		@TotalAdvanceWithVAT = SUM([TotalWithVATPC])
	FROM
		[Calculation]
		INNER JOIN [CalculationAdvance]
			ON [CalculationAdvance].[AdvanceCalculationID] = [Calculation].[ID]
	WHERE 
		[CalculationID] = @ID

	SET @TotalAdvanceVAT = @TotalAdvanceWithVAT - @TotalAdvance
	SET @TotalAdvanceVAT = ISNULL(@TotalAdvanceVAT, 0)
	SET @TotalAdvance = ISNULL(@TotalAdvance, 0)
	SET @TotalAdvanceWithVAT = ISNULL(@TotalAdvanceWithVAT, 0)
	SET @AdvanceDiscountPC = ISNULL(@AdvanceDiscountPC, 0)
	SET @TotalAdvanceWithoutDiscountPC = ISNULL(@TotalAdvanceWithoutDiscountPC, 0)
	SET	@TotalAdvanceWithVATWithoutDiscountPC  = ISNULL(@TotalAdvanceWithVATWithoutDiscountPC, 0)
	SET @TotalAdvanceVATWithoutDiscountPC = @TotalAdvanceWithVATWithoutDiscountPC - @TotalAdvanceWithoutDiscountPC

	SET @AdvanceDiscountType = CASE 
			WHEN @AdvanceDiscountPC > 0 THEN 2
			WHEN @AdvanceDiscountPC < 0 THEN 1
			ELSE 0
		END
	SET @AdvanceDiscountPC = ABS(@AdvanceDiscountPC)
	SET @Discount = ISNULL(@Discount, 0)

	IF (@VATIsIncluded = 0 AND @TotalLinesWithoutDiscount < @Discount AND @DiscountType = 3)
		OR (@VATIsIncluded = 1 AND (@TotalLinesWithoutDiscount + @TotalLinesVATWithoutDiscount) < @Discount AND @DiscountType = 3)
	BEGIN
		SET @DiscountType = 0
	END

	SET @DiscountWithVATPC = 
	CASE 
		WHEN @ApplyVAT/*@VATIsIncluded*/ = 1 
			THEN dbo.fnPjm6GetCalculationDiscount(@ID, 1) * (1 + @VATRate)
		ELSE dbo.fnPjm6GetCalculationDiscount(@ID, 1)
	END
		
		
	SET @TotalLines = ROUND(@TotalLines, 2)
	SET @TotalLinesVAT = ROUND(@TotalLinesVAT, 2)
	SET @TotalLinesWithVAT = ROUND(@TotalLines + @TotalLinesVAT, 2)
	SET @TotalLinesWithVATWithoutDiscount = ROUND(@TotalLinesWithoutDiscount + @TotalLinesVATWithoutDiscount, 2)
	IF @ApplyVAT/*@VATIsIncluded*/ = 1
	BEGIN
		SELECT @DiscountWithVAT =  dbo.fnPjm6GetCalculationDiscount(@ID, 0) * (1 + [Calculation].[VATRate])
		FROM [Calculation]
	END
		ELSE SELECT @DiscountWithVAT = dbo.fnPjm6GetCalculationDiscount(@ID, 0)
			
	RETURN 0
	GO
	
	GRANT EXEC ON ReportWordStatementTotalCalc TO PUBLIC
	GO