IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trServiceRequestLineUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trServiceRequestLineUpdate'
		DROP  Trigger pjm5trServiceRequestLineUpdate
	END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trServiceRequestLineMoneyUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trServiceRequestLineMoneyUpdate'
		DROP  Trigger pjm5trServiceRequestLineMoneyUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trServiceRequestLineUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:				Description:
**		--------		--------				-------------------------------------------
**    
*******************************************************************************/

PRINT 'Creating Trigger pjm5trServiceRequestLineMoneyUpdate'
GO
CREATE Trigger pjm5trServiceRequestLineMoneyUpdate 
ON [ServiceRequestLine] FOR INSERT, UPDATE, DELETE
AS
BEGIN
	IF @@ROWCOUNT = 0
		RETURN

	DECLARE
		@VATRate TPercent,
		@value TVariant		
	
	EXEC adfParameterGet @Name='VATRate', @Group='CommonBusiness', @Value = @value OUTPUT
	SET @VATRate = ISNULL(CAST(@value as decimal(18,4)), 0)

	DECLARE @totals TABLE ([ID] int, [Total] money, [TotalWithVAT] money)
	INSERT INTO @totals ([ID], [Total], [TotalWithVAT])
	SELECT
		ISNULL([inserted].[ServiceRequestID], [deleted].[ServiceRequestID]),
		ISNULL(SUM(ROUND(
				CASE [ServiceRequest].[VATIsIncluded]
					WHEN 0 THEN ISNULL(([inserted].[FactAmount] * [inserted].[Price]), 0.00) - ISNULL(([deleted].[FactAmount] * [deleted].[Price]), 0.00)
					ELSE ISNULL(ROUND(([inserted].[FactAmount] * [inserted].[Price] / (1 + @VATRate)), 2), 0.00) - ISNULL(ROUND(([deleted].[FactAmount] * [deleted].[Price] / (1 + @VATRate)), 2), 0.00)
				END, 2)), 0),		
		ISNULL(SUM(ROUND(
				CASE [ServiceRequest].[VATIsIncluded]
					WHEN 0 THEN ISNULL(ROUND(([inserted].[FactAmount] * [inserted].[Price] * (1 + @VATRate)), 2), 0.00) - ISNULL(ROUND(([deleted].[FactAmount] * [deleted].[Price] * (1 + @VATRate)), 2), 0.00)
					ELSE ISNULL(([inserted].[FactAmount] * [inserted].[Price]), 0.00) - ISNULL(([deleted].[FactAmount] * [deleted].[Price]), 0.00)
				END, 2)), 0)
	FROM
		[inserted]
		FULL OUTER JOIN [deleted]
			ON [inserted].[ID] = [deleted].[ID]
		INNER JOIN [ServiceRequest]
			ON 	[ServiceRequest].[ID] = ISNULL([inserted].[ServiceRequestID], [deleted].[ServiceRequestID])
	GROUP BY
		ISNULL([inserted].[ServiceRequestID], [deleted].[ServiceRequestID])

	UPDATE
		[ServiceRequest]
	SET
		[Total] = ISNULL([ServiceRequest].[Total], 0) + [Total].[Total],	
		[TotalWithVAT] = ISNULL([ServiceRequest].[TotalWithVAT], 0) +  + [Total].[TotalWithVAT],
		[Modified] = DEFAULT,
		[ModifiedBy] = DEFAULT
	FROM
		[ServiceRequest]
		INNER JOIN @totals AS [Total]
			ON 	[ServiceRequest].[ID] = [Total].[ID]	
					
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
END

GO
 