IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5StatementChangeState]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5StatementChangeState'
		DROP PROCEDURE [dbo].[pjm5StatementChangeState]
	END
GO

PRINT 'Creating procedure pjm5StatementChangeState'
GO

/******************************************************************************
**		File: pjm5StatementChangeState.sql
**		Name: pjm5StatementChangeState
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: vladislav.dremin
**		Date: 01/21/2005 15:47:39
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**		14.05.10	SYV				  pjm6DismissEventsOnObject
**									 pjm7AllDocumentReminderDismiss   
**		24.08.10	SYV				  pjm6DismissEventsOnObject .. 
**									       
**		16.06.11	SPV				   
*******************************************************************************/
CREATE PROCEDURE pjm5StatementChangeState
	@ID TIdentifier,
	@ToState TState
AS
	DECLARE @FromState TState,
			@TransitionTemplateID TIdentifier,
			@RightID TIdentifier,
			@Class TClass,
			@ProjectID TIdentifier,
			@PaymentValue TMoney,
			@PaymentVAT TMoney,
			@Date TDate,
			@rv int
	
	SET NOCOUNT ON
	SELECT
		@Class = [Statement].[Class],
		@FromState = [Statement].[State],
		@ProjectID = [Statement].[ProjectID],
		@Date = [Statement].[Date]
	FROM
		[Statement]
	WHERE
		[Statement].[ID] = @ID
	
	IF @ToState = 2
	BEGIN
		SELECT
			@PaymentValue = [Statement].[TotalWithVAT] - ISNULL(SUM([StatementAdvanceInvoice].[StatementPaymentValue]), 0),
			@PaymentVAT = [Statement].[VAT] - ISNULL(SUM([StatementAdvanceInvoice].[StatementPaymentVAT]), 0)
		FROM
			[Statement]
			LEFT OUTER JOIN [StatementAdvanceInvoice]
				ON [StatementAdvanceInvoice].[StatementID] = [Statement].[ID]
		WHERE
			[Statement].[ID] = @ID
		GROUP BY
			[Statement].[ID],
			[Statement].[TotalWithVAT],
			[Statement].[VAT]
	END
		
	IF @FromState = @ToState
	BEGIN
		RAISERROR('#invalidargument', 18, -1)
		RETURN 1
	END
	
	SELECT
		@TransitionTemplateID = [ID],
		@RightID = [RightID]
	FROM
		[TransitionTemplate]
	WHERE
		[TransitionTemplate].[FromState] = @FromState AND [TransitionTemplate].[ToState] = @ToState
	AND
		[TransitionTemplate].[Class] = @Class
		
	IF @TransitionTemplateID IS NULL
	BEGIN
		RAISERROR('#invalidoperation', 18, -1)
		RETURN 1
	END
	
	SET @TransitionTemplateID = dbo.adfCheckTransitionRights(@TransitionTemplateID, @RightID, dbo.adfGetCurrentUserID())
	
	IF @TransitionTemplateID IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
	
	DECLARE @EmployeeID TIdentifier
	SET @EmployeeID = dbo.adfGetCurrentEmployeeID()
	IF dbo.adfHasAccessRight(dbo.adfGetRightID('ManageAllStatements')) = 0 AND
		dbo.fnPjm5HasFinansDataAccess(@ProjectID, @EmployeeID, NULL, NULL) = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END	
	
	IF @PaymentValue <= -0.01 OR @PaymentVAT <= -0.01
	BEGIN
		RAISERROR('#wrongprepaymentvalue', 18, -1)
		RETURN 1
	END
	
	BEGIN TRANSACTION
	IF @FromState = 1 AND @ToState = 2
	BEGIN
		EXEC @rv = pjm6StatementCreateRecurrence @ID
		IF @rv != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
		UPDATE [Document]
		SET [RecurrencePatternID] = NULL
		WHERE [Class] = @Class AND [ID] = @ID
	END	

	UPDATE
		[Statement]
	SET
		[Statement].[State] = @ToState,
		[Statement].[Modified] = DEFAULT,
		[Statement].[ModifiedBy] = DEFAULT
	WHERE
		[Statement].[ID] = @ID
	AND
		[Statement].[State] = @FromState

	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END

	IF (@ToState = 3 OR @ToState = 4)
	BEGIN			
		EXEC @rv = pjm7AllDocumentReminderDismiss @Class, @ID
		IF @rv != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN @rv
		END
	END

	COMMIT TRANSACTION
	RETURN 0
GO

GRANT EXECUTE ON pjm5StatementChangeState TO PUBLIC
GO
