IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'[dbo].[pjm5TimeEntryCreateRecurrence]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
	BEGIN
		PRINT 'Dropping procedure pjm5TimeEntryCreateRecurrence'
		DROP PROCEDURE [dbo].[pjm5TimeEntryCreateRecurrence]
	END
GO

PRINT 'Creating procedure pjm5TimeEntryCreateRecurrence'
GO

/******************************************************************************
**		File: pjm5TimeEntryCreateRecurrence.sql
**		Name: pjm5TimeEntryCreateRecurrence
**		Desc: this procedure automatically generated by Avicom.Adf.Generator
**
**              
**		Return values:
** 
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**
**		Auth: pavel.boruroev
**		Date: 01/20/2005 16:11:01
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:			Description:
**		--------	--------		-------------------------------------------
**    
*******************************************************************************/
CREATE PROCEDURE [pjm5TimeEntryCreateRecurrence]
	@ID TIdentifier		OUTPUT,
	@FileAs TName,
	@ActivityTypeID TIdentifier,
	@EmployeeID TIdentifier,
	@ProjectID TIdentifier,
	@TaskID TIdentifier,
	@StartTime TDateTime,
	@Duration TCount,
	@Billable TBool,
	@Overtime TBool,
	@Color TColor,
	@Comments TNotes,
	@DocumentID TIdentifier,
	@DocumentClass TClass,
	@RecurrenceID TIdentifier
AS
	DECLARE @State TState,
			@RoundedDuration TCount,
			@ProjectState TState,
			@Rate TMoney,
			@ActivityTypeIsMandatory bit,
			@CurrencyCode TCode3,
			@VATRate TPercent,
			@rv int,
			@value TVariant,
			@CurrentEmployeeID TIdentifier,
			@NewID TIdentifier,
			@DivisionID TIdentifier
	
	SET NOCOUNT ON
	SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	IF @EmployeeID IS NULL
		SET @EmployeeID = @CurrentEmployeeID
	
	IF @ProjectID IS NULL
	BEGIN
		RAISERROR('#ProjectIsNotDefined', 18, -1)
		RETURN 1
	END
	
	IF @EmployeeID IS NULL
	BEGIN
		RAISERROR('#assignmentnotfound', 18, -1)
		RETURN 1
	END

	IF dbo.fnPjm5AllowCreateProjectEntry(@ProjectID, @EmployeeID) = 0
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END

	IF @Duration IS NULL OR @Duration < 0
	BEGIN
		RAISERROR('#DurationIsInvalid', 18, -1)
		RETURN 1
	END

	SELECT
		@CurrencyCode = [CurrencyCode]
	FROM
		[Project]
	WHERE 
		[ID] = @ProjectID
		
	SET @VATRate = dbo.fnPjm5GetVATRate()
	
	DECLARE	@AssignmentID TIdentifier
	SELECT @AssignmentID = dbo.fnPjm5GetProjectAssignment(@ProjectID, @EmployeeID)
	IF @AssignmentID IS NULL
	BEGIN
		RAISERROR('#assignmentnotfound', 18, -1)
		RETURN 1
	END
	
	SET @State = dbo.adfwfGetInitialState('6719512d-42ee-4848-a57e-f76500e67ee7')
	IF @State IS NULL
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		RETURN 1
	END
	
	SELECT
		@ProjectState = [State],
		@RoundedDuration = dbo.fnPjm5RoundedDuration(@Duration, [RoundingInterval], [RoundingMode])
	FROM
		[Project]
	WHERE
		[ID] = @ProjectID

	IF @ProjectState != 2
	BEGIN
		RAISERROR('#ProjectIsNotActive', 18, -1)
		RETURN 1
	END

	IF @TaskID IS NOT NULL 
	BEGIN
		IF NOT EXISTS (SELECT * FROM [Task] WHERE [ID] = @TaskID AND [ProjectID] = @ProjectID)
		BEGIN
			RAISERROR('#TaskFromAnotherProject', 18, -1)
			RETURN 1
		END
		IF NOT EXISTS (SELECT * FROM [Task] WHERE [ID] = @TaskID AND [State] = 2)
		BEGIN
			RAISERROR('#TaskIsDraft', 18, -1)
			RETURN 1
		END
		IF dbo.fnPjm5IsTaskAssigned(@TaskID, @EmployeeID) = 0
		BEGIN
			RAISERROR('#taskassignmentnotfound', 18, -1)
			RETURN 1
		END
	END

	SET @Rate = dbo.fnPjm5GetEmployeeRate(@EmployeeID, @ProjectID, @Overtime, CASE @DocumentClass WHEN dbo.adfGetClassByName('Pjm5.ServiceAssignment') THEN @DocumentID ELSE NULL END, 0)	
	DECLARE @var sql_variant

	EXEC adfParameterGet 'ActivityTypeIsMandatory', 'Pjm5', @var OUTPUT
	IF @var IS NULL
		SET @ActivityTypeIsMandatory = 1
	ELSE
		SET @ActivityTypeIsMandatory = CAST(@var as BIT)
	
	IF @ActivityTypeID IS NULL AND @ActivityTypeIsMandatory = 1
	BEGIN
		RAISERROR('#activitytypecannotbenull', 18, -1)
		RETURN 1
	END
	
	--     " "
	DECLARE
		@AppSyncCalendars	TBool,
		@SyncTimeEntry		TBool,
		@LocalNowDateTime	TDate

	EXEC adfParameterGet @Name='ExchangeSyncCalendars', @Group='Pjm6', @Value = @value OUTPUT
	SET @AppSyncCalendars = CAST(@value as bit)
	SET @SyncTimeEntry = 0
	SET @LocalNowDateTime = dbo.fnPjm7GetLocalDate()

	IF @AppSyncCalendars = 1 AND @StartTime > @LocalNowDateTime
	BEGIN
		DECLARE
			@UserSyncCalendars	bit,
			@TimeEntryUserID	TIdentifier

		SELECT @TimeEntryUserID = [UserID] FROM [Employee] WHERE [ID] = @EmployeeID 	

		SELECT @value = [Value] FROM [UserParameter] WHERE [Name] = 'SyncExchangeCalendar' AND [UserID] = @TimeEntryUserID
		SET @UserSyncCalendars = ISNULL(CAST(@value AS bit), 0)

		IF @UserSyncCalendars = 1
		BEGIN
			DECLARE
				@UserSyncEntriesInFuture	int

			SELECT @value = [Value] FROM [UserParameter] WHERE [Name] = 'SyncExchangeCalendarEntriesInFuture' AND [UserID] = @TimeEntryUserID
			SET @UserSyncEntriesInFuture = ISNULL(CAST(@value AS int), 1)

			SET @SyncTimeEntry = 
				(CASE
					WHEN @UserSyncEntriesInFuture = 0 THEN 1
					WHEN @UserSyncEntriesInFuture = 1 THEN 0
					WHEN @UserSyncEntriesInFuture = 2  THEN dbo.fnPjm6ExchangeSyncActivityTypeEntry(@ActivityTypeID, @TimeEntryUserID)
				END)
		END
	END


	SELECT @DivisionID = [DivisionID] FROM [Employee] WHERE [ID] = @EmployeeID

	BEGIN TRANSACTION

	INSERT INTO [TimeEntry]
		([Flag], [State], [FileAs], [ActivityTypeID], [EmployeeID], [AssignmentID], [ProjectID], [TaskID], 
		[StartTime], [Duration], [RoundedDuration], [Billable], 
		[Overtime], [Rate], [CurrencyCode], [VATRate], [Color],
		[Comments], [DocumentClass], [DocumentID], [DivisionID])
	VALUES
		(0, @State, @FileAs, @ActivityTypeID, @EmployeeID, @AssignmentID, @ProjectID, @TaskID, 
		@StartTime, @Duration, @RoundedDuration, @Billable, 
		 @Overtime, @Rate, @CurrencyCode, @VATRate, @Color,
		 @Comments, @DocumentClass, @DocumentID, @DivisionID)
			
	IF @@ERROR != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN 1
	END
	SET @NewID = SCOPE_IDENTITY()

	DECLARE @TimeEntryClass TClass
	SET @TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')	
	
	EXEC @rv = adfDocumentUDFCopy @TimeEntryClass, @ID, @NewID
	IF @rv != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN @rv
	END

	EXEC @rv = adfDocumentCategoryCopy @TimeEntryClass, @ID, @NewID
	IF @rv != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN @rv
	END

	EXEC @rv = adfDocumentContactCopy @TimeEntryClass, @ID, @NewID
	IF @rv != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN @rv
	END

	SET @ID = @NewID
	IF @RecurrenceID IS NOT NULL
	BEGIN
		UPDATE
			[Document]
		SET
			[RecurrencePatternID] = @RecurrenceID
		WHERE
			[Class] = @TimeEntryClass AND [ID] = @ID
		IF @@ERROR != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN 1
		END
	END
	
	DECLARE
		@ReminderState TState,
		@ReminderInterval TTimeSpan,
		@CurrentUserID TIdentifier
	SELECT
		@CurrentUserID = dbo.adfGetCurrentUserID(),
		@ReminderState = dbo.fnPjm5GetUserInitialReminderState(@TimeEntryClass, 'ES', @CurrentUserID),
		@ReminderInterval = dbo.fnPjm5GetReminderInterval(@TimeEntryClass, 'ES', @CurrentUserID)
		
	EXEC @rv = pjm7TimeEntryReminderCreate @ID,
		@StartTime, @ReminderState,	@ReminderInterval
	IF @rv != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN @rv
	END
	
	IF @SyncTimeEntry = 1
	BEGIN
		EXEC @rv = pjm7ExchangeSyncStateUpdate @ID, @TimeEntryClass, @SyncTimeEntry, @EmployeeID
		IF @rv != 0
		BEGIN
			ROLLBACK TRANSACTION
			RETURN @rv
		END

	END
	
	COMMIT TRANSACTION

	RETURN 0
GO

GRANT EXECUTE ON pjm5TimeEntryCreateRecurrence TO PUBLIC
GO
  