IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'pjm5trTimeEntryUpdate')
	BEGIN
		PRINT 'Dropping Trigger pjm5trTimeEntryUpdate'
		DROP  Trigger pjm5trTimeEntryUpdate
	END
GO

/******************************************************************************
**		File: 
**		Name: pjm5trTimeEntryUpdate
**		Desc: 
**
**		This template can be customized:
**              
**
**		Auth: 
**		Date: 
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-------------------------------------------
**      25.10.10    BVY         Edited [Duration] on [Duration] - 1 in #dayboundsexceeded
**		26.10.10	SYV			    MasterUpdateTimeEntry
**		01.12.10	SYV			 EntryAmount    1, 2, 5
**		19.03.13	SAP			   Amount   EntryAmount
**		01.04.13	SOV			. -   
**		25.07.13	SAP			       . 
**		30.06.14	SAP			   .     " "  
*******************************************************************************/

PRINT 'Creating Trigger pjm5trTimeEntryUpdate'
GO
CREATE TRIGGER pjm5trTimeEntryUpdate 
ON [TimeEntry] FOR UPDATE
AS
BEGIN
    IF @@rowcount = 0
       RETURN
       
    SET NOCOUNT ON
	DECLARE
		@TimeEntryClass TClass,
	    @CurrentEmployeeID TIdentifier,

		@CalculationClass TClass,
		@StatementClass TClass,
		@InvoiceClass TClass,
		@FactureClass TClass,

		@working TCount,
		@value TVariant
			



    SET @CurrentEmployeeID = dbo.adfGetCurrentEmployeeID()
	SET @TimeEntryClass = dbo.adfGetClassByName('Pjm5.TimeEntry')
	SET @CalculationClass = dbo.adfGetClassByName('Pjm5.Calculation')
	SET @StatementClass = dbo.adfGetClassByName('Pjm5.Statement')
	SET @InvoiceClass = dbo.adfGetClassByName('Pjm5.Invoice')
	SET @FactureClass = dbo.adfGetClassByName('Pjm5.Facture')

	EXEC adfParameterGet @Name='WorkTime', @Group='Pjm5', @Value = @value OUTPUT
	SET @working = CAST(@value as int)
   
    IF EXISTS (
		SELECT *
		FROM [inserted]
			INNER JOIN [deleted]
				ON [inserted].[ID] = [deleted].[ID]
		WHERE
			[deleted].[State] = 3 AND [inserted].[State] = 3 AND -- approved entry
			(
				dbo.fnPjm5IsProjectManager([inserted].[ProjectID], @CurrentEmployeeID) != 1 AND
				dbo.fnPjm5IsProjectApprover([inserted].[ProjectID], @CurrentEmployeeID) != 1 AND
				dbo.fnPjm7IsManagerAssistant([inserted].[ProjectID], @CurrentEmployeeID) != 1 AND
				(
				dbo.adfHasAccessRightByName('MasterUpdateTimeEntry') != 1 --     
				AND
				dbo.adfHasAccessRightByName('RemoveBillableFlag') != 1 --   
				)
			)
	)
	BEGIN
		RAISERROR('#accessdenied', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END

	IF EXISTS(SELECT * FROM inserted WHERE 
		CONVERT(datetime, CONVERT(varchar(8), DATEADD(minute, [Duration] - 1, [StartTime]), 112), 112) >
		CONVERT(datetime, CONVERT(varchar(8), [StartTime], 112), 112)) 
	BEGIN
		RAISERROR('#dayboundsexceeded', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	

	
	IF EXISTS(
		SELECT * 
		FROM 
			inserted 
			INNER JOIN deleted
				ON deleted.[ID] = inserted.[ID]
			INNER JOIN [Attachment]
				ON ([Attachment].[DocumentClass] = @TimeEntryClass AND
					[Attachment].[DocumentID] = inserted.[ID] AND 
					[Attachment].[AttachmentTypeID] = 3 AND
					[Attachment].[LinkDocumentClass] IN (@CalculationClass, @StatementClass, @InvoiceClass, @FactureClass))
		WHERE
			deleted.[State] IN (3, 6) AND
			inserted.[State] NOT IN (3, 6)
		)
	BEGIN
		RAISERROR('#cannotchangeentrystate', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END
	
	IF EXISTS (
		SELECT * 
		FROM 
			inserted 
			INNER JOIN deleted
				ON deleted.[ID] = inserted.[ID]
		WHERE
			(deleted.[State] <> 3 AND inserted.[State] = 3)
			AND (inserted.[AssignmentID] IS NULL)
		)
	BEGIN
		RAISERROR('#cannotchangeentrystate', 18, -1)
		ROLLBACK TRANSACTION
		RETURN
	END


	UPDATE
		[EntryAmount]
	SET		
		[Price] = ISNULL([inserted].[Rate], 0)
	FROM
		[EntryAmount]
		INNER JOIN [inserted]
			ON [inserted].[ID] = [EntryAmount].[ID]
		INNER JOIN [deleted]
			ON [inserted].[ID] = [deleted].[ID] AND 
				([inserted].[State] = [deleted].[State] -- 
					OR
				[inserted].[State] IN (1, 2, 5))
	WHERE
		[inserted].[State] != 6

	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END

		-- 
		UPDATE [EntryAmount]
		SET [Price] = [vwTimeEntry].[OriginalRate]
		FROM
			[EntryAmount]
			INNER JOIN [inserted]
				ON [inserted].[ID] = [EntryAmount].[ID] 
			INNER JOIN [vwTimeEntry]
				ON [EntryAmount].[ID] = [vwTimeEntry].[ID]	
			INNER JOIN [deleted]
				ON [inserted].[ID] = [deleted].[ID]	


	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END

	--,        
	--IF EXISTS(
	--	SELECT *
	--	FROM
	--		[inserted]
	--		INNER JOIN [deleted]
	--			ON [deleted].[ID] = [inserted].[ID] 
	--	WHERE
	--		([deleted].[Duration] != [inserted].[Duration] OR
	--		[deleted].[EmployeeID] != [inserted].[EmployeeID] OR
	--		[deleted].[StartTime] != [inserted].[StartTime]
	--		) 
	--	)
	--BEGIN

	INSERT INTO [EmployeeCalendar]
		([EmployeeID], [Date], [Working], [Planned], [Actual], [Approved])
	SELECT DISTINCT
		[EmployeeID], 
		CONVERT(datetime, CONVERT(varchar(8), [StartTime], 112), 112) as [Date],
		CASE 
			WHEN dbo.fnPjm5IsWorkingDay([StartTime]) = 1 THEN @working
			ELSE 0
		END,
		0,
		0,
		0
	FROM 
		[inserted]
	WHERE
		NOT EXISTS(SELECT * FROM [EmployeeCalendar] 
			WHERE [EmployeeID] = [inserted].[EmployeeID] AND [Date] = CONVERT(datetime, CONVERT(varchar(8), [inserted].[StartTime], 112), 112))

	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END

	UPDATE 
		[EmployeeCalendar]
	SET
		[Actual] = [Actual] - [D].[Duration]
	FROM
		EmployeeCalendar INNER JOIN (
			SELECT 
				[EmployeeID], 
				CONVERT(datetime, CONVERT(varchar(8), [StartTime], 112), 112) as [Date],
				SUM([Duration]) as [Duration]
			FROM 
				[deleted]
			WHERE
				[deleted].[State] != 4
			GROUP BY
				[EmployeeID], 
				CONVERT(datetime, CONVERT(varchar(8), [StartTime], 112), 112)) as [D]
			ON ([EmployeeCalendar].[EmployeeID] = [D].[EmployeeID] AND
				[EmployeeCalendar].[Date] = [D].[Date])
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	
	UPDATE [EmployeeCalendar]
	SET
		[Actual] = [Actual] + [I].[Duration]
	FROM
		EmployeeCalendar INNER JOIN (
			SELECT 
				[EmployeeID], 
				CONVERT(datetime, CONVERT(varchar(8), [StartTime], 112), 112) as [Date],
				SUM([Duration]) as [Duration]
			FROM 
				[inserted]
			WHERE
				inserted.[State] != 4
			GROUP BY
				[EmployeeID], 
				CONVERT(datetime, CONVERT(varchar(8), [StartTime], 112), 112)) as [I]
			ON ([EmployeeCalendar].[EmployeeID] = [I].[EmployeeID] AND
				[EmployeeCalendar].[Date] = [I].[Date])
	IF @@error != 0
	BEGIN
		ROLLBACK TRANSACTION
		RETURN
	END
	--END
END
GO

